/* Animation editor and training program. This program
   uses the mouse, but if you don't have a mouse, simply
   delete the mouse code and use the arrow keys.
*/

#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.
*/

#include "stdio.h"
#include "dos.h"
#include "conio.h"
#include "bios.h"
#include "string.h"
#include "ctype.h"
#include "stdlib.h"

#define IY        144
#define IX        240

#define XDIM        21   /* change these to change dimensions */
#define YDIM        27   /* of the symbol you want to use */

#define MAX_IMG      5   /* number of images in symbol */

#define XPAND        3   /* editor grid expansion ratio */

/* mouse motion macros */
#define NOT_MOVED 0
#define RIGHT     1
#define LEFT      2
#define UP        3
#define DOWN      4

#define MAX_POS 500  /* number of different positions that
                        can be recorded for any one symbol */

#define MAX_LINES  24
#define LINE_LEN   79

#define CHAR_HEIGHT 12  /* height of graphics character */
#define CHAR_WIDTH   8  /* width of graphics character */
 
#define BLANK      0 
#define BCK_GND    9
#define FORE_GND  15
#define HIGHLIGHT 13

char getpoint(int x, int y), getkey(void);
void mode(int mode_code), gotoxy(int x, int y);
void clr_eol(int x, int y), clrscr(void);
void display_grid(char i),  edit_image(char i);
void display_image(char i, int x, int y);
void init_symbol(char i);
void copyimage(int i, int j);
void train(void);
void getstartpos(void);
void putpoint(int x, int y, int color, int how);
int save_symbol(void), load_symbol(char *fname);
char menu(void);

void mouse_position(int *x, int *y);
void set_mouse_position(int x, int y);
void mouse_motion(int *deltax, int *deltay);
void cursor_on(void), cursor_off(void), mouse_reset(void);
void mouse_direction(int *deltax, int *deltay);
int rightb_pressed(void), leftb_pressed(void);
void mickey_ratio(int xratio, int yratio);

void record(int x, int y);
void animate(void);
mouse_menu(char opts[][20], char *codes, 
           int startx, int starty);

struct symbol_type {
  unsigned char image[MAX_IMG][XDIM][YDIM];  /* symbol */
  int x[MAX_POS], y[MAX_POS]; /* X,Y coordinates */
  int last_pos;  /* current coordinate position */
  int delay;  /* delay factor */
  int num_images; /* number of separate images in symbol */
} symbol;

void display_symbol(struct symbol_type symbol, int img, 
                    int pos);

char new_sym = 1;  /* when new symbol is being edited, this
                      is 1; otherwise, 0 */

char far *egabase;

void main(int argc, char *argv[])
{ 
  char choice, str[80];
  register int i;

  mode(16);  /* ega mode 16 - use another if you like */

#ifdef TURBOC
  egabase = (char far *) MK_FP(0xA000, 0000);
#endif
#ifdef MICROSOFTC
  egabase = (char far *) 0xA0000000;
#endif

  /* initialize the images */
  for(i=0; i<MAX_IMG; i++) init_symbol(i);

  mouse_reset(); /* reset the mouse */

  if(argc==2) {
    if(!load_symbol(argv[1])) {
      printf("cannot load file");
      exit(1);
    }
    new_sym = 0;
  }

  do {
    choice = menu();  /* get user's request */
    switch(choice) {
      case 'n': /* edit a new symbol */
         new_sym = 1; 
         for(i=0; i<MAX_IMG; i++) init_symbol(i);
        /* drop through to edit */
      case 'e':  /* edit a symbol */
        clr_eol(0, 0);
        gotoxy(0, 0);

        /* if new symbol, how many images does
            this symbol have? */
        if(new_sym) do {
          printf("How many images (1-%d)? ", MAX_IMG);
          gets(str);
          symbol.num_images = atoi(str);
        } while (symbol.num_images<1 ||
                 symbol.num_images>MAX_IMG);
        clrscr();

        /* create/edit each image in the symbol */
        for(i=0; i<symbol.num_images; i++) {
          edit_image(i);
          /* copy first image into second, etc., to 
             make slight changes easier */
          if(i+1<symbol.num_images && new_sym) 
             copyimage(i, i+1);
        }
        new_sym = 0; /* symbol now exists */

        /* request delay factor */
        do {
          gotoxy(0, 22);
          printf("enter delay factor (0-10): ");
          gets(str);
          symbol.delay = atoi(str);
        } while (symbol.delay<0 || symbol.delay>10);
        break;
      case 's': /* save the symbol */
        save_symbol();
        break;
      case 'l': /* load a symbol */
        if(load_symbol("")) new_sym = 0;
        break;
      case 't': /* train */
        train();
        break;
      case 'a': /* animate (after it is trained) */
        animate();
        break;
    }
  } while(choice!='q'); /* quit */
  mode(2);
}

/* Get user's selection. */
char menu(void)
{
  int i;
  char ch;

  char options[][20] = {
    "(E)dit", "(N)ew", "(S)save", "(L)oad",
    "(T)rain", "(A)nimate", "(Q)uit", ""
  };
  char ret_codes[] = "ensltaq";

  clr_eol(0, 0);
  gotoxy(0, 0);
  /* display options in menu */
  for(i=0; *options[i]; i++) {
    printf("%s", options[i]);
    if(*options[i+1]) printf(", ");
  }

  clr_eol(0, 1); 
  gotoxy(0, 1);
  printf("Enter letter of choice: ");
  cursor_on();
  ch = 26;
  set_mouse_position(0, 0);
  do {
    if(kbhit()) ch = getche();

    /* if left button is pressed, user is making
       a menu selection using the mouse
    */
    if(leftb_pressed()) 
      ch = mouse_menu(options, ret_codes, 0, 0);
  } while(!strchr(ret_codes, tolower(ch)));
  cursor_off();
  clrscr();

  return tolower(ch);
}

/* Process a mouse selection.   Call this function
   with a pointer to the options list, a pointer to
   the characters to be returned with each selection,
   and the X,Y location that the menu starts at.
*/
mouse_menu(char opts[][20], char *codes, 
           int startx, int starty)
{
  int x, y, i, start, end;

  mouse_position(&x, &y);
  
  /* if in correct row */
  if(y >= starty*CHAR_HEIGHT && 
     y <= starty * CHAR_HEIGHT + CHAR_HEIGHT) {

    /* see if on a menu option */
    start = startx; end = startx;
    for(i=0; *opts[i]; i++) {
      end = end + strlen(opts[i]) * CHAR_WIDTH;
      if(x>=start && x<=end) {
        return codes[i];
      }
      end += 2 * CHAR_WIDTH;  
      start = end;
    }
  }
  return -1;
}

/* Display the expanded symbol editor grid. */
void display_grid(char i)
{
  register int x, y;

  for(y = IY; y<(IY+YDIM); y++)
    for(x = IX; x<(IX+XDIM); x++) {
      if(symbol.image[i][x-IX][y-IY] == BLANK)
        putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND),
                 BCK_GND, 0);
      else 
        putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND),
                 symbol.image[i][x-IX][y-IY], 0);
    }
}

/* Edit an image.  This function displays a large
   grid to make editing easy.  It also displays the
   normal size version of the image being edited. */
void edit_image(char i)
{
  register int x, y;
  char ch, pen, temp;
  int x1, y1;

  clr_eol(0, 0);
  gotoxy(0, 0);
  printf("F1: pen down  F2: pen up  F3: erase  F4: done");
  gotoxy(0, 1);
  printf("left button toggles pen, right button erases, ");
  printf("both bottons terminate");

  x = IX; y = IY;  /* set x,y to upperleft corner */

  pen = BCK_GND;
  gotoxy(0, 23);
  printf("pen up  ");  /* tell user pen is up */

  display_image(i, 0, IY);
  display_grid(i); 
  clr_eol(0, 22);
  gotoxy(0, 22);
  printf("editing image: %c", i+'0'); 

  do {
    /* write pixel to grid */
    putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND), pen, 0);

    /* save pixel color in image, use
       blank background */
    if(pen==BCK_GND)
      symbol.image[i][x-IX][y-IY] = BLANK;
    else symbol.image[i][x-IX][y-IY] = FORE_GND;

    /* write pixel to image */
    putpoint(x-IX, y, symbol.image[i][x-IX][y-IY], 0);

    /* This code displays a dot that indicates the current
       position, waits for a keystroke and then replaces
       the original value. 
    */
    temp = getpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND));
    putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND),
             HIGHLIGHT, 0);

    /* wait for either mouse or keyboard input */ 
    ch = 0;
    x1 = y1 = NOT_MOVED;
    while(!x1 && !y1) {  /* while mouse has not moved */
      mouse_motion(&x1, &y1);  /* see if moved */

       /* if both left and right buttons pushed, terminate */
      if(leftb_pressed() && rightb_pressed()) {
        ch = 62; /* F4's scan code */
 
        /* wait for buttons to clear */
        while(leftb_pressed() || rightb_pressed()) ;
        break;
      }

      /* if left button pressed, pen up/down toggle */      
      if(leftb_pressed()) {
        if(pen==BCK_GND)
          ch = 59;  /* F1's scan code */
        else ch = 60;

        /* wait for user to release button */
        while(leftb_pressed()) ; 
        break;
      }

      /* if right button pressed, erase image */
      if(rightb_pressed()) {
        ch = 61; /* F3's scan code */

        /* wait for user to release button */
        while(rightb_pressed()) ; 
        break;
      }

      if(kbhit()) {  /* otherwise, process key input */
        ch = getkey();
        break;
      }
    }
    putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND), temp, 0); 

    /* process mouse motion */
    if(x1==LEFT) x --;
    else if(x1==RIGHT) x ++;
    if(y1==UP) y --;
    else if(y1==DOWN) y ++;
    else
    switch(ch) {
      case 75: /* left */
        x--;
        break;
      case 77: /* right */
        x++;
        break;
      case 72: /* up */
        y--;
        break;
      case 80: /* down */
        y++;
        break;
      case 71: /* up left */
        x--; y--;
        break;
      case 73: /* up right */
        x++; y--;
        break;
      case 79: /* down left*/
        x--; y++;
        break;
      case 81: /* down right */
        x++; y++;
        break;
      case 59: /* F1 - pen down */
        pen = FORE_GND;  
        gotoxy(0, 23);
        printf("pen down");
        break;
      case 60: /* F2 - pen up */
        pen = BCK_GND;  
        gotoxy(0, 23);
        printf("pen up  ");
        break;
      case 61: /* F3 - erase */
        init_symbol(i);
        display_image(i, 0, IY);
        display_grid(i);
        break;
    }
    if(x < IX) x++;  if(x > IX+XDIM-1) x--;
    if(y < IY) y++;  if(y > IY+YDIM-1) y--;

  } while(ch != 62); /* F4 - quit */
  clr_eol(0, 22);
  clr_eol(0, 23);
  /* give symbol default beginning coordinates */
  symbol.x[0] = 0;
  symbol.y[0] = 0;
  
  return;
}

/* Copy image from, to. */
void copyimage(int i, int j)
{
  register int x, y;

  for(x=0; x<XDIM; x++)
    for(y=0; y<YDIM; y++)
      symbol.image[j][x][y] = symbol.image[i][x][y];
}

/* Displays the image at specified X,Y. */
void display_image(char i, int startx, int starty)
{
  register int x, y;

  for(y = starty; y<starty+YDIM; y++)
    for(x = startx; x<startx+XDIM; x++)
      putpoint(x, y, symbol.image[i][x-startx][y-starty], 0);
}

/* Train the symbol using the mouse or arrow keys.
   Each movement you make will be recorded for later
   playback.
*/
void train(void)
{
  int x, y, x1, y1;
  char ch;
  unsigned k;

  symbol.last_pos = 0;

  getstartpos(); /* get starting position */

  x = symbol.x[0];  /* load starting coordinates */
  y = symbol.y[0];

  clrscr();
  gotoxy(0, 23);
  printf("Use arrow keys or press left button to use mouse");
  gotoxy(0, 24);
  printf("F1 or right button to stop");
  
  k = 0;
  
  for(;;) {
    ch = 0;
    record(x, y);  /* record each position */

    display_symbol(symbol, k % symbol.num_images, 
                   symbol.last_pos); /* display image */

    x1 = y1 = NOT_MOVED;
    /* while mouse has not moved */
    while(!x1 && !y1 || !leftb_pressed()) { 
      mouse_motion(&x1, &y1);  /* see if moved */
      if(kbhit()) {  /* otherwise, process key input */
        ch = getkey();
        break;
      }
       
      /* stop when right button pressed */
      if(rightb_pressed()) {
        ch = 59; /* F1's scan code */
        break;
      }
    }

    display_symbol(symbol, k % symbol.num_images,
                   symbol.last_pos); /* erase image */

    /* process mouse motion */
    if(x1==LEFT) x -= 5;
    else if(x1==RIGHT) x += 5;
    if(y1==UP) y -= 5;
    else if(y1==DOWN) y += 5;

    if(ch)  /* if key input, process */
      switch(ch) {
        case 75: /* left */
          x -=5;
          break;
        case 77: /* right */
          x +=5;
          break;
        case 72: /* up */
          y -= 5;
          break;
        case 80: /* down */
          y +=5;
          break;
        case 71: /* left-up */
          y -= 5;  x -= 5;
          break;
        case 73: /* right-up */
          y -= 5;  x += 5;
          break;
        case 79: /* left-down */
          y += 5;  x -= 5;
          break;
        case 81: /* right-down */
          y += 5;  x += 5;
          break;
        case 59: /* stop training on F1 */
          return;
      }

    /* increase last_pos */
    if(symbol.last_pos<MAX_POS-1) symbol.last_pos++;
    else putch(7); /* ring bell if out of space */

    k++;
  }
}

/* Set starting position for animation. */
void getstartpos(void)
{
  int x, y, x1, y1;
  char ch;
  unsigned k;

  clrscr();
  gotoxy(0, 23);
  printf("Use arrow keys or press left button to use mouse");
  gotoxy(0, 24);
  printf("press F1 or right button at starting position");

  symbol.x[0] = 0;  /* give symbol initial coordinates */
  symbol.y[0] = 0;
  x = 0; y = 0;

  k = 0;
  do {
    ch = 0;

    display_symbol(symbol, k % symbol.num_images,
                   symbol.last_pos); /* display image */

    x1 = y1 = NOT_MOVED;
    /* while mouse has not moved */
    while(!x1 && !y1 || !leftb_pressed()) {  
      mouse_motion(&x1, &y1);  /* see if moved */
      if(kbhit()) {  /* otherwise, process key input */
        ch = getkey();
        break;
      }

      /* stop when right button pressed */
      if(rightb_pressed()) {
        ch = 59; /* F1's scan code */
        while(rightb_pressed()); /* wait until button released */
        break;
      }
    }

    display_symbol(symbol, k % symbol.num_images,
                   symbol.last_pos); /* erase image */

    /* process mouse motion */
    if(x1==LEFT) x -= 5;
    else if(x1==RIGHT) x += 5;
    if(y1==UP) y -= 5;
    else if(y1==DOWN) y += 5;

    if(ch)  /* if key input, process */
      switch(ch) {
        case 75: /* left */
          x -=5;
          break;
        case 77: /* right */
          x +=5;
          break;
        case 72: /* up */
          y -= 5;
          break;
        case 80: /* down */
          y +=5;
          break;
        case 71: /* left-up */
          y -= 5;  x -= 5;
          break;
        case 73: /* right-up */
          y -= 5;  x += 5;
          break;
        case 79: /* left-down */
          y += 5;  x -= 5;
          break;
        case 81: /* right-down */
          y += 5;  x += 5;
          break;
        case 59:  /* F1: abort */ 
          return; 
      }

    k++;

    symbol.x[0] = x;  /* update starting position */
    symbol.y[0] = y;

  } while(ch!=59);  /* press F1 at starting position */
}

/* Record the motion entered by the user. */
void record(int x, int y)
{
  symbol.x[symbol.last_pos] = x;
  symbol.y[symbol.last_pos] = y;
}

/* Replay the animation sequence. */
void animate(void)
{
  register int i, j;
  int delay;

  /* for number of positions recorded */
  for(i=0; i<symbol.last_pos; i++ ) {
    /* display images */
    for(j=0; j<symbol.num_images; j++) {
      display_symbol(symbol, j, i);
      /* delay x 1000 */
      for(delay=0; delay<symbol.delay*1000; delay++) ;
      display_symbol(symbol, j, i);
      i++;
    }
  } 
}

/* Display a symbol at its X,Y location. */
void display_symbol(struct symbol_type symbol, 
                    int img, int pos)
{
  register int x, y;
  int xtemp, ytemp;

  ytemp = symbol.y[pos];
  xtemp = symbol.x[pos];

  for(y = ytemp; y<ytemp+YDIM; y++)
    for(x = xtemp; x<xtemp+XDIM; x++)
      if(symbol.image[img][x-xtemp][y-ytemp]==FORE_GND) 
        putpoint(x, y, 
                 symbol.image[img][x-xtemp][y-ytemp], 0x18);
}

/* Initializes the image. */
void init_symbol(char i)
{
  register int x, y;

  for(x= 0; x<XDIM; x++)
    for(y=0; y<YDIM; y++)
      symbol.image[i][x][y] = BLANK;
}

/* Saves the symbol. */
save_symbol(void)
{
  FILE *fp;
  char fname[80];
  int result;

  gotoxy(0, 22);
  printf("save to: ");
  gets(fname);

  if((fp = fopen(fname, "wb"))==NULL) {
    printf("cannot open file");
    return 0;
  }

  fwrite(&symbol, sizeof symbol, 1, fp);

  if(ferror(fp)) result = 0;
  else result = 1;
  fclose(fp);
  clr_eol(0, 22);
  return result;
}

/* Loads the symbol. */
load_symbol(char *name)
{
  FILE *fp;
  char fname[80];
  int result;

  if(!*name) {
    clr_eol(0, 22);
    gotoxy(0, 22);
    printf("load from: ");
    gets(fname);
  }
  else strcpy(fname, name);

  if((fp = fopen(fname, "rb"))==NULL) {
    printf("cannot open file");
    return 0;
  }

  fread(&symbol, sizeof symbol, 1, fp);

  if(ferror(fp)) result = 0;
  else result = 1;
  fclose(fp);
  clr_eol(0, 22);
  return result;
}

/************************************************/
/* Mouse interface functions.                   */
/************************************************/

/* Turn off the mouse cursor. */
void cursor_off(void)
{
  union REGS r;

  r.x.ax = 2; /* remove the cursor */
  int86(0x33, &r, &r);
}

/* Turn on the mouse cursor. */
void cursor_on(void)
{
  union REGS r;

  r.x.ax = 1; /* show the cursor */
  int86(0x33, &r, &r);
}

/* Return true if right button is pressed,
   false otherwise. */
rightb_pressed(void)
{
  union REGS r;

  r.x.ax = 3; /* get position and button status */
  int86(0x33, &r, &r);
  return r.x.bx & 2;
}

/* Return true if left button is pressed,
   false otherwise. */
leftb_pressed(void)
{
  union REGS r;

  r.x.ax = 3; /* get position and button status */
  int86(0x33, &r, &r);
  return r.x.bx & 1;
}

/* Set mouse cursor coordinates. */
void set_mouse_position(int x, int y)
{
  union REGS r;

  r.x.ax = 4; /* set position */
  r.x.cx = x;
  r.x.dx = y;
  int86(0x33, &r, &r);
}  

/* Return mouse cursor coordinates. */
void mouse_position(int *x, int *y)
{
  union REGS r;

  r.x.ax = 3; /* get position and button status */
  int86(0x33, &r, &r);
  *x = r.x.cx;
  *y = r.x.dx;
}  

/* Return the direction of travel. */
void mouse_motion(int *deltax, int *deltay)
{
  union REGS r;

  r.x.ax = 11; /* get direction of motion */
  int86(0x33, &r, &r);

  if((int) r.x.cx>0) *deltax = RIGHT;
  else if((int) r.x.cx<0) *deltax = LEFT;
  else *deltax = NOT_MOVED;

  if((int) r.x.dx>0) *deltay = DOWN;
  else if((int) r.x.dx<0) *deltay = UP;
  else *deltay = NOT_MOVED;
}

/* Reset the mouse. */
void mouse_reset(void)
{
  union REGS r;

  r.x.ax = 0; /* reset the mouse */
  int86(0x33, &r, &r);
  if((int) r.x.ax != -1) {
    printf("mouse hardware or software not installed");
    exit(1);
  }
  if(r.x.bx != 2) {
   printf("two-button mouse required");
   exit(1);
  }
}

/*****************************************************
 Console and keyboard functions.
*****************************************************/  

/* Return the scan code of the key pressed. 
   This function is compatible with Turbo/Borland C/C++ and
   Microsoft C/C++ (be sure to #define the appropriate
   symbol).  If you use a different compiler, consult
   the text.
*/
char getkey(void)
{
  union key {
    int i;
    char ch[2];
  } k;

#ifdef TURBOC
  k.i = bioskey(0);
#endif
#ifdef MICROSOFTC
  k.i = _bios_keybrd(0);
#endif
  return k.ch[1];
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Send the cursor to x,y. */
void gotoxy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Clears to end of line. */
void clr_eol(int x, int y)
{
   for(; x<80; x++) {
     gotoxy(x, y);
     printf(" ");
   }
}

/* Returns the value at the specified pixel. */
char getpoint(int x, int y)
{
  union REGS r;

  r.h.ah = 13;
  r.x.dx = y;
  r.x.cx = x;
  r.h.bh = 0;
  int86(16, &r, &r);
  return r.h.al;
}

/* This version of putpoint will work for all video
   adapters but is incredibly slow!
*/
/*
void putpoint(int x, int y, int color, int how)
{
  union REGS r;

  if(how==0x18) color = color | 128;
  r.h.bh = 0;
  r.h.ah = 12;
  r.h.al = color;
  r.x.dx = y;
  r.x.cx = x;
  int86(16, &r, &r);
}
*/

/* Clear the screen. */
void clrscr(void)
{  
  union REGS r;

  r.h.ah = 6;
  r.h.al = 0;
  r.h.ch = 0;
  r.h.cl = 0;
  r.h.dh = MAX_LINES;
  r.h.dl = LINE_LEN;
  r.h.bh = 0;
  int86(0x10, &r, &r);
}


/* This function sets the specified pixel to the 
   specified color using an EGA/VGA video adapter.
   The value of how may be one of these:

   action        value

   overwrite       0
   XOR             0x18
   AND             8
   OR              0x10
   
*/
#define ENABLE 0x0F
#define INDEXREG 0x3CE
#define VALREG 0x3CF

#define OUTINDEX(index, val) outp((INDEXREG), (index));\
                             outp((VALREG), (val));

#define WIDTH 80L

/* These range values assume EGA mode 16 */
#define XMAX 639
#define YMAX 349
#define XMIN 0
#define YMIN 0

void putpoint(int x, int y, int color, int how)
{
  register unsigned char mask = 0x80;
  register char far *base;
  unsigned dummy;

  /* if you want range checking at this point, activate
     this line of code.  It will slow down the function
     however.
  */
  /*if(x<XMIN || x>XMAX || y<YMIN || y>YMAX) return;*/

  base = (char far *) (egabase 
         + ( (long) y * WIDTH + (long) x/8L ));

  mask >>= x % 8;

  /* This causes the memory READ necessary to
     load the EGA/VGA's internal registers.
  */
  dummy = *base;

  OUTINDEX(0, color);
  OUTINDEX(1, ENABLE);
  OUTINDEX(3, how);
  OUTINDEX(8, mask);

  *base = 1;

  OUTINDEX(0, 0);
  OUTINDEX(1, 0);
  OUTINDEX(3, 0); 
  OUTINDEX(8, 0xff);
}

