
/* Animation subsystem and demonstration program. */

#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.
*/

#include "stdio.h"
#include "dos.h"
#include "conio.h"
#include "bios.h"
#include "string.h"
#include "ctype.h"
#include "stdlib.h"

#define XDIM        21   /* change these to change dimensions */
#define YDIM        27   /* of the symbol you want to use */

#define MAX_SYM      5   /* max number of separate symbols */
#define MAX_IMG      5   /* number of images in symbol */

#define MAX_POS 500  /* number of different positions that
                        can be recorded for any one symbol */

#define MAX_LINES  24
#define LINE_LEN   79
 
#define BLANK      0 
#define BCK_GND    9
#define FORE_GND  15
#define HIGHLIGHT 13

void mode(int mode_code);
void clrscr(void);
void putpoint(int x, int y, int color, int how);
void animate_all(void), animate_one(int sym);
void animate_some(char *syms);
int load_symbols(void);


struct symbol_type {
  unsigned char image[MAX_IMG][XDIM][YDIM]; /* symbol */
  int x[MAX_POS], y[MAX_POS]; /* X, Y coordinates */
  int last_pos; /* last position */
  int delay;  /* delay factor */
  int num_images; /* number of images in symbol */
} symbol[MAX_SYM];

void display_symbol(struct symbol_type symbol, int img,
                    int pos);

char far *egabase;

/* array of filenames of animation files */
char files[][14] = {
  "dog.an",
  "bird.an",
  "cat.an",
  "tree.an",
  "worm.an",
  ""  /* terminate with a null name */
};

/* will hold number of symbols actually loaded */
int num_syms; 

char syms[] = {1, 1, 0, 1, 0};

void main()
{ 
  register int i;

  mode(16);  /* ega mode 16 - use another if you like */

#ifdef TURBOC
  egabase = (char far *) MK_FP(0xA000, 0000);
#endif
#ifdef MICROSOFTC
  egabase = (char far *) 0xA0000000;
#endif

  if(!load_symbols()) {
      printf("cannot load file");
      exit(1);
  }

  /* individually animate the symbols */
  for(i=0; i<num_syms; i++)
    animate_one(i);

  animate_all();  /* animate all symbols */

  animate_some(syms); /* animate some symbols */

  getch();
  mode(2);
}

/* Replay all animation sequences. */
void animate_all(void)
{
  register int i, j;
  int delay[MAX_SYM], count[MAX_SYM], max_pos[MAX_SYM];
  char on[MAX_SYM];
  char active;

  /* initialize counters */
  for(i=0; i<num_syms; i++) {
    count[i] = 0;
    delay[i] = symbol[i].delay;
    max_pos[i] = symbol[i].last_pos;
    on[i] = 0;
  }

  active = 1;
  do {
    /* turn on images */
    for(i=0; i<num_syms; i++) {
      if(count[i] < max_pos[i] && !on[i]) {
        display_symbol(symbol[i], 
           count[i] % symbol[i].num_images, count[i]);
        on[i] = 1;
      }
    }
    /* turn off images */
    for(i=0; i<num_syms; i++) {
      /* if delay active, don't turn off */
      if(count[i] < max_pos[i] && !delay[i]) {
        display_symbol(symbol[i],
           count[i] % symbol[i].num_images, count[i]);
        count[i]++;
        on[i] = 0;
      }

      /* update delay count */
      if(delay[i]==0) delay[i] = symbol[i].delay;
      if(delay[i]) delay[i]--;
    }
    active = 0;
    /* see if any symbols still active */
    for(j=0; j<num_syms; j++)
      if(count[j]<max_pos[j]) active = 1;
  } while(active); /* display while at least one active 
                      symbol */
}

/* Replay selected animation sequences. */
void animate_some(char *syms)
{
  register int i, j;
  int delay[MAX_SYM], count[MAX_SYM], max_pos[MAX_SYM];
  char on[MAX_SYM];
  char active;

  /* initialize counters */
  for(i=0; i<num_syms; i++) {
    count[i] = 0;
    delay[i] = symbol[i].delay;
    max_pos[i] = symbol[i].last_pos;
    on[i] = 0;
  }

  active = 1;
  do {
    /* turn on images */
    for(i=0; i<num_syms; i++) {
      if(!syms[i]) continue;  /* skip this symbol */
      if(count[i] < max_pos[i] && !on[i]) {
        display_symbol(symbol[i], 
           count[i] % symbol[i].num_images, count[i]);
        on[i] = 1;
      }
    }
    /* turn off images */
    for(i=0; i<num_syms; i++) {
      if(!syms[i]) continue;  /* skip this symbol */
      /* if delay active, don't turn off */
      if(count[i] < max_pos[i] && !delay[i]) {
        display_symbol(symbol[i],
           count[i] % symbol[i].num_images, count[i]);
        count[i]++;
        on[i] = 0;
      }

      /* update delay count */
      if(delay[i]==0) delay[i] = symbol[i].delay;
      if(delay[i]) delay[i]--;
    }
    active = 0;
    /* see if any symbols still active */
    for(j=0; j<num_syms; j++) {
      if(!syms[j]) continue;  /* skip this symbol */
      if(count[j]<max_pos[j]) active = 1;
    }
  } while(active); /* display while at least one active 
                      symbol */
}

/* Replay the animation sequence. */
void animate_one(int sym)
{
  register int i, j, t;
  int delay;

  t = symbol[sym].last_pos;
  for(i=0; i<t; i++ ) {
    for(j=0; j<symbol[sym].num_images; j++) {
      symbol[sym].last_pos = i;
      display_symbol(symbol[sym], j, i); /* turn on */

      for(delay=0; delay<symbol[sym].delay*1000; delay++) ;

      display_symbol(symbol[sym], j, i); /* turn off */
      i++;
    }
  } 
}

/* Display a symbol at its X,Y location. */
void display_symbol(struct symbol_type symbol, 
                    int img, int pos)
{
  register int x, y;
  int xtemp, ytemp;

  ytemp = symbol.y[pos];
  xtemp = symbol.x[pos];

  for(y = ytemp; y<ytemp+YDIM; y++)
    for(x = xtemp; x<xtemp+XDIM; x++)
      if(symbol.image[img][x-xtemp][y-ytemp]==FORE_GND) 
        putpoint(x, y, 
                 symbol.image[img][x-xtemp][y-ytemp], 0x18);
}

/* Loads the symbols. */
load_symbols(void)
{
  FILE *fp;
  int result, i;

  for(i=0; *files[i]; i++) {
    if((fp = fopen(files[i], "rb"))==NULL) {
      printf("cannot open file %s", files[i]);
      return 0;
    }

    fread(&symbol[i], sizeof symbol, 1, fp);

    if(ferror(fp)) result = 0;
    else result = 1;
    fclose(fp);
  }
  num_syms = i; /* set number of symbols */
  return result;
}


/*****************************************************
 Console functions.
*****************************************************/  

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* This version of putpoint will work for all video
   adapters but is incredibly slow!
*/
/*
void putpoint(int x, int y, int color, int how)
{
  union REGS r;

  if(how==0x18) color = color | 128;
  r.h.bh = 0;
  r.h.ah = 12;
  r.h.al = color;
  r.x.dx = y;
  r.x.cx = x;
  int86(16, &r, &r);
}
*/

/* Clear the screen. */
void clrscr(void)
{  
  union REGS r;

  r.h.ah = 6;
  r.h.al = 0;
  r.h.ch = 0;
  r.h.cl = 0;
  r.h.dh = MAX_LINES;
  r.h.dl = LINE_LEN;
  r.h.bh = 0;
  int86(0x10, &r, &r);
}


/* This function sets the specified pixel to the 
   specified color using an EGA/VGA video adapter.
   The value of how may be one of these:

   action        value

   overwrite       0
   XOR             0x18
   AND             8
   OR              0x10
   
*/
#define ENABLE 0x0F
#define INDEXREG 0x3CE
#define VALREG 0x3CF
#define OUTINDEX(index, val) outp((INDEXREG), (index));\
                             outp((VALREG), (val));

#define WIDTH 80L

/* These range values assume EGA mode 16 */
#define XMAX 639
#define YMAX 349
#define XMIN 0
#define YMIN 0

void putpoint(int x, int y, int color, int how)
{
  register unsigned char mask = 0x80;
  register char far *base;
  unsigned dummy;

  /* if you want range checking at this point, activate
     this line of code.  It will slow down the function
     however.
  */
  /*if(x<XMIN || x>XMAX || y<YMIN || y>YMAX) return;*/

  base = (char far *) (egabase 
         + ( (long) y * WIDTH + (long) x/8L ));

  mask >>= x % 8;

  /* This causes the memory READ necessary to
     load the EGA/VGA's internal registers.
  */
  dummy = *base;

  OUTINDEX(0, color);
  OUTINDEX(1, ENABLE);
  OUTINDEX(3, how);
  OUTINDEX(8, mask);

  *base = 1;

  OUTINDEX(0, 0);
  OUTINDEX(1, 0);
  OUTINDEX(3, 0); 
  OUTINDEX(8, 0xff);
}

