/* A bar graph generator program. */

#include "dos.h"
#include "stdio.h"
#include "conio.h"
#include "ctype.h"
#include "stdlib.h"
#include "math.h"
#include "string.h"

#define MAX_SETS 3
#define MAX_ENTRIES 38
#define MAX_LABELS 20
#define MAX_NAMES 20

#define XMAX 319
#define YMAX 199

void bargraph(double *data, int num, int offset,
              int min, int max, int width);
void min_max(double v[][MAX_ENTRIES], int entries, 
             int sets,  int *min, int *max);
void get_names(char n[][20], int num);
void get_labels(char l[][20], int num);
void grid(int min, int max);
void label(char str[][20], int num);
void hashlines(void), legend(char names[][20], int num);

void line(int startx, int starty, int endx, int endy, 
         int color);
void fill_box(int startx, int starty, int endx, int endy, 
         int color);
void mempoint(int x, int y, int color);
void palette(int pnum);
void mode(int mode_code);
void goto_xy(int x, int y), save_pic(void);
int get_offset(void), get_line_size(void);
int getmax(double *data, int num);
int getmin(double *data, int num);
int enter(double v[][MAX_ENTRIES], int *entries, int *sets);

double v[MAX_SETS][MAX_ENTRIES]; /* holds the data */
char names[MAX_NAMES][20];
char lab[MAX_LABELS][20];

main(void)
{
  int num_entries; 
  int num_sets;
  int min, max, i;
  int lines, offset;
  char save=0; /* save graph? */
  char ch;

  sin(0.1); /* to force Turbo C to link math library */

  /* read in the information */
  enter(v, &num_entries, &num_sets);

  /* find minimum and maximum values */
  min_max(v, num_entries, num_sets, &min, &max);

  /* get the names of what is being graphed */
  get_names(names, num_sets);

  /* get the labels for the graph */
  get_labels(lab, num_entries);

  /* get line thickness */
  lines = get_line_size();
  
  /* get the spacing between sets of data */
  offset = get_offset();

  /* save to disk file ? */
  printf("save the graph to disk file? (Y/N) ");
  ch = getche();
  if(tolower(ch)=='y') save = 1;

  mode(4); /* 320x200  graphics mode */
  palette(0);

  grid(min, max); /* display the grid */
  hashlines(); /* display dotted reference lines */
  label(lab, num_entries); /* label the graph */
  legend(names, num_sets); /* show the legend */

  /* plot the values */
  for(i=0; i<num_sets; i++) 
    bargraph(v[i], num_entries, i*offset, min, max, lines);

  if(save) save_pic();
  getch();
  mode(3);

  return 0;
}

/* Enter values to be plotted. */
enter(double v[][MAX_ENTRIES], /* data array */
      int *entries, /* number of entries in each set of data */
      int *sets) /* number of sets of data */
{
  int i, j, count, num;
 
  printf("How many sets of data? (1 to %d) ", MAX_SETS);
  scanf("%d", &count); fflush(stdin);
  if(count>MAX_SETS) count = MAX_SETS; /* prevent array overrun */
  *sets = count;

  printf("How many entries? (1 to %d) ", MAX_ENTRIES);
  scanf("%d", &num); fflush(stdin);
  if(num>MAX_ENTRIES) num = MAX_ENTRIES; /* prevent array overrun */
  *entries = num;

  j = 0;

  /* read in the values */
  while((j<count)) {
    printf("Data set %d\n", j+1);
    for(i=0; i<num; i++) {
      printf("%d: ", i+1);
      scanf("%lf", &v[j][i]);  fflush(stdin);
    }
    j++;
  }
  return count;
}

/* Input the names of the sets. */
void get_names(char n[][20], /* array for the names */
               int num)  /* number of sets */
{
  int i;

  for(i=0; i<num; i++) {
    printf("Enter name: ");
    gets(n[i]);
  }
}

/* Input the label of each entry. */
void get_labels(char l[][20], /* array for the labels */
                int num)  /* number of entries */
{
  int i;

  for(i=0; i<num; i++) {
    printf("Enter label: ");
    gets(l[i]);
  }
}

/* Input distance between bars in pixels */
get_offset(void)
{
  int i;

  printf("Enter distance between bars in pixels: ");
  scanf("%d", &i); fflush(stdin);
  return i;
}

/* Input bar thickness in pixels */
get_line_size(void)
{
  int i;

  printf("Enter thickness of bars in pixels: ");
  scanf("%d", &i); fflush(stdin);
  return i;
}

/* Draw the chart grid. */
void grid(int min, int max)
{
  goto_xy(0, 22); printf("%d", min);
  goto_xy(0, 0); printf("%d", max);
  line(10, 180, 300, 180, 1);
}

/* Display labels on the screen. */
void label(char str[][20], /* strings to display */
           int num) /* number of labels */
{
  int i, j, inc;

  inc = 38 / num;
  i = 2; /* initial horizontal starting point */
  for(j=0; j<num; j++) {
    goto_xy(i, 23);
    printf(str[j]);
    i += inc;
  }
}

/* Draw dotted reference lines across the screen. */
void hashlines(void)
{
  int i, j;

  for(i=10; i<300; i+=5) {
    for(j=10; j<180; j+=10)
      mempoint(i, j, 3); /* one point every 5 pixels */
  }
}

/* Show a legend. */
void legend(char names[][20],
            int num) /* number of names */
{
  int color=1, i, j;

  goto_xy(0, 24); /* legend goes on bottom line */
  j = 0;
  for(i=0; i<num; i++) {
    /* print the label */
    printf("%s    ", names[i]);

    /* Compute where colored box goes by
       converting cursor location into graphics
       coordinates.  Each character is 8 pixels wide in
       mode 4 graphics.
    */
    j += strlen(names[i])*8+4;
    fill_box(j, 192, j+12, 198, color);
    j += 28;  /* advance to next field */
    color++;
    if(color>3) color = 1;
  }
}    
   
/* Display a bar chart. */
void bargraph(double *data, /* array of data */
     int num, /* number of elements in array */
     int offset, /* determine exact screen position */
     int min, int max, /* min and max values to be plotted */
     int width) /* thickness of the lines */
{
  int y, t, incr;
  double norm_data, norm_ratio, spread;
  static int color=0;
  int tempwidth;
 
  /* always use a different color */
  color++;
  if(color>3) color = 1;

  /* determine normalization factor */
  spread = (double) max-min; 
  norm_ratio = 180/spread;  

  incr = 280/num; /* determine spacing between lines */
  tempwidth = width;
  for(t=0; t<num; ++t) {
    norm_data = data[t];

    /* adjust for negative values */
    norm_data = norm_data-(double) min;  

    norm_data *= norm_ratio; /* normalize */
    y = (int) norm_data; /* type conversion */
    do { 
      line(((t*incr)+20+offset+width), 179,  
        ((t*incr)+20+offset+width), 179-y, color);
      width--;
    } while (width);
    width = tempwidth;
  }
}

/* Find the smallest minimum and greatest maximum values 
   among all the sets of data.
*/
void min_max(double v[][MAX_ENTRIES], /* values */
     int entries, /* number of entries */
     int sets, /* number of sets */
     int *min, int *max) /* return minimum and maximum values */
{
  int i;
  int tmin, tmax;

  *min = *max = 0;

  for(i=0; i<sets; i++) {
      tmax = getmax(v[i], entries);
      tmin = getmin(v[i], entries);
      if(tmax > *max) *max = tmax;
      if(tmin < *min) *min = tmin;
  }
}

/* Returns the maximum value of the data. */
getmax(double *data, int num)
{
  int t, max;

  max = (int) data[0];
  for(t=1; t<num; ++t) 
    if(data[t]>max) max = (int) data[t];
  return max;
}

/* Returns the minimum value of the data. */
getmin(double *data, int num)
{
  int t, min;

  min = (int) data[0];
  for(t=1; t<num; ++t) 
    if(data[t]<min) min = (int) data[t];
  return min;
}

/* Draw a line in specified color
   using Bresenham's integer based algorithm.
*/
void line(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int t, distance;
  int x=0, y=0, delta_x, delta_y;
  int incx, incy;

  /* Compute the distances in both directions. */
  delta_x = endx-startx;
  delta_y = endy-starty;

  /* Compute the direction of the increment.
     An increment of 0 means either a vertical
     or horizontal line.
  */
  if(delta_x>0) incx = 1;
  else if(delta_x==0) incx = 0;
  else incx = -1;

  if(delta_y>0) incy = 1;
  else if(delta_y==0) incy = 0;
  else incy = -1;

  /* determine which distance is greater */
  delta_x = abs(delta_x);
  delta_y = abs(delta_y);
  if(delta_x>delta_y) distance = delta_x;
  else distance = delta_y;

  /* draw the line */
  for(t=0; t<=distance+1; t++) {
    mempoint(startx, starty, color);
    x += delta_x;
    y += delta_y;  
    if(x>distance) {
      x -= distance;
      startx += incx;
    }
    if(y>distance) {
      y -= distance;
      starty += incy;
    }
  }
}

/* Fill box with specified color. */
void fill_box(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int i, begin, end;

  begin = startx<endx ? startx : endx;
  end = startx>endx ? startx : endx;

  for(i=begin; i<=end;i++)
    line(i, starty, i, endy, color);
}

/* Write a point directly to mode 4 graphics RAM. */
void mempoint(int x, int y, /* coordinate of point */
              int color) /* color */
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;
  char xor; /* xor color in or overwrite */

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 0xFF3F;  /* 11111111 00111111 in binary */

  /* check range */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return;

  xor = color & 128; /* see if xor mode is set */
  color = color & 127; /* mask off high bit */
 
  /* set bit_mask and color bits to the right location */
  bit_position = x%4;
  color <<= 2*(3-bit_position);
  bit_mask.i >>= 2*bit_position;


  /* find the correct byte in screen memory */
  index = y*40 + (x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* write the color */
  if(!xor) { /* overwrite mode */
    t = *(ptr+index) & bit_mask.c[0];
    *(ptr+index) = t | color;
  }
  else { /* xor mode */
    t = *(ptr+index) | (char) 0;
    *(ptr+index) = t ^ color;
  }
}

/* Save the video graphics display. */
void save_pic(void)
{
  char fname[80];
  FILE *fp;
  register int i, j;
  int e=0;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  char far *temp;
  unsigned char buf[14][80]; /* hold the contents of screen */ 

  temp = ptr;
  /* save the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      buf[i][j] = *temp; /* even byte */
      buf[i][j+1] = *(temp+8152); /* odd byte */
      *temp = 0; *(temp+8152) = 0;  /* clear top of screen */
      temp++;
    }
      
  goto_xy(0, 0);
  printf("Filename: ");
  gets(fname);
  if(!(fp=fopen(fname, "wb"))) {
    goto_xy(0, 0);
    printf("cannot open file - press a key ");
    getch();
    e = 1;  /* flag error */
  }

  temp = ptr;
  /* restore the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      *temp = buf[i][j];
      *(temp+8152) = buf[i][j+1];
      temp++;
    }

 if(e) return; /* if file could not be opened, exit */

 /* save image to file */ 
 for(i=0; i<8152; i++) { 
    putc(*ptr, fp); /* even byte */
    putc(*(ptr+8152), fp); /* odd byte */
    ptr++;
  }
 
  fclose(fp);
}

/* Set the palette. */
void palette(int pnum)
{
  union REGS r;

  r.h.bh = 1;   /* code for palette select */
  r.h.bl = pnum;
  r.h.ah = 11;  /* set palette function */
  int86(0x10, &r, &r);
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Send the cursor to the specified X,Y position. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

