
/* Printer Graphics functions for Epson dot-matrix
   printers.
*/

#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.  If you use a different C compiler, refer
    to the text.
*/

#include "stdio.h"
#include "stdlib.h"
#include "bios.h"

void print(char ch);
void set_single_graphics(int cols);
void set_double_graphics(int cols);
void set_quad_graphics(int cols);
void reset(void);

main(void)
{
  int i;

  set_single_graphics(480);
  for(i=0; i<480; i++) print(255);

  /* draw two bars in double-density graphics */
  set_double_graphics(480);
  for(i=0; i<480; i++) print(255);
  print('\n');
  reset();
  set_double_graphics(480);
  for(i=0; i<480; i++) print(255);
  print('\n');
 
  set_quad_graphics(480);
  for(i=0; i<480; i++) print(255);
  print('\n');
  reset();
  return 0;
}

/* Select single-density graphics.  Specify number of 
   columns in cols. */
void set_single_graphics(int cols)
{
  union {
    unsigned char c[2];
    unsigned int i;
  } u;

  u.i = cols;

  print(27); print(65); print(8); /* 8/72 line spacing */
  print(27); print(75); print(u.c[0]); print(u.c[1]);
}

/* Select double-density graphics.  Specify
   number of columns in cols. */
void set_double_graphics(int cols)
{
  union {
    unsigned char c[2];
    unsigned int i;
  } u;

  u.i = cols;

  print(27); print(65); print(8); /* 8/72 line spacing */
  print(27); print(76); print(u.c[0]); print(u.c[1]);
}

/* Select quad-density graphics.  Specify
   number of columns in cols.
*/
void set_quad_graphics(int cols)
{
  union {
    unsigned char c[2];
    unsigned int i;
  } u;

  u.i = cols;

  print(27); print(65); print(8); /* 8/72 line spacing */
  print(27); print(90); print(u.c[0]); print(u.c[1]);
}

/* Reset the printer to defaults. */
void reset(void)
{
  print(27); print(64);
}

/* Send a character to the printer. */
void print(char ch)
{
#ifdef TURBOC
  biosprint(0, ch, 0);
#endif
#ifdef MICROSOFTC
  _bios_printer(0, 0, ch);
#endif
}
