////////////////////////////////////////////////////////////
// bucketb.h: Cache bucket base class definition.
// This has the data independent code for buckets.
// Copyright(c) 1993 Azarona Software. All rights reserved.
////////////////////////////////////////////////////////////
#ifndef H_BUCKETB
#define H_BUCKETB

class Cacheb; class Coptrb; class Fmgr; // Forward decl's

class Bucketb {
protected:
  long addr;      // Location of bucket's data in the file
  int refcnt;     // How many coptrs are pointing to this bucket
  char dirty;     // True when data doesn't match what's in file
  Bucketb *prev;  // Pointers to adjacent buckets in the cache
  Bucketb *next;  // in most-recently-acquired order.
  void MakeNull() { addr = 0; refcnt = 0; dirty = 0; }
public:
  Bucketb()           { } // Other classes initialize buckets
  void Lock()         { ++refcnt; }
  void Unlock()       { --refcnt; }
  int IsLocked()      { return refcnt; }
  void SetDirty()     { dirty = 1; }
  int IsNull()        { return addr == 0; }
  void Flush(Fmgr &f) { if (addr && dirty) Store(f); }
  virtual void Fetch(Fmgr &f) = 0; // Depends on bucket data
  virtual void Store(Fmgr &f) = 0; // Depends on bucket data
  friend class Cacheb;
  friend class Coptrb;
  friend void Report(Cacheb &c, int inspect=0);
};

#endif
