/*
|	FILE: CALL.CPP
|
|	Date: 30-SEPT-1991
|
|	This program is contains the source code for the program
|	CALC.CPP. It is code from:
|
|		Writing Your Own Application Language
|
|	Published by:
|		Wordware Publishing, Inc.
|		1506 Capital Avenue
|		Plano, Texas 75074
|
|	Written by:
|		Norman E. Smith, CDP
|		Copyright 1992 
|		All Rights Reserved
|
|	Right to use, copy, and modify this code is granted
|	for personal non-commercial use, provided that this 
|	copyright disclosure remains on ALL copies. Any other 
|	use, reproduction, or distribution requires the written 
|	consent of the author.
*/

#include <iostream.h>
#include <stdio.h>
#include <string.h>

#include "until.h"

/***********************+---------+
|			| startup |
|			+---------+
*/
void startup()
{
	cold();
	do_autoload();
	return;
}
/***********************+-----------+
|			| CreateNFA |
|			+-----------+
*/
NameField *CreateNFA(char *wordname)
{
	struct NameField *newword;

	newword            = (NameField*) new char [sizeof(NameField)];
	newword->len       = strlen(wordname);
	newword->name      = new char [newword->len];
	strcpy(newword->name,wordname);
	newword->smudge    = -1;
	newword->immediate = 0;
	return (newword);
}
/***********************+--------+
|			| Create |
|			+--------+
|  The word to create should be in pad. PAD is counted in the
|  Forth tradition. It must be converted to null terminated
|  for C to digest it properly. This is the generic "create a
|  word header"
|
*/
void Create()
{
	NameField  *nfa;
	DictHeader *entry;

	int  status;
	int  len;
	char string[80];

	entry = (DictHeader *) new char [sizeof(DictHeader)];

	len                   = pad[0];
	strncpy(string,&pad[1],len);
	string[len]           = NULL;

	status                = prim_find();	// search dict for this word
	if(status == 1){
		cout << string << ": Word redefined...\n";
	}

	nfa                   = CreateNFA(string);
	entry->NFA            = nfa;
	entry->NFA->smudge    = -1;
	entry->NFA->immediate = 0;
	entry->CFA            = do_variable;
	entry->PFA.lvalue     = 0;
	entry->LFA            = DP;
	DP                    = entry;
}
/***********************+---------------+*******************
|			| Compile_colon |
|			+---------------+
|  The word to create should be in pad. PAD is counted in the
|  Forth tradition. It must be converted to null terminated
|  for C to digest it properly.
|
|  Compile_colon takes a list of DictHeader pointers and inserts
|  it into the PFA. Create() is called first to allocate a skeleton
|  word. This word then fills in the CFA and PFA to make it a
|  high level Forth word.
|
|  This word essentually creates a high level Forth Word.
*/
void Compile_colon()
{
	void *ptr;
	int  len;
			/*
			| Create skeleton word header
			*/
	if(STATE == 1){
		cout << "ERROR->Already in Compile Mode.\n";
	}
	next_word();
	Create();
	STATE = 1;			// flag compile mode

	pfa_offset = 0;
	build_pfa_list();	// lookup words pfa list
	len   = popsp();	// lookup words pfa list
	len  += 4;
			/*
			| Allocate memory for the pfa list. Space for
			| a trailing null is included.
			*/
	ptr      = new DictHeader** [len*sizeof(DictHeader*)];
			/* Make a var for len*sizeof...
			| move pfa list into perm location
			*/
	memcpy(ptr,pfa_list,(len*sizeof(DictHeader*)));
			/*
			| Fill in the rest of the word.
			*/
	DP->NFA->smudge    = 0;
	DP->NFA->immediate = 0;
	DP->CFA            = do_colon;
	DP->PFA.Waddr      = (DictHeader**)ptr;
}
/***********************+------------------*****************
|			| Compile_variable |
|			+------------------+
|  The word to create should be in pad. PAD is counted in the
|  Forth tradition. It must be converted to null terminated
|  for C to digest it properly.
|
|  Compile_variable takes a list of DictHeader pointers and inserts
|  it into the PFA. Create() is called first to allocate a skeleton
|  word.
|
*/
void Compile_variable()
{
			/*
			| Create skeleton word header
			*/
	next_word();
	Create();
			/*
			| Fill in the rest of the word.
			*/
	DP->NFA->smudge    = 0;
	DP->NFA->immediate = 0;
	DP->CFA            = do_variable;
	DP->PFA.lvalue     = 0;
}
/***********************+------------------+*****************
|			| Compile_constant |
|			+------------------+
|  The word to create should be in pad. PAD is counted in the
|  Forth tradition. It must be converted to null terminated
|  for C to digest it properly.
|
|  Compile_variable takes a list of DictHeader pointers and inserts
|  it into the PFA. Create() is called first to allocate a skeleton
|  word. The value to initialize the constant with is on the stack.
|
*/
void Compile_constant()
{
			/*
			| Create skeleton word header
			*/
	next_word();
	Create();
			/*
			| Fill in the rest of the word.
			*/
	DP->NFA->smudge    = 0;
	DP->NFA->immediate = 0;
	DP->CFA            = do_constant;
	DP->PFA.lvalue     = popsp();
}
/***********************+----------------+
|			| build_pfa_list |
|			+----------------+
*/
void build_pfa_list()
{
	int  status;
	char *string;
	DictHeader *save_WA;

	save_WA    = WA;
		/*
		|  Get next token from input stream.
		*/
	next_word();
		/*
		|  Loop through tokens, compiling them into
		|  dictionary. Continue until state is 0;
		*/
	while(STATE){
		status = prim_find();	// search dict for this word
		switch(status){
		case -1:		// This is an immediate word
			(WA->CFA)();	// execute the immed word
			if(STATE == 0){
				goto FinishDef;
			}
			break;
		case 0:				// make sure can find the work
			pushsp((long)pad);
			number();		// Try to convert it to a #
			if(ABORT){		// Nope, an error
				string = &pad[1];
				cout << string << ": Word Not Found in Line\n";
				cout << tib;
				STATE  = 0;	// Reset Compile Mode
				abort_F();
				break;
			}else{			// Got a number
				literal();	// compile it into dict
			}			// BREAK left out on purpose!
		case 1:				// word found...
			pfa_list[pfa_offset++] = WA;
			break;
		default:
			cout << "Invalid prim_find return in build_pfa_list\n";
		}
		if(pfa_offset > PFA_SIZE){
			cout << "Definition too long. \n";
			abort_F();
			STATE = 0;
		}
		next_word();
	}
FinishDef:
	pfa_list[pfa_offset] = 0;		// pfa_list is null terminated
	WA                   = save_WA;		// restore original WA
	pushsp(pfa_offset);
}
/***********************+--------+
|			| set_WA |
|			+--------+
| This word is used only at setup to get a WA for use
| later by the compiler.
*/
DictHeader* set_WA(char* string)
{
	long len;
	long status;

	strcpy(&pad[1],string);
	len    = strlen(string);
	pad[0] = len;
	status = prim_find();
	if(!status){
		cout << string << ": Word Not Found.\n";
		return(0);
	}
	return(WA);
}
/***********************+-----------+
|			| immediate |
|			+-----------+
*/
void immediate()
{
	DP->NFA->immediate = 1;
}
/***********************+------+
|			| semi |
|			+------+
*/
void semi()
{
	if(!STATE){
		cout << "ERROR-> Not in Compile Mode.\n";
	}else{
		STATE = 0;			// Turn off compile mode
	}
}
/***********************+--------+
|			| parens |
|			+--------+
|	{ text }
|
| { starts a comment that is terminated with }.
*/
void parens()
{
	pushsp('}');
	word();
	drop();
}
/***********************+---------+
|			| literal |
|			+---------+
|	literal	( n --- )
*/
void literal()
{
	pfa_list[pfa_offset++] = LIT_WA;
	WA                     = (DictHeader*)popsp();
}
/***********************+------------+
|			| Compile_if |
|			+------------+
*/
void Compile_if()
{
	pfa_list[pfa_offset++] = ZERO_BRAN_WA;
	pushrs((long)&pfa_list[pfa_offset++]);
	WA = 0;
}
/***********************+---------------+
|			| Compile_endif |
|			+---------------+
| This word 
*/
void Compile_endif()
{
	DictHeader **branch;
	long       offset;

	r_from();
	offset  = popsp();
	branch  = (DictHeader**) offset;
	offset  = (long) &pfa_list[pfa_offset] - offset;
	offset  = offset / sizeof(DictHeader*);
	*branch = (DictHeader*) offset;
	WA      = 0;
}
/***********************+--------------+
|                       | Compile_else |
|			+--------------+
*/
void Compile_else()
{
	DictHeader **branch;
	long       offset;
		/*
		| Branch around the ELSE part and set up
		| code to jump to the then
		*/
	r_from();			// save addr of the if
	pfa_list[pfa_offset++] = BRANCH_WA;
	pushrs((long)&pfa_list[pfa_offset++]);
		/*
		| Note: This code is really THEN with the r_from() left
		|       out...
		*/
	offset  = popsp();
	branch  = (DictHeader**) offset;
	offset  = (long) &pfa_list[pfa_offset] - offset;
	offset  = offset / sizeof(DictHeader*);
	*branch = (DictHeader*) offset;
	WA = 0;
}
/***********************+------------+
|			| Compile_do |
|			+------------+
*/
void Compile_do()
{
	pfa_list[pfa_offset] = DO_WA;
	pushrs((long)&pfa_list[pfa_offset++]);
}
/***********************+--------------+
|			| Compile_loop |
|			+--------------+
*/
void Compile_loop()
{
	long       offset;

	pfa_list[pfa_offset++] = LOOP_WA;
	r_from();
	offset  = popsp();
	offset  = offset - (long) &pfa_list[pfa_offset];
	offset  = (offset / sizeof(DictHeader*)) + 1;
	pfa_list[pfa_offset++] = (DictHeader*) offset;
	WA      = 0;
}
/***********************+-------------------+
|			| Compile_dot_quote |
|			+-------------------+
*/
void Compile_dot_quote()
{
	char *const_string;
	long len;

	pushsp('\"');
	word();
	drop();
	pfa_list[pfa_offset++] = DOT_QUOTE_WA;
	len = pad[0];
	len+= 2;
	const_string = new char [len];
	memcpy(const_string,pad,len);
	*(const_string + len - 1) = NULL;
	pfa_list[pfa_offset++] = (DictHeader*) const_string;
	WA = 0;
}
/***********************+-----------+
|			| do_create |
|			+-----------+
|	create 	( --- )
|
|	Usage:
|		create junk
|
|	creates an empty word definition named junk
*/
void do_create()
{
	next_word();
	Create();
}
/***********************+-------------+
|			| Compile_i() |
|			+-------------+
*/
void Compile_i()
{
	pfa_list[pfa_offset++] = I_WA;
}
/***********************+-----------------+
|			| build_primitive |
|			+-----------------+
|
*/
void build_primitive(char* string,void (*prim)())
{
	int not_found;
		/*
		|  The first thing to do is convert a C null terminated
		|  string into Forth counted form.
		*/
	null_to_counted(string,pad);
		/*
		|  Search the dict and flag a match for duplicate
		|  word definitions. FALSE means the word was found.
		*/
	not_found = prim_find();
		/*
		|  Create a word header, then
		|  set the CFA to prim and
		|  clear the smudge flag
		*/
	if(not_found==0){
		Create();
		DP->CFA         = prim;
		DP->NFA->smudge = 0;
	}
}
/***********************+-------------+**********************
|			| build_iprim |
|			+-------------+
|
*/
void build_iprim(char* string,void (*prim)())
{
	int not_found;
		/*
		|  The first thing to do is convert a C null terminated
		|  string into Forth counted form.
		*/
	null_to_counted(string,pad);
		/*
		|  Search the dict and flag a match for duplicate
		|  word definitions. FALSE means the word was found.
		*/
	not_found = prim_find();
		/*
		|  Create a word header, then
		|  set the CFA to prim and
		|  clear the smudge flag
		*/
	if(not_found==0){
		Create();
		DP->CFA            = prim;
		DP->NFA->smudge    = 0;
		DP->NFA->immediate = -1;
	}
}
