/*
    clip.c

    Clipboard support

    This does copy and paste operations on the block of data
    addressed by pFileData and dwFileLength.

*/

#include "global.h"

//
// Set the state of the edit menu items
//

void InitClipMenu(HMENU hmenuPopup)
{
    //
    // See if there is anything we have that we might want the
    // user to be able to copy to the clipboard
    //

    EnableMenuItem(hmenuPopup, IDM_COPY, MF_ENABLED);
}

//
// Create a CF_TEXT Clipboard object from our data
//

static HANDLE CreateCFTEXTObject(LPSTR lpData, DWORD dwLength)
{
    HANDLE hData;
    LPSTR pData = NULL;

    //
    // Allocate the memory block
    //

    hData = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,
                        dwLength);
    if (!hData) {
        Message(0,"Out of memory");
        return NULL;
    }

    pData = GlobalLock(hData);
    _fmemcpy(pData, lpData, LOWORD(dwLength));
    GlobalUnlock(hData);
    return hData;
}

//
// Process a clipboard command
//

void ClipCommand(HWND hWnd, WPARAM wParam)
{
    HANDLE hData;
    LPSTR pData = NULL;

    switch (wParam) {
    case IDM_COPY:

        pData = CreateTextImage();
        hData = CreateCFTEXTObject(pData, lstrlen(pData));
        DeleteTextImage(pData);
        if (!hData) {
            Message(0, "Unable to create Clipboard object");
        } else {
            OpenClipboard(hWnd);
            EmptyClipboard();
            SetClipboardData(CF_TEXT, hData);
            CloseClipboard();
        }
        break;

    default:
        break;
    }
}
