////////////////////////////////////////////////////////////////////
// coptrb.h: Cached object pointer base class definition.
// Does all the things that don't depend on the data stored.
// Copyright(c) 1993 Azarona Software. All rights reserved.
////////////////////////////////////////////////////////////////////
#ifndef H_COPTRB
#define H_COPTRB

class Bucketb;  class Cacheb; // Forward decl's

class Coptrb {
protected:
  long addr;        // File address of bucket data
  Bucketb *bkt;     // Pointer to bucket in the cache
  Cacheb *cache;    // Pointer to the cache being used
  char bound;       // True if pointer is bound to a bucket
  void Copy(const Coptrb &c);
  Bucketb *Alloc(unsigned n, long p=0);
  void Delete(unsigned n);
  void Bind();
  Coptrb(Cacheb &c, long p);
  Coptrb(const Coptrb &c);
  void operator=(const Coptrb &c) { }; // Disallowed
public:
  ~Coptrb();
  void Grab();
  void Release();
  void NotifyUseOf();
  operator long() const;
};

inline Coptrb::~Coptrb()
// We know we can release the bucket when we're through
// with the coptr.
{
  Release();
}

inline Coptrb::operator long() const
// We let a coptr look like a file address, since in a way
// it's just a smart version of a file address.
{
  return addr;
}

inline void Coptrb::Grab()
// Like Bind(), but makes no assumptions about the coptr
// being unbound coming in. For public use.
{
  if (!bound) Bind();
}

#endif
