// data.c

#include "midisx.h"
#include <commdlg.h>
#include <string.h>
#include <memory.h>

// allocate the MIDI input buffer 
BOOL AllocInputBuffer(
    HWND hWnd,
    UINT cbSize)
{
    ReleaseInputBuffer();
        
    hInputBuffer = GlobalAlloc(GHND, (DWORD)cbSize);    
    if (!hInputBuffer)    
    {
        MessageBox(hWnd, "Insufficient memory", "AllocInputBuffer", MB_ICONSTOP);
        return (FALSE);
    }
        
    lpInputBuffer = (LPBYTE)GlobalLock(hInputBuffer);    
    if (!lpInputBuffer)
    {                    
        GlobalFree(hInputBuffer);
        MessageBox(hWnd, "Error locking input buffer", "AllocInputBuffer", MB_ICONSTOP);
        return (FALSE);
    }    
  
    return (TRUE);
}

// release the MIDI input buffer 
void ReleaseInputBuffer(void)
{
    if (lpInputBuffer)
    {          
        GlobalUnlock(hInputBuffer);               
        lpInputBuffer = NULL;
    }
                  
    if (hInputBuffer)
    {
        GlobalFree(hInputBuffer);        
        hInputBuffer = NULL;
    }   
    
    // set global flag for status of input buffer 
    fUnsavedData = FALSE;        
}
 
// query the user to save data in MIDI input buffer                              
int QuerySaveChanges(
    HWND hWnd)
{   
    int nResponse;
   
    if (!fUnsavedData)    
        // no new data to save
        return (IDYES);
 
    nResponse = MessageBox(hWnd, "Save data in MIDI input buffer?",
        "MidiSX", MB_YESNOCANCEL | MB_ICONQUESTION);

    switch (nResponse)
    {
        case IDYES:
            SaveInputBuffer(hWnd);
            break;

        case IDCANCEL:
        case IDNO:
            break;
    }
   
    return (nResponse);
}
     
void SaveInputBuffer(
    HWND hWnd)
{
    OPENFILENAME ofn;
    char szDirName[256];
    char szFile[256], szFileTitle[256];
    UINT u;
    char chReplace;                  
    char szFilter[] = "Raw SysEx(*.RSX)|*.rsx|"; // arbitrary extension
    
    // Retrieve the system directory name, and store it in szDirName
    GetSystemDirectory(szDirName, sizeof(szDirName));
                                                
    // retrieve wildcard in filter                                                
    chReplace = szFilter[strlen(szFilter) - 1]; 

    for (u = 0; szFilter[u] != '\0'; u++)
        if (szFilter[u] == chReplace)
            szFilter[u] = '\0';

    // Set all structure members to zero
    memset(&ofn, 0, sizeof(OPENFILENAME));

    // Initialize the OPENFILENAME members

    szFile[0] = '\0';

    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner = hWnd;
    ofn.lpstrFilter = szFilter;
    ofn.lpstrFile= szFile;
    ofn.nMaxFile = sizeof(szFile);
    ofn.lpstrFileTitle = szFileTitle;

    ofn.nMaxFileTitle = sizeof(szFileTitle);
    ofn.lpstrInitialDir = szDirName;
    ofn.Flags = OFN_SHOWHELP | OFN_OVERWRITEPROMPT;

    if (GetSaveFileName(&ofn))
    {       
        UINT cbData;
        UINT cbWritten;
        OFSTRUCT ofs;
        HFILE hFile;                                    
        LPBYTE lpData = lpInputBuffer;  // point to beginning of input buffer

        // write input buffer to file until MIDI_EOX (0xF7) or end of input
        // buffer is encountered, whichever is first      

        // check for system exclusive status byte        
        if (*lpData == MIDI_SYSEX) 
            // count the first byte
            cbData = 1;
        else
        {
            // data error
            MessageBox(hWnd, "System Exclusive status byte not found", 
                "SaveInputBuffer", MB_ICONHAND);
            return;
        }    
                                                
        // scan input buffer for MIDI_EOX sentinel                      
        while (*(++lpData) != MIDI_EOX)
            // increment the byte count and check buffer limit
            if (++cbData == cbInputBuffer)
                // the end of the input buffer was reached
                break;
        
        if ( cbData < cbInputBuffer && *lpData == MIDI_EOX)
            // count the MIDI_EOX sentinel
            cbData++;
            
        // open the file
        hFile = OpenFile(szFileTitle, &ofs, OF_CREATE | OF_WRITE);
        if (hFile == -1)
        {
            // file error
            MessageBox(hWnd, "Error creating file", "SaveInputBuffer", 
                MB_ICONHAND);
            return;
        }    
        
        // write input buffer to file
        cbWritten = _lwrite(hFile, lpInputBuffer, cbData);   
        if (cbWritten != cbData)         
            // file error
            MessageBox(hWnd, "Error writing file", "SaveInputBuffer", 
                MB_ICONHAND);          
   
        // close the file 
        if (_lclose(hFile))  
        {
            // file error
            MessageBox(hWnd, "Error closing file", "SaveInputBuffer", 
                MB_ICONHAND);   
            return;
        }
        
        fUnsavedData = FALSE;
    }   
}
