/* A database subsystem. */

#define TURBOC
/* If you use Microsoft C/C++, #define MICROSOFTC
   instead of TURBOC.   If you have a different
   C compiler, see the text.
*/

#include "stdio.h"
#include "ctype.h"
#include "dos.h"
#include "conio.h"
#include "string.h"
#include "stdlib.h"
#include "bios.h"

#define MAX_FIELDS 10
#define NAME_SIZE  40
#define MAX_LINES  24
#define LINE_LEN   79
#define FNAME_SIZE  8

enum field_type {STRING, DOUBLE, INT, CHAR};

struct descriptor {
  int position;  /* position of field in record */
  enum field_type type; /* type of field */
  char name[NAME_SIZE]; /* name of field */
  int size;  /* size of field in bytes */
  int x, y;  /* location on screen */
} fields[MAX_FIELDS];

int num_fields;  /* number of fields in a record */
int record_size; /* record size in bytes */
int sort_field;  /* number of field to sort on */
int num_recs;    /* number of records in file */

/* This structure manages the database information. */
struct db_type {  
  char *data;  /* pointer to data */
  struct db_type *prior; /* pointer to previous record */
  struct db_type *next;  /* pointer to next record */
} *first, *last, *cur_rec;

void db(void);
void db_gets(char *str, int maxlen), gotoxy(int x, int y);
void clr_eol(int x, int y);
void clrscr(void), define(void), display_rec(char *p);
void init(void), display_fields(void);
void dls_store(struct db_type *i, struct db_type **first,
               struct db_type **last);
void delete(struct db_type **start, struct db_type **last);
void prompt(char *s), print(void);
void change(char *data, int field);
int save(char *fname), load(char *fname);
int menu(void), enter(void), select_field(char *str);
struct db_type *find( char *key, 
                      struct db_type *from, int field);
struct db_type *search(void), *modify(void);
struct db_type *browse(void);

main(void)
{
  db();
  return 0;
}

/* Database subsystem entry funtion.  You can delete this
   function and substitute your own, if you like.
*/
void db(void)
{
  int choice;
  char fname[80];

  clrscr();
  init();  /* initialize */

  do { 
    choice = menu();
    switch(choice) {
      case 1: /* define structure of database */
        define();
        break;
      case 2: /* enter info */
        enter();
        break;
      case 3: /* browse in the file */
        cur_rec = browse();
        break;
      case 4: /* search */
        cur_rec = search();
        break;
      case 5: /* modify */
        cur_rec = modify();
        break;
      case 6: /* delete */
        delete(&first, &last);
        break;
      case 7: /* print */
        print();
        break;
      case 8: /* save data */
        prompt("Enter file to save (ENTER to skip): ");
        db_gets(fname, FNAME_SIZE);
        if(*fname) save(fname);
        break;
      case 9: /* load data */
        prompt("Enter file to load (ENTER to skip): ");
        db_gets(fname, FNAME_SIZE);
        if(*fname) load(fname);
        break;
    }
  } while(choice!=0);
}

/* Initialize */
void init(void)
{
  num_fields = 0;
  num_recs = 0;
  sort_field = 0;
  first = cur_rec = last = NULL;
}

/* Prompt the user for selection. */
menu(void)
{
  char ch;

  clrscr();
  gotoxy(0, 0);
  printf("0. quit\n");
  printf("1. Define database\n");
  printf("2. Enter information\n");
  printf("3. Browse\n");
  printf("4: Search\n");
  printf("5: Modify\n");
  printf("6: Delete\n");
  printf("7: Print\n");
  printf("8: Save database\n");
  printf("9: Load database\n");
  printf("(%d records in file)", num_recs);
  do {
    gotoxy(0, 12);
    printf("Enter number of your choice: ");
    ch = getche();
  } while(ch <'0' || ch >'9');
  return ch - '0';
}

/* Define the structure of the database. */
void define(void)
{
  char ch;
  int i;
  struct db_type *temp;
  clrscr();

  /* free any previously allocated memory */
  while(first) {
    temp = first->next;
    free(temp->data);  /* free the data */
    free(temp);        /* free the control structure */
    first = temp;
  }
  init();

  record_size = 0;

  do {
    gotoxy(0, 0);
    printf("number of fields (1-%d) (0 to skip): ", MAX_FIELDS);
    scanf("%d%*c", &num_fields);
    if(!num_fields) return;
  } while (num_fields<0 || num_fields>MAX_FIELDS);

  for(i=0; i<num_fields; i++) {
    clrscr();  
    gotoxy(0, 0);
    fields[i].position = record_size;
    printf("Enter name of field %d (1-40 chars): ", i+1);
    db_gets(fields[i].name, 40);
   
    /* The field type is not used by any code in this
       example, but it may be needed by your applications.
    */
    printf("\nField types are: (S)tring, (D)ouble, (I)nt");
    printf(" and (C)har\n");   
    do {
      printf("Enter field type (use first letter): ");
      ch = getche();
      ch = tolower(ch);
    }while(!strchr("sdic", ch));
    switch(ch) {
      case 's': fields[i].type = STRING;
        break;
      case 'd': fields[i].type = DOUBLE;
        break;
      case 'i': fields[i].type = INT;
        break;
      case 'c': fields[i].type = CHAR;
        break;
    }
    printf("\n");

    do {
      printf("enter length of field: ");
      scanf("%d%*c", &fields[i].size);
    } while(fields[i].size<0 || fields[i].size>LINE_LEN);
    fields[i].size++; /* make room for null terminator */

    do {
      printf("enter X and Y coordinates of field: ");
      scanf("%d%d%*c", &fields[i].x, &fields[i].y);
    } while(fields[i].x<0 || 
            fields[i].x+fields[i].size>=LINE_LEN 
            || fields[i].y<0 || fields[i].y>=MAX_LINES);

    printf("Is this field defined OK? (Y/N): ");
    ch = getche();
    if(tolower(ch)!='y')
      i--; /* reenter */
    else
      record_size += fields[i].size;
  }
  printf("\nPress a key to see fields and screen layout");
  getch();
  clrscr();
  display_fields();
  sort_field = select_field("Select field to sort on");
}

/* Enter information into the database. */
enter(void)
{
  register int i, field;
  char ch;
  struct db_type *p;

  for(;;) {
    /* allocate memory for control structure */
    p = malloc(sizeof(struct db_type));  
    if(!p) {
      printf("out of memory\n");
      return 0;
    }
    /* allocate memory for actual data */
    p->data = malloc(record_size);
    if(!p->data) {
      printf("out of memory\n");
      return 0;
    }

    clrscr();
    display_fields();
  
    /* input the data */
    for(i=0; i<num_fields; i++) {
      gotoxy(fields[i].x+strlen(fields[i].name)+1, fields[i].y);
      db_gets(&(p->data)[fields[i].position], fields[i].size-1);     
      if(!(*p->data+fields[0].position)) return 1;
    }
    num_recs++;
    
    prompt("Entry OK? (Y/N): ");
    ch = getche();
    ch = tolower(ch);
    while(ch!='y') {
      field = select_field("Select field to modify");
      change(p->data, field);  /* change field */
      prompt("Entry OK? (Y/N): ");
      ch = getche();
      ch = tolower(ch);
    }
    dls_store(p, &first, &last); /* store it */
  } 
}

/* Display the database fields. */
void display_fields(void)
{
  register int i, j;

  for(i=0; i<num_fields; i++) {
    gotoxy(fields[i].x, fields[i].y);
    printf("%s ", fields[i].name);
    for(j=0; j<fields[i].size-1; j++) putch('.');
  }
}

/* Display one record's information. */
void display_rec(char *p)
{
  register int i;

  display_fields();
  for(i=0; i<num_fields; i++) {
    gotoxy(fields[i].x+strlen(fields[i].name)+1, fields[i].y);
    printf("%s", p+fields[i].position);
  }
}

/* Return the number of the field the user selects
   using the arrow keys.
*/
select_field(char *str)
{
  union key {
    int i;
    char ch[2];
  } k;
  register int i, len;

  prompt(str); /* display prompting message */

  for(i=0;;) {
    /* position cursor at start of each field */
    len = strlen(fields[i].name) + 1;
    gotoxy(fields[i].x+len, fields[i].y);

#ifdef TURBOC
    k.i = bioskey(0);
#endif
#ifdef MICROSOFTC
    k.i = _bios_keybrd(0);
#endif

    if(!k.ch[0])
      switch(k.ch[1]) {
        case 72: /* up */
          if(i) i--;
          break;
        case 80: /* down */
          if(i<num_fields-1) i++;
          break;
      }
    else 
      if(k.ch[0]=='\r') return i;
  } 
}

/* Save both data and description to disk. */
save(char *fname)
{
  struct db_type *info;
  register int i;  
  FILE *fp;
  char name[80];

  strcpy(name, fname);
  strcat(name, ".dat");
  fp = fopen(name, "wb");
  if(!fp) {
    printf("cannot open data file\n");
    return 0;
  }
  prompt("saving data");

  info = first;  
  while(info) { /* save each record */
    fwrite(info->data, record_size, 1, fp);
    info = info->next;  /* get next db_type */
    if(ferror(fp)) {
      printf("error writing file\n");
      fclose(fp);
      return 0;
    }
  }
  fclose(fp);

  strcpy(name, fname);
  strcat(name, ".def");
  fp = fopen(name, "wb");
  if(!fp) {
    printf("cannot open definition file\n");
    fclose(fp);
    return 0;
  }

  putw(record_size, fp);
  putw(num_fields, fp);
  putw(sort_field, fp);

  for(i=0; i<num_fields; i++) {
    fwrite(&fields[i], sizeof(fields[i]), 1, fp);
    if(ferror(fp)) {
      printf("error writing file\n");
      fclose(fp);
      return 0;
    }
  }
  fclose(fp);
  return 1;  
}

/* Load the data and definition files. */
load(char *fname)
{
  struct db_type *info;
  FILE *fp, *fp2;
  register int i;
  char name[80];

  strcpy(name, fname);
  strcat(name, ".dat");
  fp = fopen(name, "rb");
  if(!fp) {
    printf("cannot open file\n");
    return 0;
  }

  strcpy(name, fname);
  strcat(name, ".def");
  fp2 = fopen(name, "rb");
  if(!fp2) {
    printf("cannot open file\n");
    fclose(fp);
    return 0;
  }

  /* free any previously allocated memory */
  while(first) {
    info = first->next; 
    free(info->data); /* free the data */
    free(info);  /* free the control structure */
    first = info;
  }

  /* reset top and bottom pointers */
  first = last = cur_rec = NULL;
  num_recs = 0;

  prompt("loading file");

  /* load DEF file first */
  record_size = getw(fp2);
  num_fields = getw(fp2);
  sort_field = getw(fp2);
  for(i=0; i<num_fields; i++) {
    if(1!=fread(&fields[i], sizeof(fields[i]), 1, fp2)) {
      printf("error reading defintion file\n");
      fclose(fp); fclose(fp2); getch();
      return 0;
    }
  }
  fclose(fp2);

  while(!feof(fp)) {
    /* allocate memory for control structure */
    info = malloc(sizeof(struct db_type));
    if(!info) {
      printf("out of memory");
      fclose(fp);
      return 0;
    }
    /* allocate memory for actual data */
    info->data = malloc(record_size);
    if(!info->data) {
      printf("out of memory");
      fclose(fp);
      return 0;
    }    
    /* read records */
    if(1!=fread(info->data, record_size, 1, fp)) {
      if(ferror(fp)) {
        printf("error reading data file\n");
        fclose(fp); 
        return 0;
      }
      fclose(fp);
      cur_rec = first;
      return 1;
    }
    dls_store(info, &first, &last);
    num_recs++;
  }
  fclose(fp);
  cur_rec = first;
  return 0;
}

/* Modify a record in the list and return
   a pointer to that record. */
struct db_type *modify(void)
{
  struct db_type *info;
  char key[80], ch;
  register int field;

  clrscr();

  /* find field to use for searching */
  display_fields();
  field = select_field("Select field");

  /* find record to search for */
  prompt("Enter key: ");
  db_gets(key, fields[field].size);

  /* find first match, if any */
  info = find(key, first, field);
  while(info) { 
    display_rec(info->data);
    prompt("Modify this record? (Y/N): ");
    ch = getche();
    ch = tolower(ch);
    if(ch!='y') /* find next match, if any */
      info = find(key, info->next, field);
    else break;
  }
  if(!info) return NULL; /* not found */
  
  /* which field to modify? */
  field = select_field("Select field to modify");
  change(info->data, field); /* change data */
  return info;  /* return current location */
}

/* Change the specified record and field. */
void change(char *data, int field)
{
  int i;

  display_rec(data);
  prompt("Enter new data");

  /* display record, position cursor at beginning
     of the field, and input new data
 */
  gotoxy(fields[field].x, fields[field].y);
  printf("%s ", fields[field].name);
  for(i=0; i<fields[field].size-1; i++) putch('.');
  i = strlen(fields[field].name)+1;
  gotoxy(fields[field].x+i, fields[field].y);
  db_gets(data+fields[field].position, fields[field].size-1);
}

/* Browse through the list and return pointer
   to the last record examined. */
struct db_type *browse(void)
{
  union key {
    int i;
    char ch[2];
  } k;  
  struct db_type *info;
  
  clrscr();
  clr_eol(0, MAX_LINES);
  gotoxy(0, MAX_LINES);
  printf("Down Arrow: forward   Up Arrow: backward");
  printf("   Home: beginning   End: end");

  /* If a current location has been defined,
     start at that point; otherwise, start at
     the top of the file.
  */
  if(cur_rec) info = cur_rec;
  else info = first;

  while(info) {
    display_rec(info->data); /* display record */

#ifdef TURBOC
    k.i = bioskey(0);
#endif
#ifdef MICROSOFTC
    k.i = _bios_keybrd(0);
#endif

    if(!k.ch[0])
      switch(k.ch[1]) {
        case 71: /* Home */
          info = first;
          break;
        case 72: /* up */
          if(info->prior)
            info = info->prior;  /* get previous rec */
          break;
        case 79: /* End */
          info = last;
          break;
        case 80: /* down */
          if(info->next) 
            info = info->next;  /* get next rec */
          break;
      }
    else 
      return info;  /* return current location */
  }
  return NULL;
}

/* Find an entry and return a pointer to it. */
struct db_type *find(char *key, struct db_type *from,
                     int field)
{
  struct db_type *info;

  info = from;
  while(info) {
    if(!strcmp(key, info->data+
               fields[field].position)) return info;
    info = info->next;  /* get next db_type */
  }
  return NULL;  /* not found */
}

/* Look for a record in the list and return
   a pointer to the last match found. */
struct db_type *search(void)
{
  int field;
  char key[80], ch;
  struct db_type *info, *temp;

  clrscr();
  display_fields();
  field = select_field("Select field to search on");
  prompt("Enter key: "); /* search string */
  db_gets(key, fields[field].size);
  
  ch = '\0';

  /* find first match, if any */
  info = find(key, first, field); 
  while(info && ch != 'q') {
    /* save pointer in case no other matches are found */
    temp = info;  

    display_rec(info->data);

    /* find next match, if any */
    info = find(key, info->next, field);

    /* tell user if more matches exist */
    if(info) printf("\nmore, ");
    else printf("\nNo more matches, ");
    printf("press a key ('q' to quit): ");
    ch = getch();
    ch = tolower(ch);
  }
  return temp; /* return pointer to last match */
}

/* Print the file either selectively or in full. 

   NOTE: This function works only with DOS, so
   you may need to change it for other environments.

*/
void print(void)
{
  register int i, j, field;
  int blank_lines;
  struct db_type *info;
  char *p, selective, key[80], s[80], ch;

  clrscr();
  prompt("Selective search? (Y/N): ");
  ch = getche();
  if(tolower(ch)=='y') {
    display_fields();
    field = select_field("Select field");
    prompt("Enter key: ");
    db_gets(key, fields[field].size);
    selective = 1;
  }
  else selective = 0;

  /* Number of blank lines between records */
  prompt("Enter number of blank lines: ");
  db_gets(s, 20);
  blank_lines = atoi(s);
  
  prompt("Printing...");

  if(selective)
    info = find(key, first, field);
  else
    info = first;

  while(info) {
    /* display each record as it prints */
    display_rec(info->data);

    for(i=0; i<num_fields; i++) {
      p = info->data+fields[i].position;
      /* print the field */
      while(*p) {
#ifdef TURBOC
        biosprint(0, *p, 0);
#endif
#ifdef MICROSOFTC
        _bios_printer(0, 0, *p);
#endif
        p++;
      }
      /* output a CR-LF */
#ifdef TURBOC
      biosprint(0, '\r', 0);
      biosprint(0, '\n', 0);
#endif
#ifdef MICROSOFTC
      _bios_printer(0, 0, '\r');
      _bios_printer(0, 0, '\n');
#endif
    }
    /* output blank lines between records */
    for(j=0; j<blank_lines; j++) {
#ifdef TURBOC
      biosprint(0, '\r', 0);
      biosprint(0, '\n', 0);
#endif
#ifdef MICROSOFTC
      _bios_printer(0, 0, '\r');
      _bios_printer(0, 0, '\n');
#endif
    }

    /* get next record */
    if(selective) 
      info = find(key, info->next, field);
    else
      info = info->next;
  }
#ifdef TURBOC
  biosprint(0, '\f', 0); /* form feed */
#endif
#ifdef MICROSOFTC
  _bios_printer(0, 0, '\f');
#endif
}

/****************************************************
 Doubly Linked List Routines.
****************************************************/

/* Create a doubly linked list in sorted order.
*/
void dls_store(
  struct db_type *i,   /* new element */
  struct db_type **first, /* first element in list */
  struct db_type **last /* last element in list */
)
{
  struct db_type *old, *p;

  if(*last==NULL) {  /* first element in list */
    i->next = NULL;
    i->prior = NULL;
    *last = i;
    *first = i;
    return;
   }

  p = *first; /* first at top of list */
  
  old = NULL;
  while(p) { /* find location to insert record */
    if(strcmp(p->data+fields[sort_field].position, 
              i->data+fields[sort_field].position)<0) { 
      old = p;
      p = p->next;
    }
    else { 
      if(p->prior) {/* goes in middle of list */
        p->prior->next = i;
        i->next = p;
        i->prior = p->prior;
        p->prior = i;  
        return;
      }
      i->next = p; /* new first element */
      i->prior = NULL;
      p->prior = i;
      *first = i;
      return;
    }
  }
  old->next = i; /* put on end */
  i->next = NULL;
  i->prior = old;
  *last = i;
}

/* Remove an element from the list. */
void delete(struct db_type **start, struct db_type **last)
{
  struct db_type *info;
  char key[80], ch;
  int field;

  clrscr();
  display_fields();
  field = select_field("Select field");
  prompt("Enter key: ");
  db_gets(key, fields[field].size);

  info = find(key, first, field);
  while(info) { 
    display_rec(info->data);
    prompt("Delete this record? (Y/N): ");
    ch = getche();
    ch = tolower(ch);
    if(ch!='y') 
      info = find(key, info->next, field);
    else break;
  }
  if(!info) return; /* not found */

  /* if deleting current rec, set cur_rec to NULL */
  if(cur_rec==info) cur_rec = NULL;

  num_recs--;
  
  if(*start==info) { /* deleting first record */
    *start=info->next;
    if(*start) (*start)->prior = NULL;
    else *last = NULL;
  }
  else { 
    info->prior->next = info->next;
    if(info!=*last)  /* deleting from middle */
      info->next->prior = info->prior;
    else
      *last = info->prior; /* deleting last record */
  }  
  free(info->data); /* free data */
  free(info);  /* free control structure */
}
  
/******************************************************
Miscellaneous support functions.
******************************************************/

/* Display a message on the bottom line of the screen. */
void prompt(char *s)
{
  clr_eol(0, MAX_LINES);
  gotoxy(0, MAX_LINES);
  printf(s);
}

/* Read a string from the keyboard, but do not
   scroll the display when a RETURN is entered.
*/
void db_gets(char *str, int maxlen)
{
  char *p;
  int i;

  p = str;

  for(i=0; ; ) {
    *str = getch();
    if(*str=='\r') { /* return when RETURN entered */
      *str = '\0';  /* NULL terminate */
      return;
    }

    if(*str=='\b') {  /* backspace */
      if(str>p) {
        str--;
        putch('\b');
        putch('.');
        putch('\b');
        i--;
      }
    }
    else if(i<maxlen) {
      putch(*str);
      str++;
      i++;
    }
  }
}

/* Send cursor to specified X,Y (0,0 is upper
   left corner). */
void gotoxy(int x, int y)
{
  union REGS i;

  i.h.dh = y;
  i.h.dl = x;
  i.h.ah = 2;
  i.h.bh = 0;
  int86(16, &i, &i);
}

/* Clear to end of specified line. */
void clr_eol(int x, int y)
{
  gotoxy(x, y);
  for(; x<LINE_LEN; x++) {
    printf(" ");
  }
}

#define MAX_LINES 24
#define LINE_LEN  79
/* Clear the screen. */
void clrscr(void)
{  
  union REGS r;

  r.h.ah = 6;
  r.h.al = 0;
  r.h.ch = 0;
  r.h.cl = 0;
  r.h.dh = MAX_LINES;
  r.h.dl = LINE_LEN;
  r.h.bh = 7;
  int86(0x10, &r, &r);
}
