// dialog.c

#include "midisx.h"
#include "stdlib.h"
#include "memory.h" 
#include <string.h>

extern char * szAppName;

void ValidateUInt(HWND, WORD, UINT, UINT); 
void ValidateHexChar(HWND, WORD);
BOOL ValidateSysExString(HWND, WORD, UINT);
BYTE stoh(char *);

BOOL FAR PASCAL __export AboutProc(
    HWND hDlg,
    UINT uMessage,
    WORD wParam,              
    LONG lParam)
{
    switch (uMessage)
    {        
        case WM_COMMAND:            
            if (wParam == IDOK)
            {
                EndDialog(hDlg, TRUE);
                return (TRUE);
            }
            
            break;
            
        default:
            break;
    }
    
    return (FALSE);
}

BOOL FAR PASCAL __export CommandStringProc(
    HWND hDlg,
    UINT uMessage,
    WORD wParam,
    LONG lParam)
{
    static BOOL fInitialized = FALSE; 
    
    switch (uMessage)
    {
        case WM_INITDIALOG:         
            SetDlgItemText(hDlg, IDC_COMMAND_STRING, achCommandString);
            fInitialized = TRUE;                   
            return (TRUE);

        case WM_COMMAND:
            switch (wParam)
            {          
                case IDC_COMMAND_STRING:
                    if (fInitialized && HIWORD(lParam) == EN_UPDATE)
                        // validate input character before it is displayed
                        ValidateHexChar(hDlg, wParam);
                    break;                                                   
                
                case IDOK:                 
                    // validate entire string
                    if (!ValidateSysExString(hDlg, IDC_COMMAND_STRING, 
                            sizeof(achCommandString)))
                    {
                        // user made mistake in command string
                        SetFocus(GetDlgItem(hDlg, IDC_COMMAND_STRING));                            
                        return (TRUE);
                    }
                    
                    // save changes to system exclusive command string           
                    GetDlgItemText(hDlg, IDC_COMMAND_STRING, 
                        achCommandString, sizeof(achCommandString));
                    
                    TranslateSysExString();       
                   
                    EndDialog(hDlg, TRUE);
                    return (TRUE);
                    
                case IDCANCEL:        
                    // discard changes to system exclusive command string
                    EndDialog(hDlg, FALSE);
                    return (TRUE);
          
                default:
                    break;
            }
        
        default:
            break;
    }
    
    return (FALSE);
}

BOOL FAR PASCAL __export InputBufferProc(
    HWND hDlg,
    UINT uMessage,
    WORD wParam,
    LONG lParam)
{
    static BOOL fInitialized = FALSE; 
    
    switch (uMessage)
    {
        case WM_INITDIALOG:         
            SetDlgItemInt(hDlg, IDC_INPUT_BUFFER_SIZE, cbInputBuffer, FALSE);          
            fInitialized = TRUE;                   
            return (TRUE);

        case WM_COMMAND:
            switch (wParam)
            {
                case IDC_INPUT_BUFFER_SIZE:
                    if (fInitialized && HIWORD(lParam) == EN_UPDATE)
                        // validate input before it is displayed
                        ValidateUInt(hDlg, wParam, INPUT_BUFFER_MIN, INPUT_BUFFER_MAX);
                    break;               
        
                case IDOK:    
                {                  
                    // get the requested buffer size
                    UINT cbNewSize = GetDlgItemInt(hDlg, 
                        IDC_INPUT_BUFFER_SIZE, NULL, FALSE);             
                        
                    if (cbNewSize == cbInputBuffer)
                    {
                        // no change in size requested
                        EndDialog(hDlg, FALSE);
                        return (TRUE);
                    }
                    
                    // a new buffer size was requested   
                    // check for unsaved data because the buffer
                    // will be initialized when it is reallocated                    
                    if (QuerySaveChanges(hDlg) == IDCANCEL ||
                            !AllocInputBuffer(hDlg, cbNewSize))
                    {
                        EndDialog(hDlg, FALSE);
                        return (TRUE);
                    }
                        
                    // update global variable for input buffer size
                    cbInputBuffer = cbNewSize;
    
                    // force display to be repainted
                    InvalidateRect(GetParent(hDlg), NULL, FALSE);
                                        
                    EndDialog(hDlg, TRUE);
                    return (TRUE);
                }
                
                case IDCANCEL:        
                    // discard changes to buffer size
                    EndDialog(hDlg, FALSE);
                    return (TRUE);
          
                default:
                    break;
            }
        
        default:
            break;
    }
    
    return (FALSE);
}

// validate unsigned integer input before it is displayed
void ValidateUInt(
    HWND hDlg,      // handle of dialog box
    WORD wCtrlID,   // control ID of edit containing input   
    UINT uMinimum,  // minimum unsigned integer value accepted
    UINT uMaximum)  // maximum unsigned integer value accepted
{       
    int i, j, nChars;
    DWORD dwValue;
    char achInput[6];          // integers can be up to 5 characters
    BOOL fChanged = FALSE;
                     
    memset(achInput, 0, 6);                     
    nChars = GetDlgItemText(hDlg, wCtrlID, achInput, 6);
          
    // accept only numeric characters
    for (i = 0; i < nChars; i++)
        if (achInput[i] < '0' || achInput[i] > '9')
        {
            // found non-numeric character; remove it
            for (j = i; j < nChars; j++)
                achInput[j] = achInput[j + 1]; 
        
            nChars--;        
            fChanged = TRUE;
        }

    // strip leading zero
    if (nChars > 1 && achInput[0] == '0')
    {        
        for (i = 0; i < nChars; i++)
            achInput[i] = achInput[i + 1]; 
                
        fChanged = TRUE;
    }
    
    if (nChars)
    {  
        // convert input string to integer     
        dwValue = atol(achInput);

        // enforce the input range        
        if (dwValue < uMinimum)
        {
            dwValue = uMinimum;
            fChanged = TRUE;
        }
        else if (dwValue > uMaximum)
        {
            dwValue = uMaximum;
            fChanged = TRUE;
        }
    }
    else if (fChanged)
        // no characters but changes were made - so one was removed.
        // replace single character out of range with minimum
        dwValue = uMinimum;
    
    if (fChanged)
        // send the changed data to the edit control
        SetDlgItemInt(hDlg, wCtrlID, (UINT)dwValue, FALSE);
} 
 
// validate hexadecimal character input for system exclusive command
// before it is displayed
void ValidateHexChar(
    HWND hDlg,      // handle of dialog box
    WORD wCtrlID)   // control ID of edit containing input   
{       
    int i, j, nChars;
    char achInput[COMMAND_STRING_SIZE + 2];  // allow for terminating null
    BOOL fChanged = FALSE;
                     
    memset(achInput, 0, COMMAND_STRING_SIZE + 2);  
    nChars = GetDlgItemText(hDlg, wCtrlID, achInput, COMMAND_STRING_SIZE + 2);
        
    // accept only hexadecimal characters and spaces
    for (i = 0; i < nChars; i++)
        if ((achInput[i] < '0' || achInput[i] > '9') &&
            (achInput[i] < 'a' || achInput[i] > 'f') &&
            (achInput[i] < 'A' || achInput[i] > 'F') &&
            (achInput[i] != ' '))
        {
            // found bad character; remove it
            for (j = i; j < nChars; j++)
                achInput[j] = achInput[j + 1]; 
        
            nChars--;        
            fChanged = TRUE;
        }

    if (nChars > COMMAND_STRING_SIZE)
    {                
        // there is one too many characters in the edit control, and it wasn't
        // a bad character which we've just removed, so truncate the buffer
        
        MessageBox(hDlg, "Maximum command string length reached", szAppName,
            MB_ICONSTOP);
        
        // remove the last character in the buffer
        memset(achInput + COMMAND_STRING_SIZE, 0, 1);
        nChars--;
        fChanged = TRUE;
    }
        
    if (fChanged)
        // send the changed data to the edit control
        SetDlgItemText(hDlg, wCtrlID, achInput);
} 
   
// validate hexadecimal string input for system exclusive command 
// this validates the format of the entire string
BOOL ValidateSysExString(
    HWND hDlg,      // handle of dialog box
    WORD wCtrlID,   // control ID of edit containing input
    UINT uMaxChars) // maximum number of characters accepted   
{       
    int nChars;
    char achInput[COMMAND_STRING_SIZE];
    char achDelimiters[] = " ";
    char * szToken;
    char * szLastToken = NULL;
    int cbMessage = 0;                     
    
    memset(achInput, 0, COMMAND_STRING_SIZE);                     
    nChars = GetDlgItemText(hDlg, wCtrlID, achInput, COMMAND_STRING_SIZE);
          
    // find first token
    szToken = strtok(achInput, achDelimiters);
    
    if (stricmp(szToken, "F0"))
    {
        MessageBox(hDlg, "First byte must be 'F0'", szAppName, MB_ICONSTOP);
        return (FALSE);
    }
    
    while (szToken = strtok(NULL, achDelimiters))
    {
        if (strlen(szToken) != 2)
        {
            MessageBox(hDlg, "Each byte requires 2 characters", 
                szAppName, MB_ICONSTOP);
            return (FALSE);
        }
        
        szLastToken = szToken;
        cbMessage++;
    }   
    
    if (szLastToken && stricmp(szLastToken, "F7"))
    {
        MessageBox(hDlg, "Last byte must be 'F7'", szAppName, MB_ICONSTOP);
        return (FALSE);
    }
    
    if (cbMessage < 2)
    {
        MessageBox(hDlg, "System exclusive message has no data bytes",
            szAppName, MB_ICONSTOP);
        return (FALSE);
    }

    return (TRUE);        
} 
                                           
void TranslateSysExString(void)
{   
    char achBuf[COMMAND_STRING_SIZE];  
    char achDelimiters[] = " ";
    char * szToken;
    BYTE * pBinary = abSysExCommand;
                     
    // reinitialize the binary command string
    memset(pBinary, 0, SYSEX_STRING_SIZE);
    
    // make a copy of the ASCII command string
    strcpy(achBuf, achCommandString);
    
    // note that the entire ASCII string has been pre-validated during input!
    
    // get first token    
    szToken = strtok(achBuf, achDelimiters);                
    
    // convert first token to binary
    *pBinary = stoh(szToken);
    cbSysExCommand = 1;
    
    // convert remaining tokens from ASCII command string into binary
    while (szToken = strtok(NULL, achDelimiters))
    {                  
        *(++pBinary) = stoh(szToken);
        cbSysExCommand++;
    }      
}
                    
// convert pre-validated 2-character ASCII representation of hex value to binary
BYTE stoh(char * szHex)
{   
    BYTE bValue;
    
    // convert first character
    if (*szHex >= 'a' && *szHex <= 'f')
        bValue = (10 + (*szHex - 'a')) << 4;
    else if (*szHex >= 'A' && *szHex <= 'F')
        bValue = (10 + (*szHex - 'A')) << 4;    
    else        
        bValue = (*szHex - '0') << 4;    

    // point to second character    
    szHex++;
                                
    // add value of converted second character                                
    if (*szHex >= 'a' && *szHex <= 'f')
        bValue += 10 + (*szHex - 'a');
    else if (*szHex >= 'A' && *szHex <= 'F')
        bValue += 10 + (*szHex - 'A');    
    else        
        bValue += *szHex - '0';    
   
    return (bValue);   
}
                                                               
BOOL FAR PASCAL __export MidiSXHelpProc(
    HWND hDlg,
    UINT uMessage,
    WORD wParam,              
    LONG lParam)
{
    switch (uMessage)
    {        
        case WM_COMMAND:            
            if (wParam == IDOK)
            {
                EndDialog(hDlg, TRUE);
                return (TRUE);
            }
            
            break;
            
        default:
            break;
    }
    
    return (FALSE);
}
                              
   