// display.c

#include "midisx.h" 
#include <string.h>     // required for strlen()
#include <math.h>       // required for max()
                    
int cxClient, cyClient; // client area width and height                    
int cxChar, cyChar;     // character width and height
int nRows, nColumns;    // size of display
                                         
// Get character size of fixed point font
void GetCharSize(
    HWND hWnd)
{
    TEXTMETRIC tm;
    HDC hDC = GetDC(hWnd);
    
    GetTextMetrics(hDC, &tm);   
    cxChar = tm.tmAveCharWidth;
    cyChar = tm.tmHeight;
    
    ReleaseDC(hWnd, hDC);
}

// set variables needed for displaying received data
void SizeDisplay(
    HWND hWnd, 
    int cx,
    int cy)   
{   
    // remember client area size for DisplayData()
    cxClient = cx;
    cyClient = cy;
    
    // minimum buffer size is 1 x 1
    nRows = max(1, cyClient / cyChar);
    nColumns = max(1, cxClient / cxChar);
        
    // cause screen to be updated
    InvalidateRect(hWnd, NULL, TRUE);
}

// display data in input buffer in client area of application window
void DisplayData(
    HDC hDC)               
{   
    int i, j;
    char achDataByte[4];               // text representation of data byte
    char achBuffer[160];               // will contain a row of text
    LPBYTE lpData = lpInputBuffer;      // point to data in input buffer
    LPBYTE lpEndBuffer;                 // point to end of input buffer
    BOOL fDone = FALSE;

    // clear the display area
    PatBlt(hDC, 0, 0, cxClient, cyClient, WHITENESS); 
           
    if (*lpData != MIDI_SYSEX)
        // if the input buffer doesn't begin with a system exclusive status
        // byte, then there is no data to display
        return;
                                
    // point to end of input buffer                
    lpEndBuffer = lpInputBuffer + cbInputBuffer;
                   
    // display all the data that will fit into the client area
    // one row at a time
    for (i = 0; i < nRows; i++)
    {        
        // initialize text buffer
        memset(achBuffer, 0, 160);
        
        // fill the text buffer with a row of data in hexadecimal                 
        for (j = 0; j < nColumns; j += 3, lpData++)
        {
            wsprintf(achDataByte, "%02X ", *lpData);
            strcat(achBuffer, achDataByte);
                                                
            if (*lpData == MIDI_EOX || lpData == lpEndBuffer)
            {
                // end of exclusive found or end of input buffer reached;
                // there is no more data
                fDone = TRUE;         
                break;
            }
        }  
        
        // display a complete row of text
        TextOut(hDC, 0, cyChar * i, achBuffer, strlen(achBuffer));     
        
        if (fDone)
            return;
    }
}

