/////////////////////////////////////////////////////////////////////
// exchdlr.cpp: "Exception handler" class methods.
// *** THIS IS A STDIO VERSION, UNLIKE IOSTREAM VERSION IN BOOK ***
// Copyright(c) 1993 Azarona Software. All rights reserved.
/////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include "exchdlr.h"

ExceptionHandler DefExceptionHandler;
ExceptionHandler *excp = &DefExceptionHandler;

char *exception_msg[20] = {
  "No exception intended, none taken",         // NONE
  "Using a failed or closed file",             // NOTREADY
  "Using a failed, closed, or read-only file", // NOTWRITEABLE
  "Trying to write to a read-only file",       // READONLYERR
  "Trying to use a closed file",               // NOTOPENERR
  "Error creating file",                       // CREATERR
  "Error opening file",                        // OPENERR
  "Error closing file",                        // CLOSERR
  "Error seeking in file",                     // SEEKERR
  "Error reading/writing to file",             // RWERR
  "Unexpected end of file",                    // EOFERR
  "Wrong file type",                           // FTYPERR
  "File corrupted",                            // CORRUPTERR
  "Invalid path",                              // PATHERR
  "Dangling 'smart pointer'",                  // DANGLINGPTR
  "Accessing a null 'smart pointer'",          // NULLPTR
  "Cache full",                                // CACHEFULL
  "Stack full",                                // STKFULL
  "Stack empty",                               // STKEMPTY
  "Assertion failed"                           // ASSERTERR
};

void ExceptionHandler::PrtException()
{
  printf("%s", exception_msg[exception]);
}

void ExceptionHandler::PrtException(ExceptionCode ec)
{
  printf("%s", exception_msg[ec]);
}

void ExceptionHandler::Report(char *src, int lno,
   char *msg, char *dev)
// Prints out a message corresponding to the exception. If msg
// isn't null, it is printed on the next line. If dev (presumably 
// a device name) isn't null, it is printed on the line below msg.
// NOTE: Some exception codes don't cause anything to be printed.
{
  if (exception == OPENERR)  return; // Trouble opening file
  if (exception == CREATERR) return; // Trouble creating file
  if (exception == FTYPERR) return; // Wrong file type
  if (exception == PATHERR) return; // Invalid path

  if (exception != NONE) {
     if (src) {
        printf("\nEXCEPTION ON line %d of '%s':\n", lno, src);
        printf("%s\n", exception_msg[exception]);
     }
     else {
        printf("\nEXCEPTION: %s\n", exception_msg[exception]);
     }
     if (msg) printf("%s\n", msg);
     if (dev) printf("Error occurred on: %s\n", dev);
  }
  fflush(stdout);
}

void ExceptionHandler::Throw(ExceptionCode ec, char *src, 
  int lno, char *msg, char *dev)
// Prints out a message corresponding to the exception code. 
// Then the execption is handled.
{
  exception = ec;
  Report(src, lno, msg, dev);
  TakeAction();
}

void ExceptionHandler::TakeAction()
// For most exceptions, we'll exit the program.
{
  if (exception == OPENERR) return;  // Trouble opening file
  if (exception == CREATERR) return; // Trouble creating file
  if (exception == FTYPERR) return;  // Wrong file type
  if (exception == PATHERR) return;  // Invalid path
  exit(1);
}
