///////////////////////////////////////////////////////////////
// exchdlr.h: "Exception handler" class definition.
// Note: We are in no way trying to emulate C++ exception
// handling. The class defined allows us to conveniently 
// abort the program when an error occurs.
// Copyright(c) 1993 Azarona Software. All rights reserved.
///////////////////////////////////////////////////////////////
#ifndef H_EXCHDLR
#define H_EXCHDLR

#define THROW(c) Throw(c, __FILE__, __LINE__)

enum ExceptionCode {
  NONE,
  NOTREADY,
  NOTWRITEABLE,
  READONLYERR,
  NOTOPENERR,
  CREATERR,
  OPENERR,
  CLOSERR,
  SEEKERR,
  RWERR,
  EOFERR,
  FTYPERR,
  CORRUPTERR,
  PATHERR,
  DANGLINGPTR,
  NULLPTR,
  CACHEFULL,
  STKFULL,
  STKEMPTY,
  ASSERTERR
};

class ExceptionHandler {
protected:
  virtual void Report(char *src, int lno, char *msg, char *dev);
  virtual void TakeAction();
public:
  ExceptionCode exception;
  ExceptionHandler();
  void Clear();
  void PrtException();
  void PrtException(ExceptionCode ec);
  virtual void Throw(ExceptionCode ec, 
    char *src=0, int lno=0, char *msg=0, char *dev=0);
};

inline ExceptionHandler::ExceptionHandler()
{
  exception = NONE;
}

inline void ExceptionHandler::Clear()
{
  exception = NONE;
}          

extern ExceptionHandler DefExceptionHandler;
extern ExceptionHandler *excp;

#endif

