
/* Font Editor Program. */

#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.
*/

#include "stdio.h"
#include "dos.h"
#include "conio.h"
#include "bios.h"
#include "string.h"
#include "ctype.h"

#define IY        144
#define IX        240

#define XDIM        7   /* change these to change dimensions */
#define YDIM        9   /* of the font you want to use */
#define NUM_CHARS 127   /* number of chars in a font */

#define XPAND       3

#define MAX_LINES  24
#define LINE_LEN   79
 
#define BLANK      0 
#define BCK_GND    9
#define FORE_GND  15
#define HIGHLIGHT 13

char getpoint(int x, int y), getkey(void);
void mode(int mode_code), gotoxy(int x, int y);
void clr_eol(int x, int y), clrscr(void);
void display_grid(char chr),  edit_char(char chr);
void display_char(char chr, int x, int y);
void init_char(char chr), clear_chars(void);
int save_font(void), load_font(char *fname);
void putpoint(int x, int y, int color, int how);
char menu(void);

unsigned char font[NUM_CHARS][XDIM][YDIM];  /* fonts */

char far *egabase;

void main(int argc, char *argv[])
{ 
  char ch, choice;
  mode(16);

#ifdef TURBOC
  egabase = (char far *) MK_FP(0xA000, 0000);
#endif
#ifdef MICROSOFTC
  egabase = (char far *) 0xA0000000;
#endif

  clear_chars();

  if(argc==2) load_font(argv[1]);

  do {
    choice = menu();
    switch(choice) {
      case 'e':  /* edit a character */
        clr_eol(0, 0);
        gotoxy(0, 0);
        printf("enter character to edit: ");
        ch = getch(); printf("%c", ch);
        edit_char(ch);
        break;
      case 's': /* save */
        save_font();
        break;
      case 'l': /* load */
        load_font("");
        break;
    }
  } while(choice!='q'); /* quit */
  mode(2);
}

/* Get user's selection. */
char menu(void)
{
  char ch;

  clr_eol(0, 0);
  gotoxy(0, 0);

  printf("(E)dit character, (S)ave fonts, ");
  printf("(L)oad fonts, (Q)uit\b");
  do {
    clr_eol(0, 1);
    gotoxy(0, 1);
    printf("Enter letter of choice: ");
    ch = getche();
  } while(!strchr("eslq", tolower(ch)));
 
  return tolower(ch);
}

/* Display the expanded grid. */
void display_grid(char chr)
{
  register int x, y;

  for(y = IY; y<(IY+YDIM); y++)
    for(x = IX; x<(IX+XDIM); x++) {
      if(font[chr][x-IX][y-IY] == BLANK)
        putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND),
                 BCK_GND, 0);
      else 
        putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND),
                 font[chr][x-IX][y-IY], 0);
    }
}

/* Edit a character.  This function displays a large
   grid to make editing easy.  It also displays the
   normal size version of the char being edited. */
void edit_char(char chr)
{
  register int x, y;
  char ch, pen, temp;

  clr_eol(0, 0);
  gotoxy(0, 0);
  printf("F1: pen down  F2: pen up  F3: erase  F4: done");

  x = IX; y = IY;  /* set x,y to upper left corner */
  pen = BCK_GND;
  gotoxy(0, 23);
  printf("pen up  ");  /* tell user pen is up */

  display_char(chr, 0, IY);
  display_grid(chr); 
  clr_eol(0, 22);
  gotoxy(0, 22);
  printf("editing character: %c", chr); 

  do {
    /* write pixel to grid */
    putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND), pen, 0);

    /* save pixel color in char, use
       blank background */
    if(pen==BCK_GND)
      font[chr][x-IX][y-IY] = BLANK;
    else font[chr][x-IX][y-IY] = FORE_GND;

    /* write pixel to char */
    putpoint(x-IX, y, font[chr][x-IX][y-IY], 0);

    /* This code displays a dot that indicates the current
       position, waits for a keystroke and then replaces
       the original value. 
    */
    temp = getpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND));
    putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND),
             HIGHLIGHT, 0); 
    ch = getkey();
    putpoint(x+((x-IX)*XPAND), y+((y-IY)*XPAND), temp, 0); 

    switch(ch) {
      case 75: /* left */
        x--;
        break;
      case 77: /* right */
        x++;
        break;
      case 72: /* up */
        y--;
        break;
      case 80: /* down */
        y++;
        break;
      case 71: /* up left */
        x--; y--;
        break;
      case 73: /* up right */
        x++; y--;
        break;
      case 79: /* down left*/
        x--; y++;
        break;
      case 81: /* down right */
        x++; y++;
        break;
      case 59: /* F1 - pen down */
        pen = FORE_GND;  
        gotoxy(0, 23);
        printf("pen down");
        break;
      case 60: /* F2 - pen up */
        pen = BCK_GND;  
        gotoxy(0, 23);
        printf("pen up  ");
        break;
      case 61: /* F3 - erase */
        init_char(chr);
        display_char(chr, 0, IY);
        display_grid(chr);
        break;
    }
    if(x < IX) x++;  if(x > IX+XDIM-1) x--;
    if(y < IY) y++;  if(y > IY+YDIM-1) y--;

  } while(ch != 62); /* F4 - quit */
  clr_eol(0, 22);
  clr_eol(0, 23);
  return;
}

/* Displays the char at specified X,Y. */
void display_char(char chr, int startx, int starty)
{
  register int x, y;

  for(y = starty; y<starty+YDIM; y++)
    for(x = startx; x<startx+XDIM; x++)
      putpoint(x, y, font[chr][x-startx][y-starty], 0);
}

/* Initialize all characters. */
void clear_chars(void)
{
  register int i;

  for(i=0; i<NUM_CHARS; i++)
    init_char(i);
}

/* Initializes the character. */
void init_char(char chr)
{
  register int x, y;

  for(x= 0; x<XDIM; x++)
    for(y=0; y<YDIM; y++)
      font[chr][x][y] = BLANK;
}

/* Saves the font. */
save_font(void)
{
  FILE *fp;
  char fname[80];
  int result;

  gotoxy(0, 22);
  printf("save to: ");
  gets(fname);

  if((fp = fopen(fname, "wb"))==NULL) {
    printf("cannot open file");
    return 0;
  }

  fwrite(font, sizeof font, 1, fp);

  if(ferror(fp)) result = 0;
  else result = 1;
  fclose(fp);
  clr_eol(0, 22);
  return result;
}

/* Loads the font. */
load_font(char *name)
{
  FILE *fp;
  char fname[80];
  int result;

  if(!*name) {
    clr_eol(0, 22);
    gotoxy(0, 22);
    printf("load from: ");
    gets(fname);
  }
  else strcpy(fname, name);

  if((fp = fopen(fname, "rb"))==NULL) {
    printf("cannot open file");
    return 0;
  }

  fread(font, sizeof font, 1, fp);

  if(ferror(fp)) result = 0;
  else result = 1;
  fclose(fp);
  clr_eol(0, 22);
  return result;
}

  

/* Return the scan code of the key pressed. 
   This function is compatible with Turbo/Borland C/C++ and
   Microsoft C/C++ (be sure to #define the appropriate
   symbol).  If you use a different compiler, consult
   the text.
*/
char getkey(void)
{
  union key {
    int i;
    char ch[2];
  } k;

#ifdef TURBOC
  k.i = bioskey(0);
#endif
#ifdef MICROSOFTC
  k.i = _bios_keybrd(0);
#endif
  return k.ch[1];
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Send the cursor to x,y. */
void gotoxy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Clears to end of line. */
void clr_eol(int x, int y)
{
   for(; x<80; x++) {
     gotoxy(x, y);
     printf(" ");
   }
}

/* Returns the value at the specified pixel. */
char getpoint(int x, int y)
{
  union REGS r;

  r.h.ah = 13;
  r.x.dx = y;
  r.x.cx = x;
  r.h.bh = 0;
  int86(16, &r, &r);
  return r.h.al;
}

/* This version of putpoint will work for all video
   adapters but is incredibly slow!
*/
/*
void putpoint(int x, int y, int color, int how)
{
  union REGS r;

  if(how==0x18) color = color | 128;
  r.h.bh = 0;
  r.h.ah = 12;
  r.h.al = color;
  r.x.dx = y;
  r.x.cx = x;
  int86(16, &r, &r);
}
*/

/* Clear the screen. */
void clrscr(void)
{  
  union REGS r;

  r.h.ah = 6;
  r.h.al = 0;
  r.h.ch = 0;
  r.h.cl = 0;
  r.h.dh = MAX_LINES;
  r.h.dl = LINE_LEN;
  r.h.bh = 7;
  int86(0x10, &r, &r);
}


/* This function sets the specified pixel to the 
   specified color using an EGA/VGA video adapter.
   The value of the parameter "how" may be one of
   these:

   action        value

   overwrite       0
   XOR             0x18
   AND             8
   OR              0x10
   
*/
#define ENABLE 0x0F
#define INDEXREG 0x3CE
#define VALREG 0x3CF

#define OUTINDEX(index, val) outp((INDEXREG), (index));\
                             outp((VALREG), (val));

#define WIDTH 80L

/* These range values assume EGA mode 16 */
#define XMAX 639
#define YMAX 349
#define XMIN 0
#define YMIN 0

void putpoint(int x, int y, int color, int how)
{
  register unsigned char mask = 0x80;
  register char far *base;
  unsigned dummy;

  /* if you want range checking at this point, activate
     this line of code.  It will slow down the function
     however.
  */
  /*if(x<XMIN || x>XMAX || y<YMIN || y>YMAX) return;*/

  base = (char far *) (egabase 
         + ( (long) y * WIDTH + (long) x/8L ));

  mask >>= x % 8;

  /* This causes the memory READ necessary to
     load the EGA/VGA's internal registers.
  */
  dummy = *base;

  OUTINDEX(0, color);
  OUTINDEX(1, ENABLE);
  OUTINDEX(3, how);
  OUTINDEX(8, mask);

  *base = 1;

  OUTINDEX(0, 0);
  OUTINDEX(1, 0);
  OUTINDEX(3, 0); 
  OUTINDEX(8, 0xff);
}
