
/* Font display subsystem and test program. */

#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.
*/

#include "stdio.h"
#include "dos.h"
#include "conio.h"
#include "bios.h"
#include "string.h"
#include "stdlib.h"

#define IY        144
#define IX        240
#define XDIM        7
#define YDIM        9
#define NUM_CHARS 127
#define XPAND       3
#define XMAX      639
#define YMAX      349

#define BCK_GND     0
#define FORE_GND   15

void mode(int mode_code);
void display_char(char chr);
int  load_font(char *fname);
void putpoint(int x, int y, int color, int how);
void font_puts(char *str);
void font_xy(int x, int y);
void setscale(int x, int y);
void setcolor(int color);

unsigned char font[NUM_CHARS][XDIM][YDIM];  /* fonts */

char far *egabase;
int xscale=1, yscale=1;
int fontx=0, fonty=0;
int color=FORE_GND;

/* A simple main() to demonstrate the font routines */
void main(int argc, char *argv[])
{ 
  mode(16);

#ifdef TURBOC
  egabase = (char far *) MK_FP(0xA000, 0000);
#endif
#ifdef MICROSOFTC
  egabase = (char far *) 0xA0000000;
#endif

  if(argc==2) {
    if(!load_font(argv[1])) exit(1);
  }
  else exit(1);
  
  font_xy(0, 0);
  font_puts("ABCDEFGHIJKLMNOPQRSTUVWXYZ$.!");
  font_puts("abcdefghijklmnopqrstuvwxyz");

  font_xy(0, 100);
  setcolor(12);
  font_puts("This is an example\nline of text.");

  setcolor(14);
  setscale(2, 3);
  font_xy(0, 200);
  font_puts("This is a bigger line of text.");
  
  setscale(3, 4);
  font_xy(0, 300);
  font_puts("This is even bigger!");

  getch();
  mode(2);
}

/* Display a string. */ 
void font_puts(char *str)
{
  while(*str) {
    display_char(*str);
    str++;
  }
}

/* Position the font cursor to the specified X,Y */
void font_xy(int x, int y)
{
  if(x<0 || x>XMAX) return;
  if(y<0 || y>YMAX) return;
  fontx = x;
  fonty = y;
}

/* Set the scale factors. */
void setscale(int x, int y)
{
  if(x<0 || y<0) return;

  xscale = x;
  yscale = y;
}

/* Set the foreground color. */
void setcolor(int c)
{
  if(c<0) return;
  color = c;
}

/* Displays the character at current X,Y. */
void display_char(char chr)
{
  register int x, y, i, j, clr;

  /* is a carriage return */
  if(chr=='\n') {
    fontx = 0;
    fonty += YDIM*yscale + 1;
    return;
  }

  for(y = 0; y<YDIM; y++)
    for(x = 0; x<XDIM; x++) {
      for(i=0; i<xscale; i++) 
        for(j=0; j<yscale; j++) {
          if(font[chr][x][y]==FORE_GND) clr = color;
          else clr = BCK_GND; 
          putpoint(x*xscale+fontx+i, 
                   y*yscale+fonty+j, clr, 0);
        }
    }
    fontx += XDIM*xscale + 1;
}

/* Loads the font. */
load_font(char *fname)
{
  FILE *fp;
  int result;

  if((fp = fopen(fname, "rb"))==NULL) {
    printf("cannot open file");
    return 0;
  }

  fread(font, sizeof font, 1, fp);

  if(ferror(fp)) result = 0;
  else result = 1;
  fclose(fp);
  return result;
}

  
/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* This version of putpoint will work for all video
   adapters but is incredibly slow!
*/
/*
void putpoint(int x, int y, int color, int how)
{
  union REGS r;

  if(how==0x18) color = color | 128;
  r.h.bh = 0;
  r.h.ah = 12;
  r.h.al = color;
  r.x.dx = y;
  r.x.cx = x;
  int86(16, &r, &r);
}
*/

/* This function sets the specified pixel to the 
   specified color using an EGA/VGA video adapter.
   The value of how may be one of these:

   action        value

   overwrite       0
   XOR             0x18
   AND             8
   OR              0x10
   
*/
#define ENABLE 0x0F
#define INDEXREG 0x3CE
#define VALREG 0x3CF
#define OUTINDEX(index, val) outp((INDEXREG), (index));\
                             outp((VALREG), (val));

#define WIDTH 80L

/* These range values assume EGA mode 16 */
#define XMAX 639
#define YMAX 349
#define XMIN 0
#define YMIN 0

void putpoint(int x, int y, int color, int how)
{
  register unsigned char mask = 0x80;
  register char far *base;
  unsigned dummy;

  /* if you want range checking at this point, activate
     this line of code.  It will slow down the function
     however.
  */
  /*if(x<XMIN || x>XMAX || y<YMIN || y>YMAX) return;*/

  base = (char far *) (egabase 
         + ( (long) y * WIDTH + (long) x/8L ));

  mask >>= x % 8;

  /* This causes the memory READ necessary to
     load the EGA/VGA's internal registers.
  */
  dummy = *base;

  OUTINDEX(0, color);
  OUTINDEX(1, ENABLE);
  OUTINDEX(3, how);
  OUTINDEX(8, mask);

  *base = 1;

  OUTINDEX(0, 0);
  OUTINDEX(1, 0);
  OUTINDEX(3, 0); 
  OUTINDEX(8, 0xff);
}
