
/* A simple animated video game of TAG.

   The object of the game is for your "person" to
   tag the other "person" by running into them.  

   Your person is green, the computer's is yellow. No
   red boundary may be crossed.

   For the tag to count, the two "people" must be within
   one pixel of being directly on top of one another.
*/

#include "dos.h"
#include "stdio.h"
#include "math.h"
#include "time.h"
#include "conio.h"

#define XMAX 319
#define YMAX 199

#define COMPUTER 0
#define HUMAN 1

#define IDLE 0
#define DOWN 1
#define UP -1
#define LEFT -1
#define RIGHT 1

void line(int startx, int starty, int endx, int endy, 
         int color);
void mempoint(int x, int y, int color);
void palette(int pnum);
void mode(int mode_code);
void display_object(int ob[][5], int lines);
void xhairs(int x, int y), goto_xy(int x, int y);
void save_pic(void), load_pic(void);
void it_comp_move(int ob1[][5], int ob2[][5], int human[][5], 
                  int sides);
void not_it_comp_move(int ob1[][5], int ob2[][5],
                      int directx, int directy, int sides);
void show_score(int it, long htime, long ctime);
void update_object(int ob[][5], int x, int y,
                   register int sides);
int readkey(void), kbhit(void);
int is_legal(int ob[][5], int x, int y, int sides);
int tag(int ob1[][5], int ob2[][5]);

int human[4][5] = { /* your sprites */
  6, 1, 6, 6, 1,
  2, 4, 9, 3, 1,
  1, 9, 6, 6, 1,
  11, 9, 6, 6, 1
};

int human2[4][5] = {
  6, 1, 6, 6, 1,
  2, 4, 9, 3, 1,
  3, 9, 6, 6, 1,
  9, 9, 6, 6, 1
};

int computer[4][5] = { /* computer's sprites */
  6, 180, 6, 185, 3,
  2, 183, 9, 182, 3,
  1, 188, 6, 185, 3,
  11, 188, 6, 185, 3
};

int computer2[4][5] = {
  6, 180, 6, 185, 3,
  2, 183, 9, 182, 3,
  3, 188, 6, 185, 3,
  9, 188, 6, 185, 3
};

int directx, directy; /* direction of human */

main(void)
{
  union k { /* holds both scan and key codes */
    char c[2];
    int i;
  } key;

  int deltax = 0, deltay = 0; /* direction of movement */
  int swaph = 0, swapc = 0; /* select sprites */
  int it = COMPUTER; /* who's "it" */
  long htime, ctime, starttime, curtime;
  int count; /* used to let the players separate slightly
                after a tag */ 

  mode(4);  /* switch to mode 4 graphics */
  palette(0); /* palette 0 */
  
  load_pic(); /* get the game board */

  time(&starttime); /* setup the clocks */
  htime = ctime = 0;
  
  display_object(human, 4);
  display_object(computer, 4);
  count = 0;

  /* main game loop */
  do {
    /* update the score counters */
    time(&curtime);
    if(it==COMPUTER) htime += curtime-starttime;
    else ctime += curtime-starttime;
    time(&starttime); 
    show_score(it, htime, ctime);
  
    if(kbhit()) { /* if keypressed */
      directx = directy = IDLE; /* reset direction for each move */
      key.i = readkey(); /* read the key */
      deltax = 0; deltay = 0;
      if(!key.c[0]) switch(key.c[1]) {
        case 75: /* left */
          deltax = -1;
          directx = LEFT;
          break;
        case 77: /* right */
          deltax = 1;
          directx = RIGHT;
          break;
        case 72: /* up */
          deltay = -1;
          directy = UP;
          break;
        case 80: /* down */
          deltay = 1;
          directy = DOWN;
          break;
        case 71: /* up left */
          deltax = -1; deltay = -1;
          directy = UP; directx = LEFT;
          break;
        case 73: /* up right */
          deltay = -1; deltax = 1;
          directy = UP; directx = RIGHT;
          break;
        case 79: /* down left*/
          deltay = 1; deltax = -1;
          directy = DOWN; directx = LEFT;
          break;
        case 81: /* down right */
          deltay = 1; deltax = 1;
          directy = DOWN; directx = RIGHT;
          break;
      }
    }
    /* turn off player's person */
    if(swaph<5) display_object(human, 4);
    else display_object(human2, 4);

    /* see if move is legal */
    if(is_legal(human, deltax, deltay, 4)) {
      update_object(human, deltax, deltay, 4);
      update_object(human2, deltax, deltay, 4);
    }

    /* see if a tag has occured */
    if(!count && tag(human, computer)) {
      it = !it;  /* switch who is "it" */
      count = 6; /* don't allow an instant re-tag */
    }

    swaph++;
    if(swaph==10) swaph = 0;
    /* redisplay person in new position */ 
    if(swaph<5) display_object(human, 4);
    else display_object(human2, 4);

    /* turn off computer's person */
    if(swapc<5) display_object(computer, 4);
    else display_object(computer2, 4);

    /* generate the computer's move */
    if(it==COMPUTER) it_comp_move(computer, computer2, human, 4);
    else not_it_comp_move(computer, computer2, directx, directy, 4);
    if(!count && tag(human, computer)) {
      it = !it; /* switch who is "it" */
      count = 6; /* don't allow an instant re-tag */
      /* if computer tag's person, shift computer's  X
         position by 2 so that it is not trivial to re-tag.
      */
      if(is_legal(computer, 2, 0, 4)) { 
        update_object(computer, 2, 0, 4);
        update_object(computer2, 2, 0, 4);
      }
      else {
        update_object(computer, -2, 0, 4);
        update_object(computer2, -2, 0, 4);
      }

    }
    swapc++;
    if(swapc==10) swapc = 0;
    /* display computer's person */
    if(swapc<5) display_object(computer, 4);
    else display_object(computer2, 4);

    if(count) count--;
  } while (key.c[0]!='q' && htime<999 && ctime<999);
  mode(3);

  if(ctime>htime) printf("Computer wins!");
  else printf("You win!");

  return 0;
}

/* Display the score. 
   When the YOU or ME is in caps, points 
   are scored.
*/
void show_score(int it, long htime, long ctime)
{
  goto_xy(6, 24); 
  if(it==COMPUTER) printf("YOU:%ld", htime);
  else printf("you:%ld", htime);
  goto_xy(26, 24); 
  if(it==HUMAN) printf("ME:%ld", ctime);
  else printf("me:%ld", ctime);
}

/* Set the palette. */
void palette(int pnum)
{
  union REGS r;

  r.h.bh = 1;   /* code for palette select */
  r.h.bl = pnum;
  r.h.ah = 11;  /* set palette function */
  int86(0x10, &r, &r);
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Draw a line in specified color
   using Bresenham's integer based algorithm.
*/
void line(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int t, distance;
  int x=0, y=0, delta_x, delta_y;
  int incx, incy;

  /* Compute the distances in both directions. */
  delta_x = endx-startx;
  delta_y = endy-starty;

  /* Compute the direction of the increment.
     An increment of 0 means either a vertical
     or horizontal line.
  */
  if(delta_x>0) incx = 1;
  else if(delta_x==0) incx = 0;
  else incx = -1;

  if(delta_y>0) incy = 1;
  else if(delta_y==0) incy = 0;
  else incy = -1;

  /* determine which distance is greater */
  delta_x = abs(delta_x);
  delta_y = abs(delta_y);
  if(delta_x>delta_y) distance = delta_x;
  else distance = delta_y;

  /* draw the line */
  for(t=0; t<=distance+1; t++) {
    mempoint(startx, starty, color);
    x += delta_x;
    y += delta_y;  
    if(x>distance) {
      x -= distance;
      startx += incx;
    }
    if(y>distance) {
      y -= distance;
      starty += incy;
    }
  }
}

/* Write a point directly to mode 4 graphics RAM. */
void mempoint(int x, int y, /* coordinate of point */
              int color) /* color */
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;
  char xor; /* xor color in or overwrite */

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 0xFF3F;  /* 11111111 00111111 in binary */

  /* check range */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return;

  xor = color & 128; /* see if xor mode is set */
  color = color & 127; /* mask off high bit */
 
  /* set bit_mask and color bits to the right location */
  bit_position = x%4;
  color <<= 2*(3-bit_position);
  bit_mask.i >>= 2*bit_position;


  /* find the correct byte in screen memory */
  index = y*40 + (x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* write the color */
  if(!xor) { /* overwrite mode */
    t = *(ptr+index) & bit_mask.c[0];
    *(ptr+index) = t | color;
  }
  else { /* xor mode */
    t = *(ptr+index) | (char) 0;
    *(ptr+index) = t ^ color;
  }
}

/* Display crosshair locator. */
void xhairs(int x, int y)
{
  line(x-4, y, x+3, y, 1 | 128);
  line(x, y+4, x, y-3, 1 | 128);
}


/* Read byte directly from the video RAM in mode 4. */
unsigned char read_point(int x, int y)
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 3;       /* 11111111 00111111 in binary */

  /* check range for mode 4 */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return 0;

  /* set bit_mask and color_code bits to the right location */
  bit_position = x%4;
  bit_mask.i <<= 2*(3-bit_position);
  /* find the correct byte in screen memory */
  index = y*40 +(x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* read the color */
  t = *(ptr+index) & bit_mask.c[0];
  t >>= 2*(3-bit_position);
  return t;
}

/* Save the video graphics display. */
void save_pic(void)
{
  char fname[80];
  FILE *fp;
  register int i, j;
  int e=0;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  char far *temp;
  unsigned char buf[14][80]; /* hold the contents of screen */ 

  temp = ptr;
  /* save the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      buf[i][j] = *temp; /* even byte */
      buf[i][j+1] = *(temp+8152); /* odd byte */
      *temp = 0; *(temp+8152) = 0;  /* clear top of screen */
      temp++;
    }
      
  goto_xy(0, 0);
  printf("Game board: ");
  gets(fname);
  if(!(fp=fopen(fname, "wb"))) {
    goto_xy(0, 0);
    printf("cannot open file - press a key ");
    getch();
    e = 1;  /* flag error */
  }

  temp = ptr;
  /* restore the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      *temp = buf[i][j];
      *(temp+8152) = buf[i][j+1];
      temp++;
    }

 if(e) return; /* if file could not be opened, exit */

 /* save image to file */ 
 for(i=0; i<8152; i++) { 
    putc(*ptr, fp); /* even byte */
    putc(*(ptr+8152), fp); /* odd byte */
    ptr++;
  }
 
  fclose(fp);
}

/* Load the video graphics display. */
void load_pic(void)
{
  char fname[80];
  FILE *fp;
  register int i, j;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  char far *temp;
  unsigned char buf[14][80]; /* hold the contents of screen */ 

  temp = ptr;
  /* save the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      buf[i][j] = *temp;
      buf[i][j+1] = *(temp+8152);
      *temp = 0; *(temp+8152) = 0; /* clear the top of the screen */
      temp++;
    }
      
  goto_xy(0, 0);
  printf("Filename: ");
  gets(fname);
  if(!(fp=fopen(fname, "rb"))) {
    goto_xy(0, 0);
    printf("cannot open file\n");
    temp = ptr;
    /* restore the top of the current screen */
    for(i=0; i<14; i++)
      for(j=0; j<80; j+=2) {
        *temp = buf[i][j];
        *(temp+8152) = buf[i][j+1];
        temp++;
      }
    return;
  }

  /* load image from file */
  for(i=0; i<8152; i++) { 
    *ptr = getc(fp); /* even byte */
    *(ptr+8152) = getc(fp); /* odd byte */
    ptr++;
  }

  fclose(fp);
}

/* Send the cursor to the specified X,Y position. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}


/* Display an object. */
void display_object(int ob[][5], int lines)
{
  register int i;

  for(i=0; i<lines; i++)
    line(ob[i][0], ob[i][1],
         ob[i][2], ob[i][3], ob[i][4] | 128);
}

/* Update an object's position as specified in x, y. */
void update_object(int ob[][5], /* object */
  int x, int y, /* amount to update */
  register int sides) /* number of sides */
{
  sides--;
  for( ; sides>=0; sides--) {
    ob[sides][0] += x;
    ob[sides][1] += y;
    ob[sides][2] += x;
    ob[sides][3] += y;
  }
}

/* See if a prospective move is legal. 
   Returns 1 if legal; 0 otherwise. 
*/
is_legal(int ob[][5], /* object */
  int x, int y, /* increment to move */
  int sides) /* number of sides the object has */
{
  if(x==0 && y==0) return 1;
  sides--;
  for( ; sides>=0; sides--) {
    /* check for out of range */
    if(ob[sides][0]+x>XMAX || ob[sides][1]+y >YMAX) return 0;
    if(ob[sides][2]+x<0 || ob[sides][3]+y<0) return 0;
    /* check for obstacle */
    if(read_point(ob[sides][0]+x, ob[sides][1] + y)==2) return 0;
    if(read_point(ob[sides][2]+x, ob[sides][3] + y)==2) return 0;
  }
  return 1;
}

/* Generate the computer's move when it is "it". */
void it_comp_move(int ob1[][5], int ob2[][5],
  int human[][5], 
  int sides)
{
  register int  x, y;
  static skip = 0;

  skip++;
  if(skip==3) {
    skip = 0;
    return; /* skip every other time to slow computer */
  }
  x = 0; y = 0;

  /* move toward the human */
  if(human[0][0]<ob1[0][0]) x = -1;
  else if(human[0][0]>ob1[0][0]) x = 1;

  if(human[0][1]<ob1[0][1]) y = -1;
  else if(human[0][1]>ob1[0][1]) y = 1;

  if(is_legal(ob1, x, y, sides)) {
    update_object(ob1, x, y, sides);
    update_object(ob2, x, y, sides);   
  }
  else { /* if not legal, try to go around */
    if(x && is_legal(ob1, x, 0, sides)) {
      update_object(ob1, x, 0, sides);
      update_object(ob2, x, 0, sides);   
    }
    else if(is_legal(ob1, 0, y, sides)) {
      update_object(ob1, 0, y, sides);
      update_object(ob2, 0, y, sides);   
    }
  }
}

/* Generate the computer's move when it is not "it". */
void not_it_comp_move(int ob1[][5], int ob2[][5],
  int directx, int directy, /* direction of human's last move */
  int sides)
{
  register int  x, y;
  static skip = 1;

  skip++;
  if(skip==3) {
    skip = 0;
    return; /* skip every other time to slow computer */
  }
  x = 0; y = 0;

  /* move in opposite direction as human */
  x = -directx;
  y = -directy;

  if(is_legal(ob1, x, y, sides)) {
    update_object(ob1, x, y, sides);
    update_object(ob2, x, y, sides);   
  }
  else { /* if not legal, try to go around */
    if(x && is_legal(ob1, x, 0, sides)) {
      update_object(ob1, x, 0, sides);
      update_object(ob2, x, 0, sides);   
    }
    else if(is_legal(ob1, 0, y, sides)) {
      update_object(ob1, 0, y, sides);
      update_object(ob2, 0, y, sides);   
    }
  }
}

/* See if a tag has taken place. */
tag(int ob1[][5], int ob2[][5])
{
  register int i;
 
  /* To tag, one figure must be within one pixel of being 
     of being directly on top of the other.
  */
  for(i=-1; i<2; i++) {
    if(ob1[0][0]==ob2[0][0]+i && ob1[0][1]==ob2[0][1]+i) {
      return 1;
    }
  }
  return 0;
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}


