

/* Get a file from the file server. */

#define PORT 0

#include "dos.h"
#include "stdio.h"
#include "conio.h"
#include "stdlib.h"
#include "ctype.h"

void rec_file(char *fname);
void send_file_name(char *f), get_file_name(char *f);
void wait(int port), sport(int port, char c);
void port_init(int port, unsigned char code);
int rport(int port), check_stat(int port);
long filesize(FILE *fp);

main(int argc, char *argv[])
{
  if(argc!=2) {
    printf("Usage: get <filename>");
    exit(1);
  }

  port_init(PORT, 231); /* initalize the serial port */

  rec_file(argv[1]);
}

/* Receive a file. */
void rec_file(char *fname)
{
  FILE *fp;  
  char ch;
  int i;
  union {
    char c[sizeof(long)];
    long count;
  } cnt;

  printf("Receiving file %s\n", fname);

  if(!(fp=fopen(fname, "wb"))) {
    printf("Cannot open output file\n");
    exit(1);
  }

  sport(PORT, 's'); /* tell server to send a file */
  wait(PORT); /* wait until server is ready */

  send_file_name(fname);

  /* get file length */
  for(i=0; i<sizeof(long); i++) {
    cnt.c[i] = rport(PORT);
    sport(PORT, '.'); /* acknowledge */
  }
  printf("File size: %ld\n", cnt.count);
  
  for(i=0; cnt.count; cnt.count--) {
    ch = rport(PORT);
    putc(ch, fp);
    if(ferror(fp)) {
      printf("Error writing file");
      exit(1);
    }
    sport(PORT, '.'); /* acknowledge */

    i++;
    if(i==1024) {
      printf(".");
      i = 0;
    }
  }
  fclose(fp);
}

/* Send the file name */
void send_file_name(char *f)
{
  do {
     sport(PORT, '?'); 
  } while(!kbhit() && !(check_stat(PORT)&256));
  if(kbhit()) {
    getch();
    exit(1);
  }
  wait(PORT);  /* wait for receiver to acknowledge */

  /* actually send the name */
  while(*f) {
    sport(PORT, *f++);
    wait(PORT); /* wait for receiver to acknowledge */
  }
  sport(PORT, '\0'); /* null terminator */
  wait(PORT);
}

/* Wait for a response. */
void wait(int port)
{
  if(rport(port) != '.') {
    printf("Communication error\n");
    exit(1);
  }
}

/* Send a character out the serial port. */
void sport(int port, /* i/o port */
           char c) /* character to send */
{
  union REGS r;

  r.x.dx = port; /* serial port */
  r.h.al = c; /* char to send */
  r.h.ah = 1;  /* send character function */
  int86(0x14, &r, &r);
  if(r.h.ah & 128) {
    printf("Send error detected in serial port");
    exit(1);
  }
}

/* Read a character from a port. */
rport(int port) /* i/o port */
{
  union REGS r;

  /* wait for a character */
  while(!(check_stat(port)&256)) 
    if(kbhit()) { /* abort on keypress */
      getch();
      exit(1);
    }

  r.x.dx = port; /* serial port */
  r.h.ah = 2;  /* read character function */
  int86(0x14, &r, &r);
  if(r.h.ah & 128) 
    printf("Read error detected in serial port");
  return r.h.al;
}

/* Check the status of the serial port. */
check_stat(int port) /* i/o port */
{
  union REGS r;

  r.x.dx = port; /* serial port */
  r.h.ah = 3;  /* read status */
  int86(0x14, &r, &r);
  return r.x.ax;
}

/* Initialize the port. */
void port_init(int port, unsigned char code)
{
  union REGS r;

  r.x.dx = port; /* serial port */
  r.h.ah = 0;  /* initialize port function */
  r.h.al = code; /* initialization code - see text for details */
  int86(0x14, &r, &r);
}


