/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
/*
     INTO THE THIRD DIMENSION
     ------------------------
     Garry J. Vass
     Gundhof Str 18
     D6000 Frankfurt 71
     Germany

     This is a collection of classes (a school of classes? a herd of
     classes? perhaps a gaggle of classes?  maybe a galaxy of objects?
     no wait - an Eintopf of Klassen).  Anyway, this is an Eintopf
     of classes that I put together to do crude wire frame plots of 
     three dimensional functions.  They are not efficient, they are 
     not generic, they are not flexible, they are not great, they are 
     not the product of inspired genius.  They are simple, portable, 
     and they work for me.  I commented in some places 
     and didn't comment in other places.  
     
     Oh yeah, don't even think about firing this up without
     a screamer in front of you.  
     
     Take it or leave it.  Such is life in the download lane.  
     
     Contrary to my fellow uploaders, if you find some 
     improvement to be made, don't tell me about it.  In all
     likelihood, I will never get around to fixing anything.
     
     You are free to reuse this code in commercial applications
     as long as you change all the variable names and claim
     to be the original author.  If you give me credit, be
     sure to include a German translation show I can show it
     off to my friends.
     
     I ported the code from a Smalltalk V implementation 
     into C++.  If you are interested in the Smalltalk 
     classes, they should laying around in some other 
     forum or board.
     
     The code is not warranted to work.  Even to the extent
     that it does what it says it does.  

     Basically, a three dimensional function is derived from
     z = f( x, y ) where something interesting results.  My approach
     was to calculate such a point, call it a logical point and
     to convert logical points into physical points, ones that could
     be displayed on a two dimensional surface.  The transformation
     looks like this:
                           | a value
                           |     | a value
                           |     |         a result
                           |     |         |
          logical  point ( x,    y,    f( x, y ) ) becomes
                           |     |         |
          physical point   |     |         a color
                           |     a screen coordinate
                           a screen coordinate

    Beyond that, there's not much to say from a design perspective.
    Like any C++ file, you should probably examine it from the bottom
    up.  To create a plot of your own, simply hack at one of the samples
    until you get the idea of what's going on.
    
    I developed this under Borland, Microsoft, and Zortech.  In December
    1992, these were respectively (3.1, 7.0, and 3.0), and they
    worked fine.  To compile under Borland, use the command line:
        bcc gjv3d.cpp graphics.lib
    To compile under Zortech, use the command line:
        ztc gjv3d.cpp fg.lib
    To compile under Microsoft, use the command line:
        cl gjv3d.cpp graphics.lib
        
    Because of the different coordinate mapping systems in these
    three compilers, one of them will produce plots that appear
    to be upsidedown - depending upon your point of view.  If
    a plot seems to be upsidedown, add 180 degrees to the y
    axis rotation.
    
    With respect to coding style, classes are named such that
    the command, 
                 grep c[A-Z]+ *.c*  will locate them.  
                 grep cBASE[A-Z]+ locates virtual classes
    Similarly,
                 grep e[A-Z]+ locates enumerations, 
                 grep s[A-Z]+ locates structures,
                 and grep t[A-Z]+ locates typedefs.  
    
    The opening bracket for a method is on the same line as the declaration.
    The opening bracket for a function is on a separate line. 
    There are NEVER spaces before the parenthesis of an argument
    list, a looping construct, or a decision construct.  There is
    ALWAYS one space following any "(" or "[" (expect for null argument
    lists).  
    
    Alignment is based upon the first character of the 
    controlling logic structure or condition, for example:
         while( thisandthat ) {
                dothat();
         }
         if( a > b ) {
             dothat();
         }
*/
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
/*
    Here are the defines for supporting the various compiler
    venders.  Uncomment or comment as needed.
*/    
//#define __ZORTECH__
//#define __BORLAND__
#define __MICROSOFT__
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
#include   <stdio.h>
#include  <stdlib.h>
#include    <math.h>
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
#define MIN(x,y) ((x)<(y) ? (x) : (y))
#define MAX(x,y) ((x)>(y) ? (x) : (y))
#define RADIANS_PER_DEGREE   0.0174533
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
class cBASE_GRAPHICS {
public:
    virtual void start() = 0;
    virtual void draw_line( int color, int x1, int y1, int x2, int y2 ) = 0;
    virtual int xpixels( void ) = 0;
    virtual int ypixels( void ) = 0;
    virtual int maxcolors( void ) = 0;
};
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
/*
   The Micosoft implementation
*/
#ifdef __MICROSOFT__
#include <graph.h>
class cPLOTTER : public cBASE_GRAPHICS {
private:
    struct videoconfig vc;
public:
    cPLOTTER( void ) {}
    void start( void ) {
        if( !_setvideomode( _VRES16COLOR ) ) {
            exit( 0 );
        }
        _getvideoconfig( &vc );
    }
    void draw_line( short color, short x1, short y1, short x2, short y2 ) {
        _setcolor( color );
        _moveto( x1, y1 );
        _lineto( x2, y2 );
    }
    int xpixels( void ) {
        return( vc.numxpixels );
    }
    int ypixels( void ) {
        return( vc.numypixels );
    }
    int maxcolors( void ) {
        return( 16 );
    }
};    
#endif
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
/*
   The Zortech implementation
*/
#ifdef __ZORTECH__
#include <fg.h>
class cPLOTTER : public cBASE_GRAPHICS {
public:
    cPLOTTER( void ) {}
    void start( void ) {
        fg_init_vga13();
    }
    void draw_line( int color, int x1, int y1, int x2, int y2 ) {
        fg_line_t line_data;
        line_data[ FG_X1 ] = x1;
        line_data[ FG_X2 ] = x2;
        line_data[ FG_Y1 ] = y1;
        line_data[ FG_Y2 ] = y2;
        fg_drawlineclip( fg_color[ color ], FG_MODE_SET, ~0, FG_LINE_SOLID, line_data, fg_displaybox );
    }
    int xpixels( void ) {
        return fg_displaybox[ FG_X2 ];
    }
    int ypixels( void ) {
        return fg_displaybox[ FG_Y2 ]; 
    }
    int maxcolors( void ) {
        return( 16 );
    }
};    
#endif
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
/*
   The Borland implementation
*/
#ifdef __BORLAND__
#include <graphics.h>
class cPLOTTER : public cBASE_GRAPHICS {
public:
    cPLOTTER( void ) {}
    void start( void ) {
        int gd = VGA;
        int gm = VGAHI;
        initgraph( &gd, &gm, NULL );
    }
    void draw_line( int color, int x1, int y1, int x2, int y2 ) {
        setcolor( color );
        line( x1, y1, x2, y2 );
    }
    int xpixels( void ) {
        return( getmaxx() );
    }
    int ypixels( void ) {
        return( getmaxy() );
    }
    int maxcolors( void ) {
        return( getmaxcolor() );
    }
};
#endif
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
cPLOTTER Plotter;  // finally, a plotter class is instantiated.
                   // I know it is uncool to instantiate a 
                   // virtual class as a global variable, but
                   // life is short and you only go around once.
                   // With luck, all of the code that follows
                   // is independent of machine characteristics.
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
class c3DPOINT {
public:
    double x;  // the "X" coordinate
    double y;  // the "Y" coordinate
    double z;  // the "Z" coordinate
    // the single constructor for this class simply assigns
    // a value to each of the coordinates.
    c3DPOINT( double _x, double _y, double _z ) {
        x = _x;
        y = _y;
        z = _z;
    }
    // an overloaded "+" operator is provided so that
    // points can be "added".
    c3DPOINT operator+( c3DPOINT& p ) {
        c3DPOINT r( x + p.x, y + p.y, z + p.z );
        return( r );
    }
    // transformation operations involving trigonometic points
    // are done in radians.  Users prefer to visualize and work
    // in degrees, hence, a conversion function.
    friend double degrees_to_radians( double degrees ) 
    {
        double angle;
        while( degrees >= 360 )
               degrees -= 360;
        while( degrees < 0 )
               degrees += 360;
        angle = RADIANS_PER_DEGREE * degrees;
        return( angle );
    }
};
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
class c3DMATRIX {  // the base class for rotation and transformation
protected:
    double mat[ 3 ][ 3 ]; // a 3 x 3 table of doubles
};
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
class cPPROJ : public c3DMATRIX {
private:
    double   distance; // distance from the eye to the object
    c3DPOINT   origin; // origin point of the projection
    double       xcos; // cosine of the angle of rotation about the "X" axis
    double       xsin; // sine of the angle of rotation about the "X" axis
    double       ycos; // cosine of the angle of rotation about the "Y" axis
    double       ysin; // sine of the angle of rotation about the "Y" axis
    double       zcos; // cosine of the angle of rotation about the "Z" axis
    double       zsin; // sine of the angle of rotation about the "Z" axis
    // calculation of intermediate trig points
    void calc_trig( double xaxis_rotation_in_radians,
                    double yaxis_rotation_in_radians,
                    double zaxis_rotation_in_radians ) {
        xcos = cos( xaxis_rotation_in_radians );
        xsin = sin( xaxis_rotation_in_radians );
        ycos = cos( yaxis_rotation_in_radians );
        ysin = sin( yaxis_rotation_in_radians );
        zcos = cos( zaxis_rotation_in_radians );
        zsin = sin( zaxis_rotation_in_radians );
    }
public:
    cPPROJ( double distance_from_the_eye,
            double xaxis_rotation_in_radians,
            double yaxis_rotation_in_radians,
            double zaxis_rotation_in_radians ) :
        origin( 0, 0, 0 ) {
        calc_trig( xaxis_rotation_in_radians,
                   yaxis_rotation_in_radians,
                   zaxis_rotation_in_radians );
        distance = distance_from_the_eye;
        mat[ 0 ][ 0 ] = ( ( ycos * zcos ) - ( zsin * xsin * ysin ) );
        mat[ 0 ][ 1 ] = ( ( 0 - ( ycos * zsin ) ) - ( xsin * zcos * ysin ) );
        mat[ 0 ][ 2 ] = ( xcos * ysin );
        mat[ 1 ][ 0 ] = ( zsin * xcos );
        mat[ 1 ][ 1 ] = ( xcos * zcos );
        mat[ 1 ][ 2 ] = ( xsin );
        mat[ 2 ][ 0 ] = ( ( 0 - ( zcos * ysin ) ) - ( zsin * xsin * ycos ) );
        mat[ 2 ][ 1 ] = ( ( zsin * ysin ) - ( xsin * zcos * ycos ) );
        mat[ 2 ][ 2 ] = ( xcos * ycos );
        origin.x = distance_from_the_eye * xcos * zsin;
        origin.y = distance_from_the_eye * xcos * zcos;
        origin.z = distance_from_the_eye * xsin;
    }
    c3DPOINT three2two( c3DPOINT& p ) { // keep inline
        double e1 = ( mat[ 0 ][ 0 ] * ( p + origin ).x ) +
                    ( mat[ 0 ][ 1 ] * ( p + origin ).y ) +
                    ( mat[ 0 ][ 2 ] * ( p + origin ).z );
        double e2 = ( mat[ 1 ][ 0 ] * ( p + origin ).x ) +
                    ( mat[ 1 ][ 1 ] * ( p + origin ).y ) +
                    ( mat[ 1 ][ 2 ] * ( p + origin ).z );
        double e3 = ( mat[ 2 ][ 0 ] * ( p + origin ).x ) +
                    ( mat[ 2 ][ 1 ] * ( p + origin ).y ) +
                    ( mat[ 2 ][ 2 ] * ( p + origin ).z );
        c3DPOINT result( ( distance * ( e1 / e2 ) ),
                         ( distance * ( e3 / e2 ) ),
                           p.z );
        return( result );
    }
};
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
class cSCALER {
public:
     double     maxx;  // maximum "X" value encountered in the series
     double     minx;  // minimum "X" value encountered in the series
     double     maxy;  // maximum "Y" value encountered in the series
     double     miny;  // minimum "Y" value encountered in the series
     double     maxz;  // maximum "Z" value encountered in the series
     double     minz;  // minimum "Z" value encountered in the series
     double   xrange;  // maximum - minimum
     double   yrange;  // maximum - minimum
     double   zrange;  // maximum - minimum
     int     screenx;  // physical "X" dimension of the viewing surface
     int     screeny;  // physical "Y" dimension of the viewing surface
     int      colors;  // number of colors available
     int       first;  // a lazy programmer's working variable
     cSCALER( void ) {
         first = 1;
         xrange = yrange = zrange = 0;
         screenx = screeny = colors = 0;
     }
     void track( c3DPOINT& p );
     int calc_ranges( int _screenx, int _screeny, int _colors );
     void calc_screen( c3DPOINT& p, int *x, int *y, int *c );
};
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
class cEXTREMA { // Definition of the cEXTREMA class. A graphic object
                 // in two dimensions is defined by a range of "X" values
                 // and a range of "Y" values.  Each range has a minimum
                 // and maximum value and a "step" (i.e., the increment
                 // value).  Popular extrema for the Mandelbrot set,
                 // for example are:
                 //             X from -2 to 2 step 0.01
                 //             Y from -1.5 to 1.5 step 0.01
protected:
    double     min;  // the minimum value for evaluation
    double     max;  // the maximum value for evaluation
    double    step;  // the "tick" or increment value
    double current;  // the current value
public:
    cEXTREMA( double _min, double _max, double _step ) {
        min     = _min;
        max     = _max;
        step    = _step;
        current = min - step;
    }
    double next( void ) { // returns the next value in the the series
        return( current += step );
    }
    int done( void ) { // answers if the range is complete
        return( current > max );
    }
    void reset( void ) {
        current = min - step;
    }
    friend class cORGANIZER;
};
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
class cORGANIZER {
private:
    cEXTREMA *X;
    cEXTREMA *Y;
    cPPROJ   *Proj;
    cSCALER  Scaler;
    double (*func)( double, double );
    long total_number_of_points; // bookkeeping for later installment as DLL
public:
    cORGANIZER( cPPROJ *_P ) {
        Proj = _P;
        total_number_of_points = 0L;
    }
    void set_x_extrema( cEXTREMA *_X ) {
        X = _X;
    }
    void set_y_extrema( cEXTREMA *_Y ) {
        Y = _Y;
    }
    void set_function( double (*_func)( double, double ) ) {
        func = _func;
    }
    int compute_scale( int screenx, int screeny, int colors );
    void create_wireplot( cBASE_GRAPHICS *g );
};
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
// out of line functions begin here
    int cORGANIZER::compute_scale( int screenx, int screeny, int colors ) {
        for( double y = Y -> max; y > Y -> min; y -= Y -> step ) {
             for( double x = X -> min; x < X -> max; x += X -> step ) {
                  c3DPOINT l( x, y, func( x, y ) );
                  c3DPOINT ph = ( Proj -> three2two( l ) );
                  Scaler.track( ph );
                  ++total_number_of_points;
             }
        }
        // return value: 1 if a plot can be generated
        //               0 if a plot cannot be generated
        int retval = Scaler.calc_ranges( screenx, screeny, colors );
        return( retval );
    }

    void cORGANIZER::create_wireplot( cBASE_GRAPHICS *g ) {
        for( double y = Y -> max; y > Y -> min; y -= Y -> step ) {
             for( double x = X -> min; x < X -> max; x += X -> step ) {
                  c3DPOINT p1( x, y, func( x, y ) );
                  c3DPOINT p2( x + X -> step, y, func( x + X -> step, y ) );
                  c3DPOINT p3( x + X -> step, y + Y -> step, func( x + X -> step, y + Y -> step ) );
                  int sx1, sy1, sx2, sy2, c1, c2;
                  Scaler.calc_screen( Proj -> three2two( p1 ), &sx1, &sy1, &c1 );
                  Scaler.calc_screen( Proj -> three2two( p2 ), &sx2, &sy2, &c2 );
                  g -> draw_line( MIN( c1, c2 ), sx1, sy1, sx2, sy2 );
                  Scaler.calc_screen( Proj -> three2two( p3 ), &sx1, &sy1, &c1 );
                  g -> draw_line( MIN( c1, c2 ), sx1, sy1, sx2, sy2 );
             }
        }
    }
     void cSCALER::track( c3DPOINT& p ) {
         if( first ) {
             first = 0;
             maxx = minx = p.x;
             maxy = miny = p.y;
             maxz = minz = p.z;
         } else {
             maxx = MAX( maxx, p.x );
             minx = MIN( minx, p.x );
             maxy = MAX( maxy, p.y );
             miny = MIN( miny, p.y );
             maxz = MAX( maxz, p.z );
             minz = MIN( minz, p.z );
         }
     }
     int cSCALER::calc_ranges( int _screenx, int _screeny, int _colors ) {
         xrange  = maxx - minx;
         yrange  = maxy - miny;
         zrange  = maxz - minz;
         screenx = _screenx;
         screeny = _screeny;
         colors  = _colors;
         if( !xrange || !yrange || !zrange ) {
             return( 0 );
         }
         return( 1 );
     }
     void cSCALER::calc_screen( c3DPOINT& p, int *x, int *y, int *c ) {
         double pctx = ( p.x - minx ) / xrange;
         double pcty = ( p.y - miny ) / yrange;
         double pctz = ( p.z - minz ) / zrange;
         *x = (int)( pctx * screenx );
         *y = (int)( pcty * screeny );
         *c = (int)( pctz * colors );
     }
// out of line functions end here
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
// samples begin here
 #define P     (  0.33267   )
 #define B1    (  0.4361836 )
 #define B2    ( -0.1201676 )
 #define B3    (  0.9372980 )
 #define ZVAL(x) ( ( 0.39894225840659091773 ) * exp( -0.5 * x * x ) )
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
double normal( double x )
{
	double t = 1.0 / ( 1.0 + P * fabs( x ) );
	double p_of_x = 1.0 - ZVAL( x ) *
                 ( t * ( B1
                 + t * ( B2
                 + t * ( B3
                 ))));
    if( x > 0.0 )
        return( p_of_x );
    return( 1.0 - p_of_x );
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
double sample1( double x, double y )
{
    double z = ( sin( x ) * cos( y ) );
    return( z );
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
int do_sample1( )
{
double d1 = 45;
double d2 = 210;
double d3 = 15;
    cEXTREMA X( -2.0, 2.0, 0.05 );
    cEXTREMA Y( -2.0, 2.0, 0.05 );
    cPPROJ proj( 100, degrees_to_radians( d1 ), degrees_to_radians( d2 ), degrees_to_radians( d3 ) );
    cORGANIZER O( &proj );
    O.set_x_extrema( &X );
    O.set_y_extrema( &Y );
    O.set_function( sample1 );
    if( O.compute_scale( Plotter.xpixels(), Plotter.ypixels(), Plotter.maxcolors() ) ) {
        O.create_wireplot( &Plotter );
    }
    return( 0 );
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
double sample2( double x, double y )
{
    double z = ( normal( x ) * cos( y ) );
    return( z );
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
int do_sample2( )
{
double d1 = 30;
double d2 = 180;
double d3 = 60;
    cEXTREMA X( -2.0, 2.0, 0.05 );
    cEXTREMA Y( -6.0, 6.0, 0.05 );
    cPPROJ proj( 100, degrees_to_radians( d1 ), degrees_to_radians( d2 ), degrees_to_radians( d3 ) );
    cORGANIZER O( &proj );
    O.set_x_extrema( &X );
    O.set_y_extrema( &Y );
    O.set_function( sample2 );
    if( O.compute_scale( Plotter.xpixels(), Plotter.ypixels(), Plotter.maxcolors() ) ) {
        O.create_wireplot( &Plotter );
    }
    return( 0 );
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
#ifdef __BORLAND__
#include <complex.h>
#endif
#ifdef __ZORTECH__
#include <complex.hpp>
#endif
double sample3( double x, double y )
{
#ifdef __MICROSOFT__
    printf( "with Microsoft, you have to provide a complex class\n" );
    exit( 0 );
    return( 0 );
#else
    complex z( y, x );
    complex c = z;
    for( double i = 0; i < 16; i++ ) {
         if( abs( z ) > 2 )
             break;
         z = ( z * z ) + c;
    }
    return( i );
#endif    
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
int do_sample3( )
{
double d1 = 80;
double d2 = 190;
double d3 = 230; 
    cEXTREMA X( -2.0, 2.0, 0.0125 );
    cEXTREMA Y( -2.0, 2.0, 0.0125 );
    cPPROJ proj( 1314.15, degrees_to_radians( d1 ), degrees_to_radians( d2 ), degrees_to_radians( d3 ) );
    cORGANIZER O( &proj );
    O.set_x_extrema( &X );
    O.set_y_extrema( &Y );
    O.set_function( sample3 );
    if( O.compute_scale( Plotter.xpixels(), Plotter.ypixels(), Plotter.maxcolors() ) ) {
        O.create_wireplot( &Plotter );
    }
    return( 0 );
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
#include   <conio.h> 
#include   <ctype.h>
int main()
{
int response;
    printf( "\n\nWelcome!  Please select one of these...\n" );
    printf( "a.    your basic sine/cosine plot (really fast)\n" );
    printf( "b.    z = Normal(x)*Normal(y) (interesting)\n" );
    printf( "c.    3D Mandelbrot (fascinating, but takes forever)\n" );
    printf( " ---> Your choice? " );
    fflush( stdin );
    response = getch();
    fflush( stdin );
    switch( toupper( response ) ) {
            case 'A':
                 Plotter.start();
                 do_sample1();
                 break;
            case 'B':
                 Plotter.start();
                 do_sample2();
                 break;
            case 'C':
                 Plotter.start();
                 do_sample3();
                 break;
    }
    printf( "All done! hit a key\n" );
    getch();
    return( 0 );
}
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
