
/* A short program to illustrate the core graphics
   functions.
 */
#include "dos.h"
#include "stdio.h"
#include "stdlib.h"
#include "conio.h"

#define XMAX 319
#define YMAX 199

void line(int startx, int starty, int endx, int endy, 
         int color);
void box(int startx, int starty, int endx, int endy, 
         int color);
void fill_box(int startx, int starty, int endx, int endy, 
         int color);
void circle(int x_center, int y_center, int radius, int color);
void fill_circle(int x_center, int y_center, int radius, int color);
void plot_circle(int x, int y, int x_center, int y_center, 
                 int color);
void mempoint(int x, int y, int color);
void palette(int pnum);
void mode(int mode_code);

double asp_ratio; /* holds aspect ratio for cirlces */

main(void)
{
  mode(4);
  palette(0);

  line(0, 0, 100, 100, 1);
  box(50, 50, 80, 90, 2);
  fill_box(100, 0, 120, 40, 3);
  circle(100, 160, 30, 2);
  fill_circle(250, 150, 20, 1);

  getch();
  mode(3);

  return 0;
}

/* Set the palette. */
void palette(int pnum)
{
  union REGS r;

  r.h.bh = 1; /* code for palette select */
  r.h.bl = pnum;
  r.h.ah = 11;  /* set palette function */
  int86(0x10, &r, &r);
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}


/* Draw a box. */
void box(int startx, int starty, /* upper left corner */
         int endx, int endy, /* lower right corner */
         int color) /* color */
{
  line(startx, starty, endx, starty, color);
  line(startx, starty, startx, endy, color);
  line(startx, endy, endx, endy, color);
  line(endx, starty, endx, endy, color);
}
 

/* Draw a line in specified color
   using Bresenham's integer based algorithm.
*/
void line(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int t, distance;
  int x=0, y=0, delta_x, delta_y;
  int incx, incy;

  /* Compute the distances in both directions. */
  delta_x = endx-startx;
  delta_y = endy-starty;

  /* Compute the direction of the increment.
     An increment of 0 means either a vertical
     or horizontal line.
  */
  if(delta_x>0) incx = 1;
  else if(delta_x==0) incx = 0;
  else incx = -1;

  if(delta_y>0) incy = 1;
  else if(delta_y==0) incy = 0;
  else incy = -1;

  /* determine which distance is greater */
  delta_x = abs(delta_x);
  delta_y = abs(delta_y);
  if(delta_x>delta_y) distance = delta_x;
  else distance = delta_y;

  /* draw the line */
  for(t=0; t<=distance+1; t++) {
    mempoint(startx, starty, color);
    x += delta_x;
    y += delta_y;  
    if(x>distance) {
      x -= distance;
      startx += incx;
    }
    if(y>distance) {
      y -= distance;
      starty += incy;
    }
  }
}

/* Fill box with specified color. */
void fill_box(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int i, begin, end;

  begin = startx<endx ? startx : endx;
  end = startx>endx ? startx : endx;

  for(i=begin; i<=end;i++)
    line(i, starty, i, endy, color);
}

/* Draw a circle using Bresenham's integer based algorithm. */
void circle(int x_center, int y_center, /* center */
            int radius, /* radius */
            int color) /* color */
{
  register int x, y, delta;

  asp_ratio = 1.0;  /* for different aspect ratios, alter 
                   this number */

  y = radius;
  delta = 3 - 2 * radius;

  for(x=0; x<y; ) {
    plot_circle(x, y, x_center, y_center, color);

    if (delta < 0)
      delta += 4*x+6;
    else {
      delta += 4*(x-y)+10;
      y--;
    }
    x++;
  }
  x = y;
  if(y) plot_circle(x, y, x_center, y_center, color);
}

/* Plot_circle actually prints the points that
   define the circle.
*/
void plot_circle(int x, int y, /* point to plot */
                 int x_center, int y_center, /* center */
                 int color) /* color */
{
  int startx, endx, x1, starty, endy, y1;

  starty = y*asp_ratio;
  endy = (y+1)*asp_ratio;
  startx = x*asp_ratio;
  endx = (x+1)*asp_ratio;

  for (x1=startx; x1<endx; ++x1)  {
    mempoint(x1+x_center, y+y_center, color);
    mempoint(x1+x_center, y_center-y, color);
    mempoint(x_center-x1, y_center-y, color);
    mempoint(x_center-x1, y+y_center, color);
  }

  for (y1=starty; y1<endy; ++y1) {
    mempoint(y1+x_center, x+y_center, color);
    mempoint(y1+x_center, y_center-x, color);
    mempoint(x_center-y1, y_center-x, color);
    mempoint(x_center-y1, x+y_center, color);
  }
}

/* Fill a circle by repeatedly calling circle()
   with smaller radius.
*/
void fill_circle(int x_center, int y_center,
                 int radius, int color)
{
  while(radius) {
    circle(x_center, y_center, radius, color);
    radius--;
  }
}

/* Write a point directly to mode 4 graphics RAM. */
void mempoint(int x, int y, /* coordinate of point */
              int color) /* color */
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;
  char xor; /* xor color in or overwrite */

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 0xFF3F;  /* 11111111 00111111 in binary */

  /* check range */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return;

  xor = color & 128; /* see if xor mode is set */
  color = color & 127; /* mask off high bit */
 
  /* set bit_mask and color bits to the right location */
  bit_position = x%4;
  color <<= 2*(3-bit_position);
  bit_mask.i >>= 2*bit_position;


  /* find the correct byte in screen memory */
  index = y*40 + (x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* write the color */
  if(!xor) { /* overwrite mode */
    t = *(ptr+index) & bit_mask.c[0];
    *(ptr+index) = t | color;
  }
  else { /* xor mode */
    t = *(ptr+index) | (char) 0;
    *(ptr+index) = t ^ color;
  }
}

