
/* Icon-based DOS Interface. */

#define TURBOC
/* If you use Microsoft C/C++, #define MICROSOFTC
   instead of TURBOC.   If you have a different
   C compiler, see the text.
*/

#include "stdio.h"
#include "dos.h"
#include "conio.h"
#include "bios.h"
#include "stdlib.h"
#include "process.h"
#include "string.h"
#include "ctype.h"

#define IY 10
#define IX 10
#define XDIM 40
#define YDIM 32
#define NUM_ICONS 10
#define XPAND 3
#define ICON_X_SPACING 50
#define ICON_Y_SPACING 40
#define NUM_ACROSS  4

#define BCK_GND   9
#define FORE_GND 15
#define HIGHLIGHT 13


/* DOS system functions */
void dir(void), copy(void), erase(void), run(void);
void type(void), icon_end(void), chkdsk(void), ver(void);

struct icon_type {
  unsigned char image[XDIM][YDIM];  /* icon */
  void (*func)();  /* pointer to function associated
                     with each icon */
  int x, y;       /* X, Y Coordinates */
} icons[NUM_ICONS];

char *icon_files[] = {
  "copy.ico",
  "erase.ico", 
  "dir.ico",
  "type.ico",
  "run.ico",
  "chkdsk.ico",
  "ver.ico",
  "end.ico",
  ""
};

void (*sys_funcs[])() = {
  copy, erase, dir, type, run, chkdsk, ver, icon_end
};

int num_icons;
char far *egabase;

char getkey(void);
void mode(int mode_code), gotoxy(int x, int y);
void clr_eol(int x, int y), clrscr(void);
void display_grid(void),  edit_icon(void);
int save_icon(void), load_icons(void);
void putpoint(int x, int y, int color, int how);
void box(int startx, int starty,
         int endx, int endy, int how);
void line(int startx, int starty,
         int endx, int endy, int how);
int icon_menu(struct icon_type *icons, int num);
void display_icons(struct icon_type *icons, 
                   register int num);
void display_icon(struct icon_type icon);
void invert_icon(struct icon_type icon);
void move(struct icon_type *icon);

void main(void)
{ 
  register int i, j;
  int selection;

  mode(16);
#ifdef TURBOC
  egabase = (char far *) MK_FP(0xA000, 0000);
#endif
#ifdef MICROSOFTC
  egabase = (char far *) 0xA0000000;
#endif
  
  if((num_icons = load_icons())==0) exit(1);

  /* Compute X,Y positions for icons. This loop 
     automatically positions the icons in
     row, each row having NUM_ACROSS icons
     in it. */
  for(i=0, j=0; i<num_icons; i++, j++) {
    /* at end of row, reset X cooridinate */
    if(!(j%NUM_ACROSS)) j=0; 

    icons[i].x = j*ICON_X_SPACING+IX;
    icons[i].y = IY + (ICON_Y_SPACING * (i/NUM_ACROSS)) ;
  }

  /* initialize function fields */
  for(i=0; i<num_icons; i++)
    icons[i].func = sys_funcs[i];

  for(;;) {
    box(0, 0, 210, 90, 0);
    selection = icon_menu(icons, num_icons);
    (*icons[selection].func)();
    printf("\npress ENTER to return to menu\n");
    getch();
    clrscr();
  } 
}

/* Display icon-based menu and return user's
   selection.  Must be passed a pointer to the
   icon array and the number of icons.
*/
icon_menu(struct icon_type *icons, int num)
{
  register int selection;
  char ch;

  display_icons(icons, num);

  gotoxy(0, 23);
  printf("Arrow keys to move, ENTER (or F1) to select,");
  printf(" F2 to move icon");
  selection = 0;
  for(;;) {
    invert_icon(icons[selection]);
    ch = getkey();
    display_icon(icons[selection]);
    switch(ch) {
      case 75: /* left */
        selection--;
        break;
      case 77: /* right */
        selection++;
        break;
      case 72: /* up */
        selection -= NUM_ACROSS;
        break;
      case 80: /* down */
        selection += NUM_ACROSS;
        break;
      case 60: /* F2: move an icon */
        move(&icons[selection]);
        clr_eol(0, 23);
        gotoxy(0, 23);
        printf("Arrow keys to move, ENTER (or F1) to select,");
        printf(" F2 to move icon");
        break;
      case 28: /* CR: */
      case 59: /* F1: make a selection */
        clr_eol(0, 23);
        return selection;
    }
    if(selection < 0) selection = num_icons-1;
    if(selection > num_icons-1) selection = 0;
  } 
}

/* Reverse the colors on the specified icon.  This
   is used as a highlight.
*/
void invert_icon(struct icon_type icon)
{
  register int x, y;

  for(y = icon.y; y<icon.y+YDIM; y++)
    for(x = icon.x; x<icon.x+XDIM; x++) 
      if(icon.image[x-icon.x][y-icon.y] == BCK_GND)
        putpoint(x, y, FORE_GND, 0);
      else
        putpoint(x, y, BCK_GND, 0);

}

/* Display an icon at its X,Y location. */
void display_icon(struct icon_type icon)
{
  register int x, y;

  for(y = icon.y; y<icon.y+YDIM; y++)
    for(x = icon.x; x<icon.x+XDIM; x++)
      putpoint(x, y, icon.image[x-icon.x][y-icon.y], 0);
}

/* Display all icons.  Must be passed a pointer
   to the icon array and the number in the array.
*/
void display_icons(struct icon_type *icons, 
                   register int num)
{
  register int i;

  for(i=0; i<num; i++)
    display_icon(icons[i]);
}

/* Reposition an icon.  Must be passed a
   pointer to the icon to reposition. */
void move(struct icon_type *icon)
{
  int x, y, oldx, oldy;
  char ch;

  clr_eol(0, 23);
  gotoxy(0, 23);
  printf("Arrow keys to move, F2 to accept, F1 to abort");
  x = oldx = icon->x;
  y = oldy = icon->y;
  do {
    /* outline the icon being moved */
    box(x, y, x+XDIM-1, y+YDIM-1, 0x18);
    ch = getkey();
    box(x, y, x+XDIM-1, y+YDIM-1, 0x18);
    switch(ch) {
      case 75: /* left */
        x -=5;
        break;
      case 77: /* right */
        x +=5;
        break;
      case 72: /* up */
        y -= 5;
        break;
      case 80: /* down */
        y +=5;
        break;
      case 71: /* left-up */
        y -= 5;  x -= 5;
        break;
      case 73: /* right-up */
        y -= 5;  x += 5;
        break;
      case 79: /* left-down */
        y += 5;  x -= 5;
        break;
      case 81: /* right-down */
        y += 5;  x += 5;
        break;
      case 59: return;  /* F1: abort */
    }
  } while(ch!=60);  /* F2: accept */
  icon->x = x;
  icon->y = y;

  /* erase old position */
  for(x=oldx; x<oldx+XDIM; x++)
    for(y=oldy; y<oldy+YDIM; y++)
      putpoint(x, y, 0, 0);

}

/* Load the icons. */
load_icons()
{
  FILE *fp;
  register int i;

  for(i=0; (i<NUM_ICONS) && *icon_files[i]; i++) {
    if((fp = fopen(icon_files[i], "rb"))==NULL) {
      printf("cannot open file %s\n", icon_files[i]);
      return 0;
    }

    fread(&icons[i], sizeof (struct icon_type), 1, fp);

    if(ferror(fp)) break;
    fclose(fp);
  }

  return i;
}

/* Draw a box in the foreground color. The
   how parameter determines how the box will
   be displayed on the screen, see putpoint()
   for details. 
*/
void box(int startx, int starty,
         int endx, int endy, int how)
{
  register int x, y;

  for(x=startx; x<endx; x++)
    putpoint(x, endy, FORE_GND, how);
  for(y=starty; y<endy; y++)
    putpoint(endx, y, FORE_GND, how);
  for(x=startx; x<endx; x++)
    putpoint(x, starty, FORE_GND, how);
  for(y=starty; y<endy; y++)
    putpoint(startx, y, FORE_GND, how);
}
  
/* List the current directory. */
void dir(void)
{
  char com[80], str[80];

  strcpy(com, "dir ");
  gotoxy(0, 10);
  printf("enter mask: ");
  gets(str);
  strcat(com, str);
  system(com);
}

/* Copy a file. */
void copy(void)
{
  char fname1[80], fname2[80];
  char com[80];

  strcpy(com, "copy ");
  gotoxy(0, 10);

  printf("From: ");
  gets(fname1);
  printf("To: ");
  gets(fname2);   
  strcat(com, fname1);
  strcat(com, " ");
  strcat(com, fname2);
  system(com);
}

/* Erase a file. */
void erase(void)
{
  char fname[80];
  char com[80];

  gotoxy(0, 10);

  strcpy(com, "erase ");

  printf("File to erase: ");
  gets(fname);
  strcat(com, fname);
  printf("Sure? (y/n): ");
  if(tolower(getche())=='y') system(com);
}

/* Run a program. */
void run(void)
{
  char prog[80];

  gotoxy(0, 10);
  
  printf("enter program name: ");
  gets(prog);
  spawnl(P_WAIT, prog, prog, "");
}

/* Type a file. */
void type(void)
{
  char fname[80];
  char com[80];

  gotoxy(0, 10);

  strcpy(com, "type ");

  printf("File to type: ");
  gets(fname);
  strcat(com, fname);
  system(com);
}

/* Check the disk. */
void chkdsk(void)
{
  char com[80], str[80];

  strcpy(com, "chkdsk ");

  gotoxy(0, 10);

  printf("enter drive specifier: ");
  gets(str);
  strcat(com, str);
  system(com);
}

/* Display version number. */
void ver(void)
{
  gotoxy(0, 10);
  system("ver");
}

/* Return to the command line. */
void icon_end(void)
{
  mode(2);
  exit(0);
}
  
/* Return the scan code of the key pressed. 
   This function is compatible with Turbo/Borland C/C++ and
   Microsoft C/C++ (be sure to #define the appropriate
   symbol).  If you use a different compiler, consult
   the text.
*/
char getkey(void)
{
  union key {
    int i;
    char ch[2];
  } k;

#ifdef TURBOC
  k.i = bioskey(0);
#endif
#ifdef MICROSOFTC
  k.i = _bios_keybrd(0);
#endif
  return k.ch[1];
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* send the cursor to x,y */
void gotoxy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Clear to end of specified line. */
void clr_eol(int x, int y)
{
   for(; x<80; x++) {
     gotoxy(x, y);
     printf(" ");
   }
}

/* Clear the screen. */
void clrscr(void)
{  
  union REGS r;

  r.h.ah = 7;
  r.h.al = 0;
  r.h.ch = 0;
  r.h.cl = 0;
  r.h.dh = 24;
  r.h.dl = 79;
  r.h.bh = 0;
  int86(0x10, &r, &r);
}

/* This version of putpoint will work for all video
   adapters but is incredibly slow!
*/
/*
void putpoint(int x, int y, int color, int how)
{
  union REGS r;

  if(how==0x18) color = color | 128;
  r.h.bh = 0;
  r.h.ah = 12;
  r.h.al = color;
  r.x.dx = y;
  r.x.cx = x;
  int86(16, &r, &r);
}
*/

/* This function sets the specified pixel to the 
   specified color using an EGA/VGA video adapter
   for modes 13 and greater.

   The value of how may be one of these:

   action        value

   overwrite       0
   XOR             0x18
   AND             8
   OR              0x10
   
*/
#define ENABLE 0x0F
#define INDEXREG 0x3CE
#define VALREG 0x3CF

#define OUTINDEX(index, val) outp((INDEXREG), (index));\
                             outp((VALREG), (val));

#define WIDTH 80L

/* These range values assume EGA mode 16 */
#define XMAX 639
#define YMAX 349
#define XMIN 0
#define YMIN 0

void putpoint(int x, int y, int color, int how)
{
  register unsigned char mask = 0x80;
  register char far *base;
  unsigned dummy;

  /* if you want range checking at this point, activate
     this line of code.  It will slow down the function
     however.
  */
  /*if(x<XMIN || x>XMAX || y<YMIN || y>YMAX) return;*/

  base = (char far *) (egabase 
         + ( (long) y * WIDTH + (long) x/8L ));

  mask >>= x % 8;

  /* This causes the memory READ necessary to
     load the EGA/VGA's internal registers.
  */
  dummy = *base;

  OUTINDEX(0, color);
  OUTINDEX(1, ENABLE);
  OUTINDEX(3, how);
  OUTINDEX(8, mask);

  *base = 1;

  OUTINDEX(0, 0);
  OUTINDEX(1, 0);
  OUTINDEX(3, 0); 
  OUTINDEX(8, 0xff);
}
