/*
    init.c

    The initialization and termination routines

*/

#include "global.h"

//
// local data
//

static LPFNNOTIFYCALLBACK pfnNotify;


//
// Initialise the first instance
//

BOOL InitFirstInstance(HANDLE hInstance)
{
    WNDCLASS wc;

    
    //
    // define the class of window we want to register
    //

    wc.lpszClassName    = szAppName;
    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon            = LoadIcon(hInstance,MAKEINTRESOURCE(IDI_APP));
    wc.lpszMenuName     = "Menu";
    wc.hbrBackground    = GetStockObject(WHITE_BRUSH);
    wc.hInstance        = hInstance;
    wc.lpfnWndProc      = MainWndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    
    if (!RegisterClass(&wc)) {
        return FALSE;
    }

}

//
// Init the current instance
//

BOOL InitCurrentInstance(HANDLE hInstance, LPSTR lpszCmdLine, int cmdShow)
{
    HDC hDC;

    //
    // Get any system metrics we use a lot
    //

    hDC = GetDC(NULL);  // get a DC
    GetTextMetrics(hDC, &tmSysFont);
    ReleaseDC(NULL, hDC);

    //
    // Create any GDI objects we use a lot
    //





    //
    // Create the main window
    //

    hwndMain = CreateWindow(szAppName,
                            szAppName,
                            WS_OVERLAPPEDWINDOW,
                            GetSystemMetrics(SM_CXSCREEN) / 3,
                            GetSystemMetrics(SM_CYSCREEN) * 3 / 4,
                            GetSystemMetrics(SM_CXSCREEN) * 2 / 3,
                            GetSystemMetrics(SM_CYSCREEN) / 4,
                            (HWND)NULL,
                            (HMENU)NULL,
                            hInstance,
                            (LPSTR)NULL
                            );
    
    if (!hwndMain) {
        return FALSE;
    }

    ShowWindow(hwndMain, cmdShow);
    UpdateWindow(hwndMain);

    //
    // Load the accelerator table
    //

    hAccTable = LoadAccelerators(hInstance, "AccTable");
    if (!hAccTable) {
        return FALSE;
    }

    //
    // Use Toolhelp to hook into the RIP info
    // for all current tasks
    //

    Status("RIP Watch started");

    HookAll(hInstance);

    return TRUE;
}

//
// Close down and tidy up
//

void Terminate()
{

    //
    // Unhook
    //

    UnhookAll();

}

//
// Hook all tasks, except our own
//

void HookAll(HINSTANCE hInstance)
{
    TASKENTRY t;

    UnhookAll();

    pfnNotify = (LPFNNOTIFYCALLBACK) MakeProcInstance((FARPROC) NotifyProc,
                                                      hInstance);
    t.dwSize = sizeof(t);
    if (!TaskFirst(&t)) {
        Status("No tasks to hook");
        return;
    }

    do {
        if (t.hTask != GetCurrentTask()) {
            if (NotifyRegister(t.hTask, pfnNotify, NF_NORMAL | NF_RIP)) {
//              Status("Hooked task %4.4XH", t.hTask);
            } else {
                Status("Failed to hook task %4.4XH", t.hTask);
            }
        }
    } while(TaskNext(&t));
}

//
// Unhook all tasks
//

void UnhookAll()
{
    TASKENTRY t;

    t.dwSize = sizeof(t);
    if (TaskFirst(&t)) {
        do {
            NotifyUnRegister(t.hTask);
        } while(TaskNext(&t));
    }

    if (pfnNotify) {
        FreeProcInstance((FARPROC)pfnNotify);
        pfnNotify = NULL;
    }
}
