/**********************************************************************
** Program:    Memory Manager
** File:       memutil.c helper routines for Memory Manager
** Model:      medium
** Windows:    3.0
** Functions:  WalkFree     shows heap- and freenodes in 2 listboxes
**             lmemfill     fills a piece of memory with a pattern
**             Shake        fragments the heap
**             DrawFree     draws freenodes and heapnodes
**             ShowFreeList fills a comboboc with the freenodes
**             ViewHeap     fills a listbox with a memorydump
****************************************************************************/
#pragma message  ("Modul: "__FILE__"  Version 1.0, Stand:  "__TIMESTAMP__)
/******* Includes ********/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "mem.h"
#include "memrc.h"
#include "memman.h"

/***** Handles for main window and instance */
extern HWND      hMain;
extern HANDLE    hInst;

/***WalkHeap****************************************************************/
#pragma message   ("Compiling:         WalkHeap")
void WalkHeap (hDlg, nIDFree, nIDHeap, nIDStatus)
HWND         hDlg;
int          nIDFree,
             nIDHeap,
             nIDStatus;
{
   char         szLine[MAXSTRLEN],
                szBuf[MAXSTRLEN];
   int          i;
   long         j;
   LPSTR        lp;

   for (i=0;i<nFreeUsed;i++)
   {
      if (FreeNodes[i].lSize > 0L)
      {
         sprintf (szLine, "%07ld\t%5ld\t%Fp", FreeNodes[i].lOffset,
                  FreeNodes[i].lSize,
                  (LPSTR) GetPointer (FreeNodes[i].lOffset));
         SendDlgItemMessage (hDlg, nIDFree, LB_ADDSTRING, 0, (LONG) (LPSTR) szLine);
      }
   }
   for (i=0;i<nHeapUsed;i++)
   {
      if (HeapNodes[i].lSize > 0L)
      {
         lp = GetPointer (HeapNodes[i].lOffset);
         for (j=0;j<HeapNodes[i].lSize && j<50;j++)
            szBuf[j] = lp[j];
         szBuf[j] = (char) 0;
         sprintf (szLine, "%07ld\t%5ld\t%Fp\t%s", HeapNodes[i].lOffset,
         HeapNodes[i].lSize, lp, szBuf);
         SendDlgItemMessage (hDlg, nIDHeap, LB_ADDSTRING, 0, (LONG) (LPSTR) szLine);
      }
   }
   sprintf (szLine, "HeapHandle:%d BasePointer:%Fp UsedFreeNodes:%u UsedHeapNodes:%u",
                     hHeap, (LPSTR) hpBase, nFreeUsed, nHeapUsed);
   SetDlgItemText (hDlg, nIDStatus, szLine);
   return;
}
/***lmemfill**************************************************************/
#pragma message   ("Compiling:         lmemfill")
void lmemfill (lp, cb, lSize)
LPSTR    lp;
char     cb;
long     lSize;
{
   long i;
   for (i=0L;i<lSize;i++)
      lp[i] = cb;
   return;
}
/***Shake*****************************************************************/
#pragma message   ("Compiling:         Shake")
void Shake (hDlg, nIDStatus, lRuns)
HWND hDlg;
int  nIDStatus;
long lRuns;
{
   LPSTR         lp[100];
   int           i, s;
   long          l;
   char          szLine[MAXSTRLEN];
   long          lErrors;
   long          lOffset;

/* Init */
   srand (1);
   lErrors = 0L;

   for (l=0L;l<lRuns;l++)
   {
      for (i=0;i<100;i++)
      {
         s = rand ();
         s = s % 512;     /* keine Objekte > 512 Byte */
         s = max (s, 90); /* die Zeile mu reinpassen */
         lp[i] = AllocMem (s);
         lOffset = GetOffset (lp[i]);
         sprintf (szLine, "Run:%ld Index:%d Offset:%ld Size:%d Pointer:%Fp Clock:%lu",
         l, i, lOffset, s, lp[i], GetTickCount ());
         SetDlgItemText (hDlg, nIDStatus, szLine);
         if (lp[i] == NULL)
            lErrors++;
         else
            lstrcpy (lp[i], szLine);
      }
      for (i=0;i<100;i++)
      {
         lOffset = GetOffset (lp[i]);
         sprintf (szLine, "Free: Run:%ld Index:%d Offset:%ld Pointer:%Fp",
         l, i, lOffset, lp[i]);
         s = FreeMem (lp[i]);
         if (s < 0)
            sprintf (szLine, "Free: Error:%d Run:%ld Index:%d Offset:%ld Pointer:%Fp",
            s, l, i, lOffset, lp[i]);
         SetDlgItemText (hDlg, nIDStatus, szLine);
      }
   }
   for (i=0;i<100;i++)
   {
      s = rand ();
      s = s % 512;     /* keine Objekte > 512 Byte */
      s = max (s, 90); /* die Zeile mu reinpassen */
      lp[i] = AllocMem (s);
      lOffset = GetOffset (lp[i]);
      sprintf (szLine, "Run:%ld Index:%d Offset:%ld Size:%d Pointer:%Fp Clock:%lu",
      l, i, lOffset, s, lp[i], GetTickCount ());
      SetDlgItemText (hDlg, nIDStatus, szLine);
      if (lp[i] == NULL)
         lErrors++;
      else
         lstrcpy (lp[i], szLine);
   }
   sprintf (szLine, "%ld Allocs failed!", lErrors);
   SetDlgItemText (hDlg, nIDStatus, szLine);
}
/***DrawHeap**************************************************************/
#pragma message   ("Compiling:         DrawHeap")
void DrawHeap (hWnd, hDC)
HWND hWnd;
HDC  hDC;
{
   char           szLine[MAXSTRLEN];
   long           lScrollPos,
                  lStart,
                  lEnd;
   RECT           rcClient;
   int            i;
   int            x1, y1, x2, y2, x0, y0, y3;
   HBRUSH         hOldBrush, hRedBrush, hGreenBrush;
   HFONT          hFont, hOldFont;

   hRedBrush = CreateSolidBrush (RGB (255, 0, 0));
   hGreenBrush = CreateSolidBrush (RGB (0, 255, 0));
   hFont = CreateFont (8, 0, 0, 0, 700, FALSE, FALSE, FALSE,
                       0, 0, 0, 0, 0, "MS Sans Serif");
   hOldFont = SelectObject (hDC, hFont);

   GetClientRect (hWnd, &rcClient);
   y0 = rcClient.bottom / 2;
   x0 = rcClient.right / 2;

   lScrollPos = (long) GetScrollPos (hWnd, SB_HORZ);
   lStart = lScrollPos * 8L;
   lEnd = (lScrollPos + (long) rcClient.right) * 8L;
   wsprintf (szLine, "Heap from Offset: %ld to %ld", lStart, lEnd);
   TextOut (hDC, x0, 20, szLine, lstrlen (szLine));
   wsprintf (szLine, "List of free nodes:");
   TextOut (hDC, 0, y0-100, szLine, lstrlen (szLine));
   MoveTo (hDC, 0, y0-20);
   LineTo (hDC, rcClient.right, y0-20);
   MoveTo (hDC, 0, y0);
   LineTo (hDC, rcClient.right, y0);
   MoveTo (hDC, 0, y0+20);
   LineTo (hDC, rcClient.right, y0+20);
   wsprintf (szLine, "Heaplist:");
   TextOut (hDC, 0, y0+100, szLine, lstrlen (szLine));

   hOldBrush = SelectObject (hDC, hGreenBrush);
   y1 = y0-20;
   y2 = y0;
   y3 = y0-45;
   for (i=0;i<nFreeUsed;i++)
   {
      if (FreeNodes[i].lSize > 0L &&
          FreeNodes[i].lOffset + FreeNodes[i].lSize > lStart &&
          FreeNodes[i].lOffset < lEnd)
       {
          x1 = ((FreeNodes[i].lOffset - lStart) / 8L);
          x2 = ((FreeNodes[i].lOffset + FreeNodes[i].lSize - lStart) / 8L);
          if (x1 < 0) x1 = 0;
          if (x2 < 0 || x2 >rcClient.right) x2 = rcClient.right;
          Rectangle (hDC, x1, y1, x2, y2);
          wsprintf (szLine, "%ld", FreeNodes[i].lOffset);
          TextOut (hDC, x1, y3, szLine, lstrlen (szLine));
          MoveTo (hDC, x1, y0);
          LineTo (hDC, x1, y3);
          y3-=20;
          if (y3==y0-105) y3=y0-45;
       }
   }

   SelectObject (hDC, hRedBrush);
   y1 = y0;
   y2 = y0+20;
   y3 = y0+25;
   for (i=0;i<nHeapUsed;i++)
   {
      if (HeapNodes[i].lSize > 0L &&
          HeapNodes[i].lOffset + HeapNodes[i].lSize > lStart &&
          HeapNodes[i].lOffset < lEnd)
       {
          x1 = ((HeapNodes[i].lOffset - lStart) / 8L);
          x2 = ((HeapNodes[i].lOffset + HeapNodes[i].lSize - lStart) / 8L);
          if (x1 < 0) x1 = 0;
          if (x2 < 0 || x2 >rcClient.right) x2 = rcClient.right;
          Rectangle (hDC, x1, y1, x2, y2);
          wsprintf (szLine, "%ld", HeapNodes[i].lOffset);
          TextOut (hDC, x1, y3, szLine, lstrlen (szLine));
          MoveTo (hDC, x1, y0);
          LineTo (hDC, x1, y3);
          y3 += 20;
          if (y3==y0+85) y3=y0+25;
       }
   }
   SelectObject (hDC, hOldFont);
   if (hFont != NULL)
      DeleteObject (hFont);
   SelectObject (hDC, hOldBrush);
   if (hRedBrush != NULL)
      DeleteObject (hRedBrush);
   if (hGreenBrush != NULL)
      DeleteObject (hGreenBrush);
   return;
}
/***ShowHeapList**********************************************************/
#pragma message   ("Compiling:         ShowHeapList")
void ShowHeapList (hDlg, nIDList)
HWND hDlg;
int  nIDList;
{
   int             i;
   long            j;
   char            szLine[MAXSTRLEN],
                   szValue[MAXSTRLEN];
   LPSTR           lp;
   long            lOffset;

   for (i=0;i<nHeapUsed;i++)
   {
      if (HeapNodes[i].lSize > 0L)
      {
         lp = GetPointer (HeapNodes[i].lOffset);
         for (j=0;j<HeapNodes[i].lSize && j<50;j++)
            szValue[j] = lp[j];
         szValue[j] = NULL;
         sprintf (szLine, "%ld, %ld, %Fp, %s", HeapNodes[i].lOffset,
                  HeapNodes[i].lSize, lp, szValue);
         SendDlgItemMessage (hDlg, nIDList, CB_ADDSTRING, 0, (LONG) (LPSTR) szLine);
      }
   }
   return;
}
/***ViewHeap**************************************************************/
#pragma message   ("Compiling:         ViewHeap")
void ViewHeap (hDlg, nIDList, lMouse)
HWND hDlg;
int  nIDList;
long lMouse;
{
   int          i;
   long         j;
   char         szLine[MAXSTRLEN];
   LPSTR        lp;

   for (i=0;i<nHeapUsed;i++)
   {
      if (HeapNodes[i].lSize > 0L &&
          HeapNodes[i].lOffset  <= lMouse &&
          HeapNodes[i].lOffset+HeapNodes[i].lSize > lMouse)
      {
         lp = GetPointer (HeapNodes[i].lOffset);
         sprintf (szLine, "%07ld\t%ld\t%Fp in Heap Index=%d", HeapNodes[i].lOffset,
                  HeapNodes[i].lSize, lp, i);
         SendDlgItemMessage (hDlg, nIDList, LB_ADDSTRING, 0, (LONG) (LPSTR) szLine);
         for (j=0;j<HeapNodes[i].lSize+1L;j=j+8)
         {
            sprintf (szLine, "%05ld\t%2x %2x %2x %2x %2x %2x %2x %2x\t%c%c%c%c%c%c%c%c", j,
               lp[j], lp[j+1], lp[j+2], lp[j+3], lp[j+4], lp[j+5], lp[j+6], lp[j+7],
               lp[j], lp[j+1], lp[j+2], lp[j+3], lp[j+4], lp[j+5], lp[j+6], lp[j+7]);
            SendDlgItemMessage (hDlg, nIDList, LB_ADDSTRING, 0, (LONG) (LPSTR) szLine);
         }
         return;
      }
   }
   for (i=0;i<nFreeUsed;i++)
   {
      if (FreeNodes[i].lSize > 0L &&
          FreeNodes[i].lOffset  <= lMouse &&
          FreeNodes[i].lOffset+FreeNodes[i].lSize > lMouse)
      {
         lp = GetPointer (FreeNodes[i].lOffset);
         sprintf (szLine, "%07ld\t%ld\t%Fp in FreeNodes Index=%d", FreeNodes[i].lOffset,
                  FreeNodes[i].lSize, lp, i);
         SendDlgItemMessage (hDlg, nIDList, LB_ADDSTRING, 0, (LONG) (LPSTR) szLine);
         for (j=0;j<FreeNodes[i].lSize+1L;j=j+8)
         {
            sprintf (szLine, "%05ld\t%2x %2x %2x %2x %2x %2x %2x %2x\t%c%c%c%c%c%c%c%c", j,
               lp[j], lp[j+1], lp[j+2], lp[j+3], lp[j+4], lp[j+5], lp[j+6], lp[j+7],
               lp[j], lp[j+1], lp[j+2], lp[j+3], lp[j+4], lp[j+5], lp[j+6], lp[j+7]);
            SendDlgItemMessage (hDlg, nIDList, LB_ADDSTRING, 0, (LONG) (LPSTR) szLine);
         }
         return;
      }
   }
   return;
}