

/* Interfacing to the Microsoft/IBM mouse. */

#include "dos.h"
#include "stdio.h"
#include "stdlib.h"

#define NOT_MOVED 0
#define RIGHT     1
#define LEFT      2
#define UP        3
#define DOWN      4

void mouse(int *ax, int *bx, int *cx, int *dx);
void set_mouse_position(int x, int y);
void cursor_off(void), cursor_on(void);
void mouse_position(int *x, int *y);
void mouse_motion(char *deltax, char *deltay);
void mouse_reset(void), read_mouse(void);
int leftb_pressed(void), rightb_pressed(void);

void goto_xy(int x, int y);
void mode(int mode_code);

main(int argc, char *argv[])
{
  char deltax, deltay;
  int x, y;

  if(argc!=2) {
    printf("usage:moustest <video mode>");
    exit(1);
  }

  mode(atoi(argv[1]));

  mouse_reset(); /* initialize the mouse */
  cursor_on(); /* turn on the cursor */

  do {
    goto_xy(0, 0);
    if(leftb_pressed()) printf("left button ");
    if(rightb_pressed()) printf("right button");

    /* show mouse location */
    mouse_position(&x, &y);
    goto_xy(0, 2);      
    printf(" %d %d - ", x, y);
    goto_xy(0, 0);

    /* see if change in position */
    mouse_motion(&deltax, &deltay);
    if(deltax || deltay) {
      printf("X Direction: ");
      switch(deltax) {
        case NOT_MOVED: printf("      ");
          break;
        case RIGHT: printf("right ");
          break;
        case LEFT: printf("left  ");
          break;
      }
      printf("Y Direction: ");
      switch(deltay) {
        case NOT_MOVED: printf("      ");
          break;
        case UP: printf("up    ");
          break;
        case DOWN: printf("down  ");
          break;
      }
    }
  /* loop until both buttons are pressed at the same time */
  } while(!(leftb_pressed() && rightb_pressed()));
  mode(3);

  return 0;
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Send the cursor to the specified X,Y position. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/************************************************/
/* Mouse interface functons.                    */
/************************************************/

/* Turn off the mouse cursor. */
void cursor_off(void)
{
  union REGS r;

  r.x.ax = 2; /* remove the cursor */
  int86(0x33, &r, &r);
}

/* Turn on the mouse cursor. */
void cursor_on(void)
{
  union REGS r;

  r.x.ax = 1; /* show the cursor */
  int86(0x33, &r, &r);
}

/* Return true if right button is pressed,
   false otherwise. */
rightb_pressed(void)
{
  union REGS r;

  r.x.ax = 3; /* get position and button status */
  int86(0x33, &r, &r);
  return r.x.bx & 2;
}

/* Return true if left button is pressed,
   false otherwise. */
leftb_pressed(void)
{
  union REGS r;

  r.x.ax = 3; /* get position and button status */
  int86(0x33, &r, &r);
  return r.x.bx & 1;
}

/* Set mouse cursor coordinates. */
void set_mouse_position(int x, int y)
{
  union REGS r;

  r.x.ax = 4; /* set position */
  r.x.cx = x;
  r.x.dx = y;
  int86(0x33, &r, &r);
}  

/* Return mouse cursor coordinates. */
void mouse_position(int *x, int *y)
{
  union REGS r;

  r.x.ax = 3; /* get position and button status */
  int86(0x33, &r, &r);
  *x = r.x.cx;
  *y = r.x.dx;
}  

/* Return the direction of travel. */
void mouse_motion(char *deltax, char *deltay)
{
  union REGS r;

  r.x.ax = 11; /* get direction of motion */
  int86(0x33, &r, &r);

  if(r.x.cx>0) *deltax = RIGHT;
  else if(r.x.cx<0) *deltax = LEFT;
  else *deltax = NOT_MOVED;

  if(r.x.dx>0) *deltay = DOWN;
  else if(r.x.dx<0) *deltay = UP;
  else *deltay = NOT_MOVED;
}

/* Reset the mouse. */
void mouse_reset(void)
{
  union REGS r;

  r.x.ax = 0; /* reset the mouse */
  int86(0x33, &r, &r);
  if((int) r.x.ax != -1) {
    printf("mouse hardware or software not installed");
    exit(1);
  }
  if(r.x.bx != 2) {
   printf("two-button mouse required");
   exit(1);
  }
}


