/* obj_gen.c  <11-25-90> <6-29-92>  TGR */

 /***************************************************************************** 
 *
 * $Id$
 *
 * $Log:	obj_gen.c,v $
 * Revision 1.1  93/01/09  11:00:34  11:00:34  rossin (Ted Rossin)
 * Initial revision
 * 
 *****************************************************************************/

/*************************************************************************
**									**
**	obj_gen.c							**
**									**
**	This file is trying to take the data structures produced by the **
**	assembler and write them out to the object file in COFF (Common **
**	Object File Format).  Refer to the TMS320C30 Assembly Language  **
**	Tools user's guide for details (Appendix A) from TI		**
**									**
*************************************************************************/

#include "port.h"
#include <stdio.h>
#ifdef MAC
#include <string.h>
#endif 
#include "as30.h"

/* #define DEBUG	1 */

#define F_RELFLG	0x0001
#define F_EXEC		0x0002
#define F_LNNO		0x0004
#define F_LSYMS		0x0010
#define F_AR32WR	0x0040

#define STYP_REG	0x0000
#define STYP_DSECT	0x0001
#define STYP_NOLOAD	0x0002
#define STYP_GROUP	0x0004
#define STYP_PAD	0x0008
#define STYP_COPY	0x0010
#define STYP_TEXT	0x0020
#define STYP_DATA	0x0040
#define STYP_BSS	0x0080
#define STYP_ALIGN	0x0100

#define T_NULL	0
#define C_NULL	0

extern char *sym,*op,*parms;
extern char *sym2,*op2,*parms2;

extern char *global_list[MAX_SYM];
extern Sym_tab *sym_tab;
extern Image *image;
extern Sec_loc_cnt *sec_loc_cnt;
extern Rel_tab *rel_tab;

extern int current_section;
extern Image *img_ptr;
extern long image_count;
extern int rel_tab_ptr;
extern int sym_ptr;
extern int obj_sym_tab_ptr;


static struct{
    unsigned short  magic;
    unsigned short  num_sec;
    long	    time;
    long	    sym_tab_start;
    long	    sym_tab_len;
    unsigned short  opt_head_size;
    unsigned short  flags;
} file_header;

static struct{
    char	    name[8];
    long	    physical_address;
    long	    virtual_address;
    long	    size;
    long	    raw_data_pointer;
    long	    relocation_pointer;
    long	    line_num_pointer;
    unsigned short  num_relocation_entries;
    unsigned short  num_line_num_entries;
    unsigned short  flags;
    char	    reserved;
    char	    memory_page_number;
} section_header;

static struct{
    unsigned long   address;
    unsigned short  sym_index;
    unsigned short  reserved;
    unsigned short  type;
} rel_entry;


static struct{
    unsigned long   offset_mode;
    unsigned long   string_offset;
    long	    value;
    short	    section;
    unsigned short  type;
    unsigned char   class;
    unsigned char   aux_entries;
} sym_tab_entry;

generate_object_file(fname)
char *fname;
{
    int i,j,index;
    char s[100];
    FILE *obj_fp;
    unsigned long sym_tab_start,string_tab_start,string_tab_size=0L;
    unsigned long str_fptr,fptr2,sym_tab_size=0L;
    unsigned short num_sections=0,num_rel_entries=0;
    Image *ptr;
    unsigned long image_index;

    basename(fname,s);
    strcat(s,".obj");
    if( (obj_fp=fopen(s,"wb")) == NULL){
	printf("Can't open object file %s for write\n",s);
	exit(1);
    }

	/* Write out file header */

    file_header.magic=0x0093;
    file_header.num_sec=2;
    file_header.time=0L;
    file_header.sym_tab_start=0L;
    file_header.sym_tab_len=0;
    file_header.opt_head_size=0;
    file_header.flags=(F_LNNO|F_LSYMS);
    fwrite(&file_header,sizeof(file_header),1,obj_fp);

	/* Write out section headers */

    for(i=0;i<MAX_SECTIONS;i++){
	if(sec_loc_cnt[i].name[0]!=0){
	    for(index=0;index<8;section_header.name[index++]=0);
	    strcpy(section_header.name,sec_loc_cnt[i].name);
	    section_header.physical_address=0L;
	    section_header.virtual_address=0L;
	    section_header.size=sec_loc_cnt[i].lc;
	    section_header.raw_data_pointer=0L;
    	    section_header.relocation_pointer=0L;
    	    section_header.line_num_pointer=0L;
    	    section_header.num_relocation_entries=0;
            section_header.num_line_num_entries=0;
	    section_header.flags = (sec_loc_cnt[i].loadable)?STYP_REG:STYP_BSS;
    	    section_header.reserved=0;
    	    section_header.memory_page_number=0;
            fwrite(&section_header,sizeof(section_header),1,obj_fp);
	    num_sections++;
	}
	else break;
    }

	/* Write out raw data */

    for(i=0;i<MAX_SECTIONS;i++){
	if(sec_loc_cnt[i].name[0]!=0){
	  if(sec_loc_cnt[i].loadable){  /* only check if loadable section */
	    str_fptr=ftell(obj_fp);
	    fptr2=(unsigned long)(sizeof(file_header)+i*sizeof(section_header)
			+20);
	    fseek(obj_fp,fptr2,0);
       	    fwrite(&str_fptr,sizeof(str_fptr),1,obj_fp);
	    fseek(obj_fp,str_fptr,0); /* Get back to the end of the file */
	    ptr=image;
	    for(image_index=0;image_index<image_count;image_index++){
		if(ptr->section == i){	/* Write out data */
            	    fwrite(&(ptr->data),sizeof(image->data),1,obj_fp);
		}
		ptr++;
	    }	    
	  }
	}
	else break;
    }

	/* Write out relocation data */

    for(i=0;i<MAX_SECTIONS;i++){
	if(sec_loc_cnt[i].name[0]!=0){
	  if(sec_loc_cnt[i].loadable){  /* only check if loadable section */
	    str_fptr=ftell(obj_fp);
	    fptr2=(unsigned long)(sizeof(file_header)+i*sizeof(section_header)
			+24);
	    fseek(obj_fp,fptr2,0);
       	    fwrite(&str_fptr,sizeof(str_fptr),1,obj_fp);
	    fseek(obj_fp,str_fptr,0); /* Get back to the end of the file */
    	    for(j=0;j<rel_tab_ptr;j++){
		if(rel_tab[j].section==i && rel_tab[j].flags!=R_RESOLVED){
		    rel_entry.address=rel_tab[j].offset;
			/* symbol index was written into image offset */
		    rel_entry.sym_index=(unsigned int)rel_tab[j].image_offset;
		    rel_entry.type=rel_tab[j].flags;
            	    fwrite(&rel_entry,sizeof(rel_entry),1,obj_fp);
		    num_rel_entries++;
		}
	    }
	    fptr2=(unsigned long)(sizeof(file_header)+i*sizeof(section_header)
			+32);
	    fseek(obj_fp,fptr2,0);
       	    fwrite(&num_rel_entries,sizeof(num_rel_entries),1,obj_fp);
	    fseek(obj_fp,0L,2); /* Get back to the end of the file */
	    num_rel_entries=0;  
	  }
	}
    }

	/* Write out Symbol Table */

	/* Write out dummy data to symbol table block so that random 
	   access can be performed into symbol table block. */

    sym_tab_start=ftell(obj_fp);
    fwrite(image,sizeof(sym_tab_entry),obj_sym_tab_ptr,obj_fp);
    string_tab_start=ftell(obj_fp);
	/* leave space for string table size */
    fwrite(&string_tab_size,sizeof(string_tab_size),1,obj_fp);

    sym_tab_entry.offset_mode=0L;  /* Always use string table */
    sym_tab_entry.type=T_NULL;	   /* No type */
    sym_tab_entry.class=C_NULL;	   /* No class */
    sym_tab_entry.aux_entries=0;   /* No auxilary entries */
    for(i=0;i<sym_ptr;i++){
	if(sym_tab[i].obj_index >= 0){  /* write out symbol table entry */

		/*  Write out string to string table */

	    fseek(obj_fp,0L,2);  /* get to end of file to write string */
	    str_fptr=ftell(obj_fp);   /* Save pointer to location of string */
	    j=fwrite(sym_tab[i].symbol,sizeof(char),strlen(sym_tab[i].symbol)+1
								,obj_fp);
	    if(j!=strlen(sym_tab[i].symbol)+1){
		printf("Write error on writing object file\n");
		exit(1);
 	    }
	    string_tab_size+=j;
	    
		/* Write symbol table entry */ 

	    sym_tab_entry.string_offset = str_fptr-string_tab_start;
 	    sym_tab_entry.value = sym_tab[i].add; 
 	    sym_tab_entry.section = sym_tab[i].section;
 	    sym_tab_entry.class = (char)sym_tab[i].flags;
	    fseek(obj_fp,sym_tab_start+(long)(sym_tab[i].obj_index
						*sizeof(sym_tab_entry)),0);
	    fwrite(&sym_tab_entry,sizeof(sym_tab_entry),1,obj_fp); 
	    sym_tab_size++;
#ifdef DEBUG
	    printf(
		"Symbol ->%s<- value=0x%08lx  class=%d  section=%d index=%d\n",
		sym_tab[i].symbol,sym_tab_entry.value,sym_tab_entry.class
		,sym_tab_entry.section,sym_tab[i].obj_index);
#endif
	}
	
    }

	/* write out string table size */

    fseek(obj_fp,string_tab_start,0);
    fwrite(&string_tab_size,sizeof(string_tab_size),1,obj_fp);

	/* Fill in missing values in header records */

    str_fptr=2L;
    fseek(obj_fp,str_fptr,0); /* point to sym_tab_start entry in file header */
    fwrite(&num_sections,sizeof(num_sections),1,obj_fp);
    str_fptr=8L;
    fseek(obj_fp,str_fptr,0); /* point to sym_tab_start entry in file header */
    fwrite(&sym_tab_start,sizeof(sym_tab_start),1,obj_fp);
    fwrite(&sym_tab_size,sizeof(sym_tab_size),1,obj_fp);

    fclose(obj_fp);

}

