/* op_info.c  <10-06-90>  TGR */

 /***************************************************************************** 
 *
 * $Id$
 *
 * $Log:	op_info.c,v $
 * Revision 1.1  93/01/09  11:05:38  11:05:38  rossin (Ted Rossin)
 * Initial revision
 * 
 *****************************************************************************/

/*---------------------------------------------------------------------------

	This program is used to create a hash table for the op_code table.
    The table created allows the assembler to use the first letter of the
    opcode to quickly jump to the section of the opcode table which may have
    the symbol being searched.  This all assumes that the opcode table has
    been created so that all opcodes that start will the same letter are 
    grouped together.

	This program is run by the makefile to generate a C source code 
    header file named ophash.h The resulting header file is merly an integer 
    array name op_code_hash of 26 entries with indexes into the opcode table.
    The output also contains an array named op_code_last of 26 entries which
    point to one element past the last element of with the same first letter
    int the opcode table.   The two arrays can be used to determine the search
    range needed to find an opcode.

    The output also defines the constant OP_CODE_ENTRIES which is the number 
    of entries in the opcode table.

-----------------------------------------------------------------------------*/

#include "as30.h"
#include <stdio.h>

#define MAX_OP_CODES	10000

extern Op_table op_table[];

main()
{
    int i,lptr;
    int hash[27],last[26];
    FILE *fp;
    
    if( (fp=fopen("ophash.h","w"))==NULL){
	printf("Can't open ophash.h for write\n");
	exit(1);
    }
    for(i=0;i<26;hash[i++] = -1);
    for(i=0;i<26;last[i++] = -1);
    for(i=0;i<MAX_OP_CODES;i++){	/* Will break out at end of table */
	if(strlen(op_table[i].mnemonic)==0) break;
	if(hash[op_table[i].mnemonic[0]-'a'] == -1)
	    hash[op_table[i].mnemonic[0]-'a']=i;
    }
    hash[26]=i;
    
    for(i=0;i<26;i++){
	if(hash[i] != -1){
	    for(lptr=i+1;lptr<27;lptr++){
	    	if(hash[lptr] != -1){ last[i]=hash[lptr]; break; }
	    }
	}
    }
    fprintf(fp,"#define OP_CODE_ENTRIES\t%d\n",hash[26]);
    fprintf(fp,"\nint op_code_hash[26] = {\n\t\t\t");
    for(i=0;i<25;i++){ 
	if(i==10 || i==20) fprintf(fp,"\n\t\t\t");
	fprintf(fp,"%d,",hash[i]); 
    }
    fprintf(fp,"%d\n\t\t\t};\n",hash[i]);
    fprintf(fp,"\nint op_code_last[26] = {\n\t\t\t");
    for(i=0;i<25;i++){ 
	if(i==10 || i==20) fprintf(fp,"\n\t\t\t");
	fprintf(fp,"%d,",last[i]); 
    }
    fprintf(fp,"%d\n\t\t\t};\n",hash[i]);
    fclose(fp);
}
