
/* A paint program that allows lines, boxes,
   and cirlces to be drawn.  You may define
   an object and rotate it in either a clockwise
   or counterclockwise direction.  Regions of the
   screen can be copied or moved.  Also, the 
   graphic image may be saved to disk and loaded
   at a later date.  
*/
#include "dos.h"
#include "stdio.h"
#include "math.h"
#include "ctype.h"
#include "conio.h"

#define NUM_SIDES 20 /* Number of sides an object may
                        have.  Enlarge as needed. */
#define XMAX 319
#define YMAX 199

void line(int startx, int starty, int endx, int endy, 
         int color);
void box(int startx, int starty, int endx, int endy, 
         int color);
void fill_box(int startx, int starty, int endx, int endy, 
         int color);
void circle(int x_center, int y_center, int radius, int color);
void fill_circle(int x_center, int y_center, int radius, 
                 int color);
void plot_circle(int x, int y, int x_center, int y_center, 
                 int color);
void mempoint(int x, int y, int color);
void palette(int pnum);
void mode(int mode_code);
void rotate_point(double theta, double *x, double *y,
                  int x_org, int y_org);
void display_object(double ob[][5], int lines);
void rotate_object(double ob[][5], double theta,
                   int x, int y, int lines);
int define_object(double ob[][5], int x, int y);
void move(int startx, int starty, int endx, int endy,
          int x, int y);
void copy(int startx, int starty, int endx, int endy, 
          int x, int y);
void xhairs(int x, int y), goto_xy(int x, int y);
void save_pic(void), load_pic(void);
int readkey(void);

/* This array will hold the coordinates of an object 
   that is defined dynamcially.
*/
double object[NUM_SIDES][5];

double asp_ratio; /* holds aspect ratio for circles */

main(void)
{
  union k{
    char c[2];
    int i;
  } key;

  int x = 10, y = 10; /* current screen position */
  int cc = 2; /* current color */
  int on_flag = 1; /* pen on or off */
  int pal_num = 1; /* palette number */

  /* the end points of a defined line, circle, or box */
  int startx = 0, starty = 0, endx = 0, endy = 0, first_point = 1;

  int inc = 1; /* movement increment */
  int  sides = 0; /* number of sides of a defined object */

  mode(4);  /* switch to mode 4 CGA/EGA graphics */
  palette(0); /* palette 0 */

  xhairs(x, y); /* show the crosshairs */
  do {
    key.i = readkey();
    xhairs(x, y);   /* plot the crosshairs */
    if(!key.c[0]) switch(key.c[1]) {
      case 72: /* up */
        if(on_flag) line(x, y, x, y-inc, cc); 
        y -= inc;
        break;
      case 80: /* down */
        if(on_flag) line(x, y, x, y+inc, cc); 
        y += inc;
        break;
      case 75: /* left */
        if(on_flag) line(x, y, x-inc, y, cc); 
        x -= inc;
        break;
      case 77: /* right */
        if(on_flag) line(x, y, x+inc, y, cc); 
        x += inc;
        break;
      case 71: /* up left */
        if(on_flag) line(x, y, x-inc, y-inc, cc); 
        x -= inc; y -= inc;
        break;
      case 73: /* up right */
        if(on_flag) line(x, y, x+inc, y-inc, cc); 
        x += inc; y -= inc;
        break;
      case 79: /* down left */
        if(on_flag) line(x, y, x-inc, y+inc, cc); 
        x -= inc; y += inc;
        break;
      case 81: /* down right */
        if(on_flag) line(x, y, x+inc, y+inc, cc); 
        x += inc; y += inc;
        break;
      case 59: inc = 1;  /* F1 - slow speed */
        break;
      case 60: inc = 5;  /* F2 - fast speed */
        break;
    }
    else switch(tolower(key.c[0])) {
      case 'o': on_flag = !on_flag; /* toggle brush */
        break;
      case '1': cc = 1; /* color 1 */
        break;
      case '2': cc = 2; /* color 2 */
        break;
      case '3': cc = 3; /* color 3 */
        break;
      case '0': cc = 0; /* color 0 */
        break;
      case 'b': box(startx, starty, endx, endy, cc);
        break;
      case 'f': fill_box(startx, starty, endx, endy, cc);
        break;
      case 'l': line(startx, starty, endx, endy, cc);
        break;
      case 'c': circle(startx, starty, endy-starty, cc);
        break;
      case 'h': fill_circle(startx, starty, endy-starty, cc);
        break;
      case 's': save_pic();
        break;
      case 'r': load_pic();
        break;
      case 'm': /* move a region */
        move(startx, starty, endx, endy, x, y);
        break;
      case 'x': /* copy a region */
        copy(startx, starty, endx, endy, x, y);
        break;
      case 'd': /* define an object to rotate */
        sides = define_object(object, x, y);
        break;
      case 'a': /* rotate the object */
        rotate_object(object, 0.05, x, y, sides);
        break;
      case '\r': /* set endpoints for line, circle, or box */
        if(first_point) {
          startx = x, starty = y;
        }
        else {
          endx = x; endy = y;
        }
        first_point = !first_point;
        break;
      case 'p': pal_num = pal_num==1 ? 2:1;
        palette(pal_num);
    }
    xhairs(x, y);
  } while (key.c[0]!='q');
  mode(3);

  return 0;
}
/* Set the palette. */
void palette(int pnum)
{
  union REGS r;

  r.h.bh = 1;   /* code for palette select */
  r.h.bl = pnum;
  r.h.ah = 11;  /* set palette function */
  int86(0x10, &r, &r);
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}


/* Draw a box. */
void box(int startx, int starty, /* upper left corner */
         int endx, int endy, /* lower right corner */
         int color) /* color */
{
  line(startx, starty, endx, starty, color);
  line(startx, starty, startx, endy, color);
  line(startx, endy, endx, endy, color);
  line(endx, starty, endx, endy, color);
}
 

/* Draw a line in specified color
   using Bresenham's integer based algorithm.
*/
void line(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int t, distance;
  int x=0, y=0, delta_x, delta_y;
  int incx, incy;

  /* Compute the distances in both directions. */
  delta_x = endx-startx;
  delta_y = endy-starty;

  /* Compute the direction of the increment.
     An increment of 0 means either a vertical
     or horizontal line.
  */
  if(delta_x>0) incx = 1;
  else if(delta_x==0) incx = 0;
  else incx = -1;

  if(delta_y>0) incy = 1;
  else if(delta_y==0) incy = 0;
  else incy = -1;

  /* determine which distance is greater */
  delta_x = abs(delta_x);
  delta_y = abs(delta_y);
  if(delta_x>delta_y) distance = delta_x;
  else distance = delta_y;

  /* draw the line */
  for(t=0; t<=distance+1; t++) {
    mempoint(startx, starty, color);
    x += delta_x;
    y += delta_y;  
    if(x>distance) {
      x -= distance;
      startx += incx;
    }
    if(y>distance) {
      y -= distance;
      starty += incy;
    }
  }
}

/* Fill box with specified color. */
void fill_box(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int i, begin, end;

  begin = startx<endx ? startx : endx;
  end = startx>endx ? startx : endx;

  for(i=begin; i<=end;i++)
    line(i, starty, i, endy, color);
}

/* Draw a circle using Bresenham's integer based algorithm. */
void circle(int x_center, int y_center, /* center */
            int radius, /* radius */
            int color) /* color */
{
  register int x, y, delta;

  asp_ratio = 1.0;  /* for different aspect ratios, alter 
                   this number */

  y = radius;
  delta = 3 - 2 * radius;

  for(x=0; x<y; ) {
    plot_circle(x, y, x_center, y_center, color);

    if (delta < 0)
      delta += 4*x+6;
    else {
      delta += 4*(x-y)+10;
      y--;
    }
    x++;
  }
  x = y;
  if(y) plot_circle(x, y, x_center, y_center, color);
}

/* Plot_circle actually prints the points that
   define the circle.
*/
void plot_circle(int x, int y, 
                 int x_center, int y_center, 
                 int color)
{
  int startx, endx, x1, starty, endy, y1;

  starty = y*asp_ratio;
  endy = (y+1)*asp_ratio;
  startx = x*asp_ratio;
  endx = (x+1)*asp_ratio;

  for (x1=startx; x1<endx; ++x1)  {
    mempoint(x1+x_center, y+y_center, color);
    mempoint(x1+x_center, y_center-y, color);
    mempoint(x_center-x1, y_center-y, color);
    mempoint(x_center-x1, y+y_center, color);
  }

  for (y1=starty; y1<endy; ++y1) {
    mempoint(y1+x_center, x+y_center, color);
    mempoint(y1+x_center, y_center-x, color);
    mempoint(x_center-y1, y_center-x, color);
    mempoint(x_center-y1, x+y_center, color);
  }
}

/* Fill a circle by repeatedly calling circle()
   with smaller radius.
*/
void fill_circle(int x_center, int y_center,
                 int radius, int color)
{
  while(radius) {
    circle(x_center, y_center, radius, color);
    radius--;
  }
}

/* Write a point directly to mode 4 graphics RAM. */
void mempoint(int x, int y, /* coordinate of point */
              int color) /* color */
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;
  char xor; /* xor color in or overwrite */

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 0xFF3F;  /* 11111111 00111111 in binary */

  /* check range */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return;

  xor = color & 128; /* see if xor mode is set */
  color = color & 127; /* mask off high bit */
 
  /* set bit_mask and color bits to the right location */
  bit_position = x%4;
  color <<= 2*(3-bit_position);
  bit_mask.i >>= 2*bit_position;


  /* find the correct byte in screen memory */
  index = y*40 + (x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* write the color */
  if(!xor) { /* overwrite mode */
    t = *(ptr+index) & bit_mask.c[0];
    *(ptr+index) = t | color;
  }
  else { /* xor mode */
    t = *(ptr+index) | (char) 0;
    *(ptr+index) = t ^ color;
  }
}

/* Display crosshair locator. */
void xhairs(int x, int y)
{
  line(x-4, y, x+3, y, 1 | 128);
  line(x, y+4, x, y-3, 1 | 128);
}


/* Read byte directly from the video RAM in mode 4. */
unsigned char read_point(int x, int y)
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 3;       /* 11111111 00111111 in binary */

  /* check range for mode 4 */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return 0;

  /* set bit_mask and color_code bits to the right location */
  bit_position = x%4;
  bit_mask.i <<= 2*(3-bit_position);
  /* find the correct byte in screen memory */
  index = y*40 +(x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* read the color */
  t = *(ptr+index) & bit_mask.c[0];
  t >>= 2*(3-bit_position);
  return t;
}

/* Save the video graphics display. */
void save_pic(void)
{
  char fname[80];
  FILE *fp;
  register int i, j;
  int e=0;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  char far *temp;
  unsigned char buf[14][80]; /* hold the contents of screen */ 

  temp = ptr;
  /* save the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      buf[i][j] = *temp; /* even byte */
      buf[i][j+1] = *(temp+8152); /* odd byte */
      *temp = 0; *(temp+8152) = 0;  /* clear top of screen */
      temp++;
    }
      
  goto_xy(0, 0);
  printf("Filename: ");
  gets(fname);
  if(!(fp=fopen(fname, "wb"))) {
    goto_xy(0, 0);
    printf("cannot open file - press a key ");
    getch();
    e = 1;  /* flag error */
  }

  temp = ptr;
  /* restore the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      *temp = buf[i][j];
      *(temp+8152) = buf[i][j+1];
      temp++;
    }

 if(e) return; /* if file could not be opened, exit */

 /* save image to file */ 
 for(i=0; i<8152; i++) { 
    putc(*ptr, fp); /* even byte */
    putc(*(ptr+8152), fp); /* odd byte */
    ptr++;
  }
 
  fclose(fp);
}

/* Load the video graphics display. */
void load_pic(void)
{
  char fname[80];
  FILE *fp;
  register int i, j;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  char far *temp;
  unsigned char buf[14][80]; /* hold the contents of screen */ 

  temp = ptr;
  /* save the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      buf[i][j] = *temp;
      buf[i][j+1] = *(temp+8152);
      *temp = 0; *(temp+8152) = 0; /* clear the top of the screen */
      temp++;
    }
      
  goto_xy(0, 0);
  printf("Filename: ");
  gets(fname);
  if(!(fp=fopen(fname, "rb"))) {
    goto_xy(0, 0);
    printf("cannot open file\n");
    temp = ptr;
    /* restore the top of the current screen */
    for(i=0; i<14; i++)
      for(j=0; j<80; j+=2) {
        *temp = buf[i][j];
        *(temp+8152) = buf[i][j+1];
        temp++;
      }
    return;
  }

  /* load image from file */
  for(i=0; i<8152; i++) { 
    *ptr = getc(fp); /* even byte */
    *(ptr+8152) = getc(fp); /* odd byte */
    ptr++;
  }

  fclose(fp);
}

/* Send the cursor to the specified X,Y position. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Move one region to another location. */
void move(int startx, int starty, /* upper left coordinate */
  int endx, int endy, /* lower right coordinate */
  int x, int y) /* upper left of region receiving the image */
{ 
  int i, j;
  unsigned char c;

  for(; startx<=endx; startx++, x++)
    for(i=starty, j=y; i<=endy; i++, j++) {     
      c = read_point(startx, i); /* read point */
      mempoint(startx, i, 0); /* erase old image */
      mempoint(x, j, c); /* write it to new location */
    }
}

/* Copy one region to another location. */
void copy(int startx, int starty, /* upper left coordinate */
  int endx, int endy, /* lower right coordinate */
  int x, int y) /* upper left of region receiving the image */
{ 
  int i, j;
  unsigned char c;

  for(; startx<=endx; startx++, x++)
    for(i=starty, j=y; i<=endy; i++, j++) {     
      c = read_point(startx, i); /* read point */
      mempoint(x, j, c); /* write it to new location */
    }
}


/* Rotate a point around the origin, specified by
   x_org and y_org, by angle theta.  
*/
void rotate_point(double theta, /* angle of rotation */
                  double *x, double *y, /* point to rotate */
                  int x_org, int y_org) /* origin */
{
  double tx, ty;

  /* normalize x and y */
  tx = *x - x_org;  
  ty = *y - y_org;
 
  /* rotate */
  *x = tx * cos(theta) - ty * sin(theta);
  *y = tx * sin(theta) + ty * cos(theta);

  /* return to PC coordinate values */
  *x += x_org;
  *y += y_org;

}

/* Rotate the specified object. */
void rotate_object(double ob[][5], /* object definition */
            double theta, /* angle of rotation in radians */
            int x, int y, /* location of origin */
            int lines) /* number of lines in image */
{
  register int j;
  char ch;

  for(;;) {
    ch = getch(); /* see which direction to rotate */
    switch(tolower(ch)) {
      case 'l':  /* counterclockwise */
        theta = theta < 0 ? -theta : theta;
        break;
      case 'r':  /* clockwise */
        theta = theta > 0 ? -theta : theta;
        break;
      default: return;
    }

    for(j=0; j<lines; j++) {     
      /* erase old line */
      line((int) ob[j][0], (int) ob[j][1],
        (int) ob[j][2], (int) ob[j][3], 0);
 
      rotate_point(theta, &ob[j][0], 
        &ob[j][1], x, y);

      rotate_point(theta, &ob[j][2], 
        &ob[j][3], x, y);

      line((int) ob[j][0], (int) ob[j][1],
        (int) ob[j][2], (int) ob[j][3], (int) ob[j][4]);
    }
  }
}

/* Display an object. */
void display_object(double ob[][5], int lines)
{
  register int i;

  for(i=0; i<lines; i++)
    line((int) ob[i][0], (int) ob[i][1],
      (int) ob[i][2], (int) ob[i][3], (int) ob[i][4]);
}


/* Define an object by specifying its endpoints */
define_object(double ob[][5], int x, int y)
{

  union k{
    char c[2];
    int i;
  } key;
  register int i, j;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  char far *temp;
  unsigned char buf[14][80]; /* hold the contents of screen */ 
  int sides = 0;

  temp = ptr;
  /* save the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      buf[i][j] = *temp;
      buf[i][j+1] = *(temp+8152);
      *temp = 0; *(temp+8152) = 0; /* clear the top of the screen */
      temp++;
    }

  i = 0;      
  xhairs(x, y);
  do {
    goto_xy(0, 0);
    printf("Define side %d,", sides+1);
    if(i==0) printf(" enter first endpoint");
    else printf(" enter second endpoint");

    key.i = readkey();
    xhairs(x, y);   /* plot the crosshairs */
    if(key.c[0]=='\r') { 
      ob[sides][i++] = (double) x;
      ob[sides][i++] = (double) y;
      if(i==4) {  
        ob[sides][4] = read_point(x, y); /* get color */
        i = 0;
        sides++;
      }
    }
 
    /* if arrow key, move the crosshairs */
    if(!key.c[0]) switch(key.c[1]) {
      case 75: /* left */
        x -= 1;
        break;
      case 77: /* right */
        x += 1;
        break;
      case 72: /* up */
        y -= 1;
        break;
      case 80: /* down */
        y += 1;
        break;
      case 71: /* up left */
        x -= 1; y -= 1;
        break;
      case 73: /* up right */
        x += 1; y -= 1;
        break;
      case 79: /* down left*/
        x -= 1; y+=1;
        break;
      case 81: /* down right */
        x += 1; y += 1;
        break;
      }
    if(key.c[1]!=59) xhairs(x, y);
  } while(key.c[1]!=59); /* F1 to stop */
  
  temp = ptr;
  /* restore the top of the current screen */
  for(i=0; i<14; i++)
    for(j=0; j<80; j+=2) {
      *temp = buf[i][j];
      *(temp+8152) = buf[i][j+1];
      temp++;
    }
  return sides;
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}


