/* Pull-down menu routines for text mode operation and
   short sample program.
*/

#include "stdio.h"
#include "dos.h"
#include "stdlib.h"
#include "bios.h"
#include "ctype.h"
#include "string.h"

#define BORDER 1
#define ESC 27
#define MAX_FRAME 10
#define REV_VID 0x70
#define NORM_VID 7

void save_video(int num);
void restore_video(int num);
void goto_xy(int x, int y), cls(void);
void write_video(int x, int y, char *p, int attrib);
void display_menu(int num);
void write_string(int x, int y, char *p, int attrib);
void write_char(int x, int y, char ch, int attrib);
void pd_driver(void);
int make_menu(int num, char *menu[], char *keys,  
              int count, int x, int y, int border);
int get_resp(int num), pulldown(int num);
int video_mode(void);
void display_menu(int num), draw_border(int num);
char far *vid_mem;
int readkey(void);

struct menu_frame {
  int startx, endx, starty, endy;
  unsigned char *p;
  char **menu;
  char *keys;
  int border, count;
  int active;
} frame[MAX_FRAME], i;

char *fruit[]= {
  "Apple",
  "Orange",
  "Pear",
  "Grape",
  "Raspberry",
  "Strawberry"
};

char *color[]= {
  "Red",
  "Yellow",
  "Orange",
  "Green",
};

char *apple_type[]= {
  "Red delicious",
  "Jonathan",
  "Winesap",
  "Rome"
};

char *grape_type[]= {
  "Concord",
  "cAnadice",
  "Thompson",
  "Red flame"
};

main(void)
{
  cls();
  goto_xy(0, 0);

  /* first, create the menu frames */
  make_menu(0, fruit, "aopgrs", 6, 20, 5, BORDER); 
  make_menu(1, color, "ryog", 4, 28, 9, BORDER);
  make_menu(2, apple_type, "rjwr", 4, 32, 12, BORDER);
  make_menu(3, grape_type, "catr", 4, 10, 9, BORDER);

  printf("Select your fruit:");

  pd_driver(); /* activate the menu system */

  return 0;
}

/* Demonstrate the pull-down menu functions. */
void pd_driver(void)
{
  int choice1, choice2, selection;

  /* now, activate as needed */
  while((choice1=pulldown(0)) != -1) {
    switch(choice1) {
      case 0:  /* wants an apple */
        while((choice2=pulldown(1)) != -1) { 
          if(choice2==0) {
            selection = pulldown(2); /* red apple */
            restore_video(2);
          }
        }
        restore_video(1);
        break;
      case 1:
      case 2: goto_xy(0, 1);
        printf("out of that selection");
        break;
      case 3:  /* wants a grape */
        selection = pulldown(3);
        restore_video(3);
        break;
      case 4:
      case 5: goto_xy(0, 1);
        printf("out of that selection");
        break;
    }
  }
  restore_video(0);
}

/* Display a pull-down menu and return selection. */
int pulldown(int num)
{
  int  vmode;

  vmode = video_mode();
  if((vmode!=2) && (vmode!=3) && (vmode!=7)) {
    printf("video must be in 80 column text mode");
    exit(1);
  }
  /* set proper address of video RAM */
  if(vmode==7) vid_mem = (char far *) 0xB0000000;
  else vid_mem = (char far *) 0xB8000000;

  /* get active window */
  if(!frame[num].active) { /* not currently in use */ 
    save_video(num);       /* save the current screen */
    frame[num].active = 1; /* set active flag */
  }

  if(frame[num].border) draw_border(num);

  display_menu(num);    /* display the menu */
  return get_resp(num); /* return response */
}


/* Construct a pull down menu frame.
   It returns 1 if the menu frame can be constructed;
   otherwise 0 is returned.
*/
make_menu(
  int num,      /* menu number */
  char *menu[], /* menu text */
  char *keys,   /* hot keys */
  int count,    /* number of menu items */
  int x, int y, /* X,Y coordinates of left hand corner */
  int border    /* no border if 0 */
)
{
  register int i, len;
  int endx, endy;
  unsigned char *p;

  if(num>MAX_FRAME) {
    printf("Too many menus\n");
    return 0;
  }

  if((y>24) || (y<0) || (x>79) || (x<0)) {
    printf("range error");
    return 0;
  } 

  /* compute the size */
  len = 0;
  for(i=0; i<count; i++) 
   if(strlen(menu[i]) > len) len = strlen(menu[i]);
  endx = len + 2 + x;
  endy = count + 1 + y;
  if((endy+1>24) || (endx+1>79)) {
    printf("menu won't fit");
    return 0;
  } 

  /* allocate enough memory to hold current contents
     of the screen */
  p = (unsigned char *) malloc(2 * (endx-x+1) * (endy-y+1));
  if(!p) exit(1); /* put your own error handler here */

  /* construct the frame */
  frame[num].startx = x; frame[num].endx = endx;
  frame[num].starty = y; frame[num].endy = endy;
  frame[num].p = p; 
  frame[num].menu = (char **) menu;
  frame[num].border = border;
  frame[num].keys = keys;
  frame[num].count = count;
  frame[num].active = 0;
  return 1;  
}

/* Display the menu in its proper location. */
void display_menu(int num)
{
  register int i, y;
  char **m;

  y = frame[num].starty+1;
  m = frame[num].menu;

  for(i=0; i<frame[num].count; i++, y++)
    write_string(frame[num].startx+1, y, m[i], NORM_VID);
}

/* Draw a border around the menu. */
void draw_border(int num)
{
  register int i;
  char far *v, far *t;

  v = vid_mem;
  t = v;

  /* draw vertical lines */
  for(i=frame[num].starty+1; i<frame[num].endy; i++) {
     v += (i*160) + frame[num].startx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
     v += (i*160) + frame[num].endx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
  }

  /* draw horizontal lines */
  for(i=frame[num].startx+1; i<frame[num].endx; i++) {
     v += (frame[num].starty*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
     v += (frame[num].endy*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
  }

  /* draw corners */
  write_char(frame[num].startx, frame[num].starty, 
             (char) 218, NORM_VID);
  write_char(frame[num].startx, frame[num].endy, 
             (char) 192, NORM_VID);
  write_char(frame[num].endx, frame[num].starty, 
             (char) 191, NORM_VID);
  write_char(frame[num].endx, frame[num].endy, 
             (char) 217, NORM_VID);
}

/* Input user's selection */
get_resp(int num)
{
  union inkey {
    char ch[2];
    int i;
  } c;
  int arrow_choice=0;
  char *key_choice;
  int x, y;

  x = frame[num].startx+1;
  y = frame[num].starty+1;

  /* highlight the first selection */
  goto_xy(x, y);
  write_string(x, y, frame[num].menu[0], REV_VID); 

  for(;;) {
    c.i = readkey();    /* read the key */

    /* reset the selection to normal video */
    goto_xy(x, y+arrow_choice);
    write_string(x, y+arrow_choice,
      frame[num].menu[arrow_choice], NORM_VID); /* redisplay */

    if(c.ch[0]) { /* is normal key */
      /* see if it is a hot key */
      key_choice = strchr(frame[num].keys, tolower(c.ch[0]));
      if(key_choice) return key_choice-frame[num].keys;

      /* check for ENTER or space bar */
      switch(c.ch[0]) {
        case '\r': return arrow_choice;
        case ' ' : arrow_choice++;
          break;
        case ESC : return -1; /* cancel */
      }
    }
    else {  /* is special key */
      switch(c.ch[1]) {
        case 72: arrow_choice--; /* up arrow */
          break;
        case 80: arrow_choice++; /* down arrow */
          break;
      }
    }
    if(arrow_choice==frame[num].count) arrow_choice=0;
    if(arrow_choice<0) arrow_choice = frame[num].count-1;

    /* highlight the next selection */
    goto_xy(x, y+arrow_choice);
    write_string(x, y+arrow_choice, 
      frame[num].menu[arrow_choice], REV_VID);
  }
}

/* Display a string with specified attribute. */
void write_string(int x, int y, char *p, int attrib)
{
  char far *v;

  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  while(*p) {
    *v++ = *p++;  /* write the character */
    *v++ = attrib;    /* write the attribute */
   }
}

/* Write a character with specified attribute. */
void write_char(int x, int y, char ch, int attrib)
{
  char far *v;
  
  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  *v++ = ch;  /* write the character */
  *v = attrib;    /* write the attribute */
}

/* Save a portion of the screen. */
void save_video(int num)
{
  register int i,j;
  char *buf_ptr;
  char far *v, far *t;

  buf_ptr = frame[num].p;
  v = vid_mem;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      t = (v + (j*160) + i*2);
      *buf_ptr++ = *t++;
      *buf_ptr++ = *t;
      *(t-1) = ' ';  /* clear the window */
    }
}

/* Restore a portion of the screen. */
void restore_video(int num)
{
  register int i,j;
  char far *v, far *t;
  char *buf_ptr;

  buf_ptr = frame[num].p;
  v = vid_mem;
  t = v;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      v = t;
      v += (j*160) + i*2; /* compute the address */
      *v++ = *buf_ptr++;  /* write the character */
      *v = *buf_ptr++;    /* write the attribute */
  }
  frame[num].active = 0; /* deactivate */
}


/* Clear the screen. */
void cls(void)
{
  union REGS r;

  r.h.ah = 6; /* screen scroll code */
  r.h.al = 0; /* clear screen code */
  r.h.ch = 0; /* start row */
  r.h.cl = 0; /* start column */
  r.h.dh = 24; /* end row */
  r.h.dl = 79; /* end column */
  r.h.bh = 7;  /* blank line is black */
  int86(0x10, &r, &r);
}

/* Send the cursor to x,y. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* returns the current video mode */
video_mode(void)
{
  union REGS r;

  r.h.ah = 15;  /* get video mode */
  return int86(0x10, &r, &r) & 255;
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}

