/*
|	FILE: PRIM.CPP
|
|	This module is contains the source code for the program
|	CALC.CPP. It is code from:
|
|		Writing Your Own Application Language
|
|	Published by:
|		Wordware Publishing, Inc.
|		1506 Capital Avenue
|		Plano, Texas 75074
|
|	Written by:
|		Norman E. Smith, CDP
|		Copyright 1992 
|		All Rights Reserved
|
|	Right to use, copy, and modify this code is granted
|	for personal non-commercial use, provided that this 
|	copyright disclosure remains on ALL copies. Any other 
|	use, reproduction, or distribution requires the written 
|	consent of the author.
*/

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>

#include "until.h"
/***********************+-------+
|			| popsp |
|			+-------+
*/
inline long popsp()
{
	long value;

	SP--;
	if(SP<0){
		cout << "P Stack underflow.";
		SP = 0;
	}
	value = pstack[SP];
	return(value);
}
/***********************+--------+
|			| pushsp |
|			+--------+
|
| This word pushes a value on the parameter stack. The stack
| is 32-bits. Stack is tested for overflow.
*/
inline void pushsp(long value)
{
	if(SP>PSTACKSIZE){
		cout << "P Stack Overflow";
		SP = PSTACKSIZE;
	}
	pstack[SP++] = value;
}
/***********************+-------+
|			| poprs |
|			+-------+
*/
inline long poprs()
{
	long value;

	RS--;
	if(RS<0){
		cout << "R Stack underflow.";
		RS = 0;
	}
	value = rstack[RS];
	return(value);
}
/***********************+--------+
|			| pushrs |
|			+--------+
*/
inline void pushrs(long value)
{
	if(RS>RSTACKSIZE){
		cout << "R Stack Overflow";
		RS = RSTACKSIZE;
	}
	rstack[RS++] = value;
}
/***********************+------+
|			| word |
|			+------+
|  This word extracts the next word from the input stream (tib)
|  and places it in pad. The parameter stack is used to pass
|  and return arguments.
|
|  NOTE: tib_len must have been set by the input function prior
|        to calling word().
|
|	word()		// ( delim --- 'pad )
*/
void word()
{
	char ch;
	char delim;		// word delimiter
	int  i;
	long X;

	if(IN  >= tib_len){
		read_next_line();	// read next line from stream
	}
	X       = popsp();
	delim   = (char) X;
	ch      = tib[IN++];
		/*
		|  Skip leading delimiters
		*/
	while(ch == delim){
		ch = tib[IN++];
		if(IN > tib_len){
			ch = delim;		// force back thru 1 more time
			read_next_line();	// read next line from stream
		}
	}
		/*
		|  Collect chars up to a match on delim. Chars are
		|  loaded into pad starting at pos. 1. pad[0] is
		|  reserved for the length byte.
		*/
	i = 1;
	while(ch != delim){
		pad[i++] = ch;
		ch       = tib[IN++];
		if(IN > tib_len){
			break;
		}
	}
	pad[i++] = NULL;
	pad[i]   = delim;
	pad[0]   = i-2;
	pushsp((long)pad);
}
/***********************+------+
|			| cold |
|			+------+
*/
void cold()
{
	static NameField  *Name0;
	static DictHeader *DP0;

	cout << "                                    CALC\n";
	cout << "                             (c) Copyright 1992\n";
	cout << "                             All Rights Reserved\n";
	cout << "                               Norman E. Smith\n";
	cout << " \n";
	cout << "                                Version ";
	cout << CALC_VERSION << "\n";
	cout << "                               ";
	cout << DATE << "\n";
	ABORT           = 0;
	QUIT            = 0;
	DP0             = (DictHeader *) new char [sizeof(DictHeader)];
	Name0           = CreateNFA("Name0");
	DP0->NFA        = Name0;
	DP0->CFA        = do_colon;
	DP0->PFA.lvalue = 0;
	DP0->LFA        = 0;
	DP              = DP0;
	tib_len         = 0;
	IN              = 0;
	START           = FALSE;
	SP              = 0;
	RS              = 0;
	STATE           = FALSE;
	READ_WA         = read_stdin;
	INPUT_SOURCE    = 0;
	make_prims();
	prompt();
}
/***********************+-------------+
|			| do_variable |
|			+-------------+
*/
void do_variable()
{
	pushsp((long) &WA->PFA.lvalue);
}
/***********************+-------------+
|			| do_variable |
|			+-------------+
|
|	Look at making this allot a single long and putting
|	its pointer into the Dict Header instead of its
|	value. This may help with the ',' and allot problems.
*/
void do_variable(DictHeader *entry)
{
	pushsp((long) &entry->PFA.lvalue);
}
/***********************+-------------+
|			| do_constant |
|			+-------------+
*/
void do_constant()
{
	pushsp(WA->PFA.lvalue);
}
/***********************+---------+
|			| do_word |
|			+---------+
*/
void do_word(DictHeader *entry)
{
	(*entry->CFA)();
}
/********************+----------+
|                    | do_colon |
|                    +----------+
| This function loops through a PFA that contains a list of
| Word Addresses (WA) to execute. The last entry must be a
| NULL to flag end of addresses.
|
| This version of do_colon is executed from inside a Forth
| word. The word address is picked up from the global variable
| WA. I did it this way because I never figured out a way to
| stick a function in a table with arguments.
*/
void do_colon()
{
	register DictHeader *word;
	DictHeader **old_IP;

	old_IP = IP;			// Save curr IP+1
	IP     = WA->PFA.Waddr;
	WA     = 0;
	word   = *IP++;

	while(word){
		WA   = word;
		(*word->CFA)();
		word = *IP++;
	}
	IP = old_IP;
}
/***********************+------+
|			| drop |
|			+------+
|	drop	( n --- )
*/
void drop()
{
	SP--;
	if(SP<0){
		cout << "P Stack underflow.";
		SP = 0;
	}
}
/***********************+-----------+
|			| exec_word |
|			+-----------+
|
|  Execute a forth word from C. The word name is passed in as
|  an argument. Pad cannot be used because it gets trashed in
|  the null_to_counted conversion. So, string is used an an
|  intermediate area.
|
|  prim_find() uses pad to look up the word, and sets WA when
|  it is found. The pad string must be null terminated.
*/
void exec_word()
{
	int  status;

	status = prim_find();
	if(status){
		(*WA->CFA)();
	}else{
		pushsp((long)pad);
		count();
		type();
		cout << " Word not found...\n";
	}
}
/***********************+-----------+
|			| prim_find |
|			+-----------+
|  This word searches the dictionary for a word name match with
|  the counted string in pad. When a match occurs, WA is set to
|  the address of the word that matches. truth is returned to
|  indicate wether or not a match was found.
|
|  FALSE means a match.
|
|  When the smudge flag is set, there is no match, even if the
|  string compare works. This prevents Forth from compiling
|  words that are not fully compiled.
|
|  Note: This version does not match the way Forth does is. It
|  	 should be used internally by primitives for increased
|	 execution speed.
*/
int prim_find()
{
	DictHeader *temp;
	char string[80];
	int  truth = FALSE;
	int  status;
	int  len;
			/*
			| Copy the word text from pad to string
			*/
	len = pad[0];
	counted_to_null(pad,string);
			/*
			| Search the dictionary starting at DP
			| looking for a match.
			*/
	temp = DP;
	while(temp->LFA){
	   if((len == temp->NFA->len)&&(temp->NFA->smudge != 1)){
		status = strncmp(string,temp->NFA->name,temp->NFA->len);
		if(status == 0){		    // 0 means match
			if(temp->NFA->smudge == 1){ // Can't use smudged word
				truth = FALSE;
				break;
			}else{
				truth = TRUE;
				break;
			}
		}
	   }
	   temp = temp->LFA;
	}
	WA = temp;
	if(truth == TRUE){		// set immediate flag
		if(WA->NFA->immediate!=0){
			truth = WA->NFA->immediate;
		}
	}
	return(truth);
}
/***********************+------------+
|			| minus_find |
|			+------------+
|     	minus_find		( 'string --- 'string 0)  Not found
|				( 'string --- WA -1 )     Found, normal
|				( 'string --- WA 1  )     Found, immediate
|
|  This word searches the dictionary for a word name match with
|  the counted string in pad.
|
|  When the smudge flag is set, there is no match, even if the
|  string compare works. This prevents Forth from compiling
|  words that are not fully compiled.
*/
void minus_find()
{
	DictHeader *temp;
	char string[32];
	char *addr;
	int  len;
	int  truth = FALSE;
	int  status;

	addr   = (char *)popsp();
	counted_to_null(addr,string);
	len    = *addr;
	temp   = DP;
			/*
			| Search the dictionary starting at DP
			| looking for a match.
			*/
	while(temp->LFA){
		if((len == temp->NFA->len) && (temp->NFA->smudge != 1)){
			status = strncmp(string,temp->NFA->name,temp->NFA->len);
			if(status == 0){		// 0 means match
				truth = TRUE;
				pushsp((long) temp);
				if(temp->NFA->immediate == 1){
					truth = -1;
				}
				break;
			}
		}
		temp = temp->LFA;
	}
	if(!truth){			// need orig addr if no match
		pushsp((long) addr);
	}
	pushsp(truth);
}
/***********************+-----------------+
|			| counted_to_null |
|			+-----------------+
|
|  This word converts a string from counted form to null
|  terminated form.
*/
void counted_to_null(char *counted, char* null)
{
	int  i;
	int  len;

	len = *counted++;
	for(i=0;i<len;i++){
		*null++ = *counted++;
	}
	*null = '\0';
}
/***********************+--------+
|			| rfetch |
|			+--------+
|
|	The value of RS is not checked, so if there is nothing
|	on the rstack, a 0 is returned rather than an error. A
|	check will be added later if this turns out to be a
|	problem.
*/
void rfetch()
{
	long value;

	value = rstack[RS-1];			// get addr of cell
	pushsp(value);				// put it on the stack
}
/***********************+------+
|			| to_r |
|			+------+
|
*/
void to_r()
{
	long value;

	value = popsp();
	pushrs(value);				// put it on the stack
}
/***********************+--------+
|			| r_from |
|			+--------+
|
|	r>  	( --- value)
*/
void r_from()
{
	long value;

	value = poprs();
	pushsp(value);				// put it on the stack
}
/***********************+-----------------+
|			| null_to_counted |
|			+-----------------+
|
|  This word converts a string from null to counted form
|  terminated form.
*/
void null_to_counted(char *null, char* counted)
{
	int  len;
	char *original;

	original  = counted++;
	len       = strlen(null);
	strcpy(counted,null);
	*original = len;
}
/***********************+-------+
|			| count |
|			+-------+
|	count	( addr --- addr len )
*/
void count()
{
	char *addr;
	int  len;

	addr = (char *)popsp();
	len  = *addr++;
	pushsp((long)addr);
	pushsp(len);
}
/***********************+-------+
|			| fetch |
|			+-------+
|
|	<-  	( addr --- val )
*/
void fetch()
{
	long value;
	long *addr;

	addr  = (long *) popsp();		// get addr of cell
	value = *addr;				// then the value
	pushsp(value);				// put it on the stack
}
/***********************+-------+
|			| store |
| 			+-------+
|
|	->	( val addr --- )
*/
void store()
{
	long *addr;
	long value;

	addr  = (long *) popsp();
	value = popsp();
	*addr = value;
}
/***********************+----------+
|			| prim_dup |
|			+----------+
|	dup	( n --- n n )
|
| This function is not named dup because there is a predefined C
| system function of that name.
|
*/
void prim_dup()
{
	long value;

	value = pstack[SP-1];
	pushsp(value);
}
/***********************+------+
|			| swap |
|			+------+
|	swap	( n1 n2 --- n2 n1 )
*/
void swap()
{
	long value;

	if(SP < 2){
		cout << "P Stack: not enough entries.";
		return;
	}
	value        = pstack[SP-1];
	pstack[SP-1] = pstack[SP-2];
	pstack[SP-2] = value;
}
/***********************+-----+
|			| dot |
|			+-----+
|	=	( n1 --- )
*/
void dot()
{
	long n1;

	n1 = popsp();
	cout << n1 << " ";
}
/***********************+------+
|			| zero |
|			+------+
*/
void zero()
{
	pushsp(0);
}
/***********************+-----+
|			| one |
|			+-----+
*/
void one()
{
	pushsp(1);
}
/***********************+-----+
|			| two |
|			+-----+
*/
void two()
{
	pushsp(2);
}
/***********************+-------+
|			| three |
|			+-------+
*/
void three()
{
	pushsp(3);
}
/***********************+-------+
|			| words |
|			+-------+
|
*/
void words()
{
	DictHeader *temp;
	char       string[80];
	int        length;
	int	   wnum = 0;

	temp = DP;
	cout << "\n";
	while(temp->LFA){
		length         = temp->NFA->len;
		strncpy(string,temp->NFA->name,length);
		string[length] = NULL;
		printf("%16s ",string);
		wnum++;
		if(wnum>3){
			wnum = 0;
			cout << "\n";
		}
		temp           = temp->LFA;
	}
        cout << "\n";
}
/***********************+--------+
|			| prompt |
|			+--------+
*/
void prompt()
{
	cout << "\n" << PROMPT;
}
/***********************+-----+
|			| bye |
|			+-----+
*/
void bye()
{
	QUIT  = 1;
	ABORT = 1;
}
/***********************+---------+
|			| abort_F |
|			+---------+
|	abort 	( --- )
*/
void abort_F()
{
	ABORT = 1;
        cout << "Aborted\n";
}
/***********************+------+
|			| warm |
|			+------+
*/
void warm()
{
	tib_len   = 0;
	IN        = 0;
	SP        = 0;
	RS        = 0;
	STATE     = FALSE;
	QUIT      = 0;
}
/***********************+-------+
|			| outer |
|			+-------+
|
|	This is the outer interpreter. It seems too simple.
|
*/
void outer()
{
	static started;
	int    found;

	if(!started){
		started = TRUE;
		startup();
	}
	QUIT = 0;
	for(;;){
	  if(QUIT){
		return;		        // Initialization
	  }else{
		warm();
	  }
	  do{
		pushsp(BLANK);
		word();			// get next word from input stream
		minus_find();		// Look it up in the dictionary
		found = popsp();
		if(found){		// found the word
			WA = (DictHeader*) popsp();
			(*WA->CFA)();
		}else{		        // Word not found, so try to turn
			drop();		//   it into a number.
			pushsp((long)pad);       // Word leaves string at pad
			number();	// Convert it to int.
		}
		gdot_s();
	  }while(!ABORT);
	  ABORT = 0;
	}
}
/***********************+----+
|			| cr |
|			+----+
|	cr	( --- )
*/
void cr()
{
	cout << "\n";
}
/***********************+-------+
|			| dot_s |
|			+-------+
|	s?	( --- )
*/
void dot_s()
{
	int TSP;			// temp stack ptr

	TSP = SP - 1;
	if(TSP < 0){
		cout << "Stack Empty\n";
		return;
	}
	while(TSP >= 0){
		cout << pstack[TSP--] << "\n";
	}
}
/***********************+--------+
|			| gdot_s |
|			+--------+
|	.g	( --- )
*/
void gdot_s()
{
	static tdepth;			// Use to blank out prev display
	int TSP;			// temp stack ptr
	int row = 2;
	int col = 70;
	int orig_row;
	int orig_col;

		/*
		|  Save current screen position
		*/
	orig_row = wherey();
	orig_col = wherex();
		/*
		|  blank out prev display
		*/
	while(tdepth){
		gotoxy(col,row++);
		cout << "          ";
		tdepth--;
	}
		/*
		|  Do the stack print
		*/
	row    = 2;
	TSP    = SP - 1;
	tdepth = SP + 1;
	if(TSP < 0){
		gotoxy(col,row);
		cout << "Stack Empty";
		gotoxy(orig_col,orig_row);	// restore cursor
		tdepth = 0;
		return;
	}
	gotoxy(col,row++);
	cout << "Stack      ";
	while(TSP >= 0){
		gotoxy(col,row++);
		cout << pstack[TSP--];
	}
	if(ABORT){
		row = 2;
		while(tdepth){
			gotoxy(col,row++);
			cout << "          ";
			tdepth--;
		}
	}
	gotoxy(orig_col,orig_row);		// restore cursor
}
/***********************+-----+
|			| cls |
|			+-----+
*/
void cls()
{
	clrscr();
}
/***********************+-----------+
|			| next_word |
|			+-----------+
|
| This word is intended for use inside primitives only.
|
*/
void next_word()
{
	pushsp(BLANK);
	word();
	drop();
}
/***********************+--------+
|			| do_lit |
|			+--------+
|
| This is the run-time portion of literal.
*/
void do_lit()
{
	pushsp((long) *IP++);
}
/***********************+-----------+
|			| dot_quote |
|			+-----------+
|	" string"
|
|  This is the run-time portion of ." and is valid only
|  in compile mode.
*/
void dot_quote()
{
	char *const_string;

	const_string = (char *)*IP++;
	const_string++;
	cout << const_string;
}
/***********************+-------+
|			| do_do |
|			+-------+
| This function is the run-time for 'do'. 
*/
void do_do()
{
	pushrs(0);
	one_minus();
	if(SP < 1){
		cout << "Stack is empty; loops require a count\n";
		abort_F();
		return;
	}
	if(pstack[SP-1] < 1){
		cout << "Loop arg must be > 0\n";
		abort_F();
		return;
	}
	to_r();
}
/***********************+---------+
|			| do_loop |
|			+---------+
| Part of do ... loop. This version of loop tests for exit at
| the bottom of the loop rather than at the top.
*/
void do_loop()
{
	long end;
	long index;

	end   = poprs();
	index = poprs();
	if(index == end){
		IP++;
	}else{
		index++;
		pushrs(index);
		pushrs(end);
		branch();
	}
}
/***********************+-----------+
|			| zero_bran |
|			+-----------+
*/
void zero_bran()
{
	long truth;
	long offset;

	truth = popsp();                // get truth from pstack
	if(truth){
		IP++;			// skip branch offset
		return;			// no branch, get out
	}
	offset = (long)*IP;		// mark branch offset point
	IP    += offset;		// jump to new code
}
/***********************+--------+
|			| branch |
|			+--------+
*/
void branch()
{
	long offset;

	offset = (long)*IP;		// mark branch offset point
	IP    += offset;		// jump to new code
}
/***********************+--------+
|			| number |
|			+--------+
|	number	( addr --- n )
|
|	This is the easy way to implement number(). It does not
|	exactly adhere to traditional Forth, but should not cause
|	a problem 99.9% of the time because 0 is a word in the
|	dictionary. 0 should be found in the dict before getting
|	to number.  It may also not work correctly for bases
|	other than decimal.
|
|	The bottom line is this way is very efficient with minimal
|	possibility of incorrect side effects.
|
|	addr is a counted string.
|
|	NOTE: The global variable ABORT is set if an error in
|	      conversion occurs.
|
|	ALTERNATIVE:	*(addr+len) = NULL;
|	This should work, but needs to be tested before being
|	put into the code.
*/
void number()
{
	char *addr;
	char nstring[32];
	long value;

	addr     = (char *) popsp();
	counted_to_null(addr,nstring);		// Look at alternative
	value    = atol(nstring);
	pushsp(value);
	if(value == 0){				// bad conversion
		cout << "? \n";
		abort_F();			// should be abort_quote
	}
}
/***********************+-------+
|			| quote |
|			+-------+
|	" string"
|
|  This is the run-time portion of ." and is valid only
|  in compile mode.
*/
void quote()
{
	pushsp((long)*IP++);
}
/***********************+----------+
|			| question |
|			+----------+
|	?	( addr --- )
|
|  Same as @ .
*/
void question()
{
	fetch();
	dot();
}
void rot();
void Compile_i();
/***********************+-----+
|			| rot |
|			+-----+
|	rot ( n1 n2 n3 --- n2 n3 n1)
*/
void rot()
{
	long n1, n2, n3;

	n3 = popsp();
	n2 = popsp();
	n1 = popsp();

	pushsp(n2);
	pushsp(n3);
	pushsp(n1);
}
/***********************+------+
|			| do_i |
|			+------+
*/
void do_i()
{
	pushsp(rstack[RS-2]);
}
/***********************+------+
|			| help |
|			+------+
*/
void help()
{
	int  i;
	int  j;
	int  match;
	int  truth;
	int  help_file;

	char *status;
	char line[132];
	char string[32];

	help_file	  = INPUT_SOURCE;
	FD[INPUT_SOURCE++] = fopen("HELP.APP","r");
	if(!FD[help_file]){
		cout << "Help file not found\n";
		return;
	}
	truth  = FALSE;
	for(i=0;i<32;i++){
		pad[i] = 0;
	}
	next_word();
	status = fgets(line,132,FD[help_file]);
	while(status){				// do until eof
		if(line[0] != BLANK){
			j = 0;
			i = 0;
			while(line[i] != BLANK){
				string[j++] = line [i++];
			}
			string[j] = NULL;
			match     = strcmp(string,&pad[1]);
			if(match == 0){
				truth  = TRUE;
				cout << line;
				status = fgets(line,132,FD[help_file]);
				if(!status) goto All_done;
				while(line[0] == BLANK){
					cout << line;
					status = fgets(line,132,FD[help_file]);
				}
				goto All_done;
			}else{
				status = fgets(line,132,FD[help_file]);
				if(!status) goto All_done;
			}
		}else{
			status = fgets(line,132,FD[help_file]);
			if(!status) goto All_done;
		}
	}
All_done:
	close_include();
	if(!truth){
		cout << "Macro not found in Help file.\n";
	}
}
/***********************+------+
|			| type |
|			+------+
| This word types a string to the screen.
|
|	type		( addr len --- )
*/
void type()
{
	int  i;
	int  len;
	char ch;
	char *string;

	len    = popsp();		// get length
	string = (char *) popsp();	// get string addr

	for(i=0;i<len;i++){
		ch = *string++;
		cout << ch;
	}
}
/***********************+-----------+
|			| one_minus |
|			+-----------+
|	1-	( n --- n-1 )
*/
void one_minus()
{
	long value;

	value = popsp();
	pushsp(value - 1);
}
