
/* Printer control subsystem for Epson dot-matrix
   printers (and compatibles): Text functions and
   demonstration program.
*/

#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.  If you use a different C compiler, refer
    to the text.
*/

#include "stdio.h"
#include "stdlib.h"
#include "bios.h"

#define ON 1
#define OFF 0
#define DRAFT 0
#define NLQ 1

/* Various mode definitions */
#define WIDE            14
#define CONDENSED       15
#define KILL_CONDENSED  18
#define KILL_WIDE       20
#define EMPHASIZED      69
#define KILL_EMPHASIZED 70
#define BOLD            71
#define KILL_BOLD       72

void print(char ch);
void print_condensed(char *s);
void print_wide(char *s), print_str(char *s);
void print_italic(char *s), print_emphasized(char *s);
void print_bold(char *s), print_underlined(char *s);
void print_super(char *s), print_sub(char *s);

void select_pica(void), select_elite(void);
void select_superscript(void), select_subscript(void);
void cancel_super_sub(void), select_style(char mode);
void select_6(void), select_8(void), select_72(char num);
void select_7_72(void), select_216(char num);
void select_underlined(void), cancel_underlined(void);
void select_mode(unsigned char mode);
void reset(void);

main(void)
{
  int i;

  print_str("This is normal.\n");
  print_condensed("This is condensed.\n");
  print_str("This is normal, again.\n");
  print_condensed("This is condensed, again.\n");
  print_wide("This is wide.\n");
  print_italic("This is italic.\n");
  print_emphasized("This is emphasized.\n");
  print_bold("This is in bold.\n");

  select_elite();
  print_str("this is in elite\n");
  select_pica();

  print_str("This is a test of ");
  select_superscript();
  print_str("superscripted");
  select_subscript();
  print_str("subscripted ");
  cancel_super_sub();
  print_str("text.\n");

  select_mode(WIDE);
  select_mode(BOLD);
  print_str("This is wide and bold.\n");
  select_mode(KILL_BOLD);
  select_mode(KILL_WIDE);

  print_str("Super- and subscripts make mathematical ");
  print_str("symbols easy: ");
  print_str("X");
  print_super("2");
  print_str(" * N");
  print_sub("i\n");

  print_underlined("This line is underlined\n");
  print_str("This line isn't underlined.\n");


  select_style(NLQ); 
  print_str("This is NLQ.\n");
  select_style(DRAFT);
  print_str("This is draft.\n");  


  select_6();
  print_str("Six lines per inch:\n");
  for(i=0; i<6; i++) print_str("x\n");

  select_8();
  print_str("8 lines per inch:\n");
  for(i=0; i<6; i++) print_str("x\n");

  select_7_72();
  print_str("About 10 lines per inch:\n");
  for(i=0; i<6; i++) print_str("x\n");

  print_str("About 20 lines per inch:\n");
  select_216(10);
  for(i=0; i<6; i++) print_str("x\n");  
  reset();
  return 0;
}

/* Print a string in condensed format. */
void print_condensed(char *s)
{
  print(27); print(15);
  print_str(s);
  print(18);  /* cancel condensed */
}

/* Print a string in wide format. */
void print_wide(char *s)
{
  print(27); print(14);
  print_str(s);
  print(20);
}  

/* Print a string in italics. */
void print_italic(char *s)
{
  print(27); print(52); /* enable italics */
  print_str(s);
  print(27); print(53); /* disable italics */
}

/* Print a string in emphasized type. */
void print_emphasized(char *s)
{
  print(27); print(69); /* enable emphasized */
  print_str(s);
  print(27); print(70); /* disable emphasized */
}
  
/* Print a string in bold. */
void print_bold(char *s)
{
  print(27); print(71); /* enable bold */
  print_str(s);
  print(27); print(72); /* disable bold */
}

/* Print an underlined string. */
void print_underlined(char *s)
{
  print(27); print(45); print(1); /* enable underlining */
  print_str(s);
  print(27); print(45); print(0); /* disable underlining */    
}

/* Select underlining. */
void select_underlined(void)
{
  print(27); print(45); print(1); /* enable underlining */
}

/* Cancel underlining. */
void cancel_underlined(void)
{
  print(27); print(45); print(0); /* disable underlining */
}
  
/* Select elite type font. */
void select_elite(void)
{
  print(27); print(77);
}

/* Select pice type font. */
void select_pica(void)
{
  print(27); print(80);
}

/* Select superscript mode. */
void select_superscript(void)
{
  print(27); print(83); print(0);
}

/* Select subscript mode. */
void select_subscript(void)
{
  print(27); print(83); print(1);
}  

/* Print a string in superscript. */
void print_super(char *s)
{
  select_superscript();
  print_str(s);
  cancel_super_sub();
}

/* Print a string in subscript. */
void print_sub(char *s)
{
  select_subscript();
  print_str(s);
  cancel_super_sub();
}

/* Cancel super- or subscript mode. */
void cancel_super_sub(void)
{
  print(27); print(84);
}

/* If mode is 0, print in draft (fast printing) mode.
   if mode is 1, print in near letter quality (NLQ)
   style.
*/
void select_style(char mode)
{
  print(27); print(120); print(mode);
}

/* Select 8 lines per page. */
void select_8(void)
{
  print(27); print(48);
}

/* Select 6 lines per page (default). */
void select_6(void)
{
  print(27); print(50);
}

/* Select 7/72 line spacing. */
void select_7_72(void)
{
  print(27); print(49);
}

/* Select num/72 line spacing */
void select_72(char num)
{
  print(27); print(65), print(num);
}

/* Select num/216 line spacing. */
void select_216(char num)
{
  print(27); print(51); print(num);
}

/* Reset the printer to defaults. */
void reset(void)
{
  print(27); print(64);
}

/* Set a mode. */
void select_mode(unsigned char mode)
{
  print(27); print(mode);
}

/* Send a string to the printer */
void print_str(char *s)
{
  while(*s) {
    print(*s);
    s++;
  }
}

/* Send a character to the printer. */
void print(char ch)
{
#ifdef TURBOC
  biosprint(0, ch, 0);
#endif
#ifdef MICROSOFTC
  _bios_printer(0, 0, ch);
#endif
}
