
/* Pop-up menu routines for text mode operation that use DOS
   video services. */

#include "stdio.h"
#include "dos.h"
#include "stdlib.h"
#include "bios.h"
#include "string.h"
#include "ctype.h"

#define BORDER 1
#define ESC 27
#define REV_VID 0x70
#define NORM_VID 7

int popup(char *menu[], char *keys, int count,
          int x, int y, int border);
void save_video(int startx, int endx, int starty, int endy,
                unsigned int *buf_ptr);
void restore_video(int startx, int endx, int starty, int endy,
                   unsigned char *buf_ptr);
void goto_xy(int x, int y), cls(void);
void write_video(int x, int y, char *p, int attrib);
void display_menu(char *menu[], int x, int y, int count);
void draw_border(int startx, int starty, int endx, int endy);
int get_resp(int x, int y, int count, char *menu[], char *keys);
int readkey(void);

char *fruit[]= {
  "Apple",
  "Orange",
  "Pear",
  "Grape",
  "Raspberry",
  "Strawberry"
};

char *color[]= {
  "Red",
  "Yellow",
  "Orange",
  "Green",
};

char *apple_type[]= {
  "Red delicious",
  "Jonathan",
  "Winesap",
  "Rome"
};

main(void)
{
  int i;

  cls();
  goto_xy(0, 0);

  for(i=0; i<25; i++)
    printf("This is a test of the pop-up window routines.\n");

  popup(fruit, "aopgrs", 6, 3, 1, BORDER); 
  popup(color, "ryog", 4, 10, 5, BORDER); 
  popup(apple_type, "rjwr", 4, 18, 10, BORDER); 

  return 0;
}

/* Display a pop-up menu and return selection.
   This function returns -2 if menu cannot be constructed;
   it returns -1 if user hits escape key;
   otherwise the item number is returned starting
   with 0 as the first (topmost) entry.
*/
int popup(
  char *menu[], /* menu text */
  char *keys,   /* hot keys */
  int count,    /* number of menu items */
  int x, int y, /* X,Y coordinates of left hand corner */
  int border    /* no border if 0 */
)
{
  register int i, len;
  int endx, endy, choice;
  unsigned int *p;

  if((y>24) || (y<0) || (x>79) || (x<0)) {
    printf("range error");
    return -2;
  } 

  /* make sure that the menu will fit */
  len = 0;
  for(i=0; i<count; i++) 
   if(strlen(menu[i]) > len) len = strlen(menu[i]);
  endx = len + 2 + x;
  endy = count + 1 + y;
  if((endy+1>24) || (endx+1>79)) {
    printf("menu won't fit");
    return -2;
  } 

  /* allocate enough memory to hold the current contents
     of the screen */
  p = (unsigned int *) malloc((endx-x+1) * (endy-y+1));
  if(!p) exit(1);  /* install your own error handler here */

  /* save what is currently on the screen */
  save_video(x, endx+1, y, endy+1, p);

  if(border) draw_border(x, y, endx, endy);

  /* display the menu */
  display_menu(menu, x+1, y+1, count);

  /* get the user's response */
  choice = get_resp(x, y, count, menu, keys);

  /* restore the original screen */
  restore_video(x, endx+1, y, endy+1, (char *) p);
  free(p);

  return choice;
}

/* Display the menu in its proper location. */
void display_menu(char *menu[], int x, int y, int count)
{
  register int i;

  for(i=0; i<count; i++, y++) {
    goto_xy(x, y);
    printf(menu[i]);
  }
}

/* Draw a border around the menu. */
void draw_border(int startx, int starty, int endx, int endy)
{
  register int i;

  /* draw vertical lines */
  for(i=starty+1; i<endy; i++) {
     goto_xy(startx, i);
     putchar(179);
     goto_xy(endx, i);
     putchar(179);
  }

  /* draw horizontal lines */
  for(i=startx+1; i<endx; i++) {
    goto_xy(i, starty);
    putchar(196);
    goto_xy(i, endy);
    putchar(196);
  }

  /* draw the corners */
  goto_xy(startx, starty); putchar(218);
  goto_xy(startx, endy); putchar(192);
  goto_xy(endx, starty); putchar(191);
  goto_xy(endx, endy); putchar(217);

}

/* Input user's selection. */
get_resp(int x, int y, int count, char *menu[], char *keys)
{
  union inkey {
    char ch[2];
    int i;
  } c;
  int arrow_choice=0;
  char *key_choice;

  x++;
  y++;

  /* highlight the first selection */
  goto_xy(x, y);
  write_video(x, y, menu[0], REV_VID); /* reverse video */

  for(;;) {
    c.i = readkey();    /* read the key */
 
    /* reset the selection to normal video */
    goto_xy(x, y+arrow_choice);
    write_video(x, y+arrow_choice,
                menu[arrow_choice], NORM_VID); /* redisplay */

    if(c.ch[0]) { /* is normal key */
      /* see if it is a hot key */
      key_choice = strchr(keys, tolower(c.ch[0]));
      if(key_choice) return key_choice-keys;

      /* check for ENTER or space bar */
      switch(c.ch[0]) {
        case '\r': return arrow_choice;
        case ' ' : arrow_choice++;
          break;
        case ESC : return -1; /* cancel */
      }
    }
    else {  /* is special key */
      switch(c.ch[1]) {
        case 72: arrow_choice--; /* up arrow */
          break;
        case 80: arrow_choice++; /* down arrow */
          break;
      }
    }
    if(arrow_choice==count) arrow_choice = 0;
    if(arrow_choice<0) arrow_choice = count-1;

    /* highlight the next selection */
    goto_xy(x, y+arrow_choice);
    write_video(x, y+arrow_choice, menu[arrow_choice], REV_VID);
  }
}

/* Display a string with specified attribute. */
void write_video(int x, int y, char *p, int attrib)
{
  union REGS r;
  register int i;

  for(i=x; *p; i++) {
    goto_xy(i, y);
    r.h.ah = 9; /* write character */
    r.h.bh = 0; /* assume active display page is 0 */
    r.x.cx = 1; /* number of times to write the character */
    r.h.al = *p++; /* character */
    r.h.bl = attrib; /* attribute */
    int86(0x10, &r, &r);
  }
}


/* Save a portion of the screen. */
void save_video(int startx, int endx, int starty, int endy,
                unsigned int *buf_ptr)
{
  union REGS r;
  register int i,j;

  for(i=startx; i<endx; i++)
    for(j=starty; j<endy; j++) {
      goto_xy(i, j); 
      r.h.ah = 8; /* read character */
      r.h.bh = 0; /* assume active display page is 0 */
      *buf_ptr++ = int86(0x10, &r, &r); /* save in buffer */
      putchar(' '); /* clear the screen */
    }
}

/* Restore a portion of the screen. */
void restore_video(int startx, int endx, int starty, int endy,
                unsigned char *buf_ptr)
{
  union REGS r;
  register int i, j;

  for(i=startx; i<endx; i++)
    for(j=starty; j<endy; j++) {
      goto_xy(i, j);
      r.h.ah = 9; /* write character */
      r.h.bh = 0; /* assume active display page is 0 */
      r.x.cx = 1; /* number of times to write the character */
      r.h.al = *buf_ptr++; /* character */
      r.h.bl = *buf_ptr++; /* attribute */
      int86(0x10, &r, &r);
  }
}


/* Clear the screen. */
void cls(void)
{
  union REGS r;

  r.h.ah = 6; /* screen scroll code */
  r.h.al = 0; /* clear screen code */
  r.h.ch = 0; /* start row */
  r.h.cl = 0; /* start column */
  r.h.dh = 24; /* end row */
  r.h.dl = 79; /* end column */
  r.h.bh = 7;  /* blank line is black */
  int86(0x10, &r, &r);
}

/* Send the cursor to specified x,y coordinates. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}


