/* Pop-up menu routines for text mode operation
   using direct video RAM reads and writes. */

#include "stdio.h"
#include "dos.h"
#include "stdlib.h"
#include "ctype.h"
#include "bios.h"
#include "string.h"

#define BORDER 1
#define ESC 27
#define REV_VID 0x70
#define NORM_VID 7

int popup(char *menu[], char *keys, int count,
          int x, int y, int border);
void save_video(int startx, int endx, int starty, int endy,
                unsigned char *buf_ptr);
void restore_video(int startx, int endx, int starty, int endy,
                   unsigned char *buf_ptr);
void goto_xy(int x, int y), cls(void);
void write_video(int x, int y, char *p, int attrib);
void display_menu(char *menu[], int x, int y, int count);
void draw_border(int startx, int starty, int endx, int endy);
void write_string(int x, int y, char *p, int attrib);
void write_char(int x, int y, char ch, int attrib);
int get_resp(int x, int y, int count, char *menu[], char *keys);
int video_mode(void);
int readkey(void);

char far *vid_mem;

char *fruit[]= {
  "Apple",
  "Orange",
  "Pear",
  "Grape",
  "Raspberry",
  "Strawberry"
};

char *color[]= {
  "Red",
  "Yellow",
  "Orange",
  "Green",
};

char *apple_type[]= {
  "Red delicious",
  "Jonathan",
  "Winesap",
  "Rome"
};

main(void)
{
  int i;

  cls();
  goto_xy(0, 0);
  for(i=0; i<25; i++)
    printf("This is a test of the pop-up window routines.\n");

  popup(fruit, "aopgrs", 6, 3, 1, BORDER); 
  popup(color, "ryog", 4, 10, 5, BORDER); 
  popup(apple_type, "rjwr", 4, 18, 10, BORDER); 

  return 0;
}

/* Display a pop-up menu and return selection.
   This function returns -2 if menu cannot be constructed;
   it returns -1 if user hits escape key;
   otherwise the item number is returned starting
   with 0 as the first (topmost) entry.
*/
int popup(
  char *menu[], /* menu text */
  char *keys,   /* hot keys */
  int count,    /* number of menu items */
  int x, int y, /* X,Y coordinates of left hand corner */
  int border    /* no border if 0 */
)
{
  register int i, len;
  int endx, endy, choice, vmode;
  unsigned char *p;

  if((y>24) || (y<0) || (x>79) || (x<0)) {
    printf("range error");
    return -2;
  } 

  vmode = video_mode();
  if((vmode!=2) && (vmode!=3) && (vmode!=7)) {
    printf("video must be in 80 column text mode");
    exit(1);
  }
  /* set proper address of video RAM */
  if(vmode==7) vid_mem = (char far *) 0xB0000000;
  else vid_mem = (char far *) 0xB8000000;

  /* make sure that menu will fit */
  len = 0;
  for(i=0; i<count; i++) 
   if(strlen(menu[i]) > len) len = strlen(menu[i]);
  endx = len + 2 + x;
  endy = count + 1 + y;
  if((endy+1>24) || (endx+1>79)) {
    printf("menu won't fit");
    return -2;
  } 

  /* allocate enough memory to hold current contents of
     the screen */
  p = (unsigned char *) malloc(2 * (endx-x+1) * (endy-y+1));
  if(!p) exit(1); /* install your own error handler here */

  /* save the current screen data */
  save_video(x, endx+1, y, endy+1, p);

  if(border) draw_border(x, y, endx, endy);

  /* display the menu */
  display_menu(menu, x+1, y+1, count);

  /* get the user's response */
  choice = get_resp(x, y, count, menu, keys);

  /* restore the original screen */
  restore_video(x, endx+1, y, endy+1,  p);
  free(p);
  return choice;
}

/* Display the menu in its proper location. */
void display_menu(char *menu[], int x, int y, int count)
{
  register int i;

  for(i=0; i<count; i++, y++) 
    write_string(x, y, menu[i], NORM_VID);
}

/* Draw a border around the window. */
void draw_border(int startx, int starty, int endx, int endy)
{
  register int i;
  char far *v, far *t;

  v = vid_mem;
  t = v;

  /* draw vertical lines */
  for(i=starty+1; i<endy; i++) {
     v += (i*160) + startx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
     v += (i*160) + endx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
  }
  /* draw horizontal lines */
  for(i=startx+1; i<endx; i++) {
     v += (starty*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
     v += (endy*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
  }
  
  /* draw corners */
  write_char(startx, starty, (char) 218, NORM_VID);
  write_char(startx, endy, (char) 192, NORM_VID);
  write_char(endx, starty, (char) 191, NORM_VID);
  write_char(endx, endy, (char) 217, NORM_VID);
}

/* Input user's selection. */
get_resp(int x, int y, int count, char *menu[], char *keys)
{
  union inkey {
    char ch[2];
    int i;
  } c;
  int arrow_choice=0;
  char *key_choice;

  x++;
  y++;

  /* highlight the first selection */
  goto_xy(x, y);
  write_string(x, y, menu[0], REV_VID); /* reverse video */

  for(;;) {
    c.i = readkey();    /* read the key */

    /* reset the selection to normal video */
    goto_xy(x, y+arrow_choice);
    write_string(x, y+arrow_choice, 
                 menu[arrow_choice], NORM_VID); /* redisplay */

    if(c.ch[0]) { /* is normal key */
      /* see if it is a hot key */
      key_choice = strchr(keys, tolower(c.ch[0]));
      if(key_choice) return key_choice-keys;
 
     /* check for ENTER or space bar */
      switch(c.ch[0]) {
        case '\r': return arrow_choice;
        case ' ' : arrow_choice++;
          break;
        case ESC : return -1;  /* cancel */
      }
    }
    else {  /* is special key */
      switch(c.ch[1]) {
        case 72: arrow_choice--; /* up arrow */
          break;
        case 80: arrow_choice++; /* down arrow */
          break;
      }
    }
    if(arrow_choice==count) arrow_choice=0;
    if(arrow_choice<0) arrow_choice = count-1;

    /* highlight the next selection */
    goto_xy(x, y+arrow_choice);
    write_string(x, y+arrow_choice, menu[arrow_choice], REV_VID);
  }
}

/* Display a string with specified attribute. */
void write_string(int x, int y, char *p, int attrib)
{
  register int i;
  char far *v;

  v = vid_mem;
  v += (y*160) + x*2;  /* compute the address */
  for(i=y; *p; i++) {
    *v++ = *p++;  /* write the character */
    *v++ = attrib;    /* write the attribute */
   }
}

/* Write character with specified attribute. */
void write_char(int x, int y, char ch, int attrib)
{
  char far *v;
  
  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  *v++ = ch; /* write the character */
  *v = attrib;  /* write the attribute */
}

/* Save a portion of the screen using direct
   video RAM accessing.
*/
void save_video(int startx, int endx, int starty, int endy,
                unsigned char *buf_ptr)
{
  register int i,j;
  char far *v, far *t;

  v = vid_mem;
  for(i=startx; i<endx; i++)
    for(j=starty; j<endy; j++) {
      t = v + (j*160) + i*2; /* compute the address */
      *buf_ptr++ = *t++; /* read the character */
      *buf_ptr++ = *t;   /* read the attribute */
      *(t-1) = ' ';  /* clear the window */
    }
}

/* Restore a portion of the screen using direct
   video RAM accessing.
*/
void restore_video(int startx, int endx, int starty, int endy,
                   unsigned char *buf_ptr)
{
  register int i,j;
  char far *v, far *t;

  v = vid_mem;
  t = v;
  for(i=startx; i<endx; i++)
    for(j=starty; j<endy; j++) {
      v = t;
      v += (j*160) + i*2; /* compute the address */
      *v++ = *buf_ptr++;  /* write the character */
      *v = *buf_ptr++;    /* write the attribute */
  }
}


/* Clear the screen. */
void cls(void)
{
  union REGS r;

  r.h.ah = 6; /* screen scroll code */
  r.h.al = 0; /* clear screen code */
  r.h.ch = 0; /* start row */
  r.h.cl = 0; /* start column */
  r.h.dh = 24; /* end row */
  r.h.dl = 79; /* end column */
  r.h.bh = 7;  /* blank line is black */
  int86(0x10, &r, &r);
}

/* Send the cursor to x,y. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Returns the current video mode. */
video_mode(void)
{
  union REGS r;

  r.h.ah = 15;  /* get video mode */
  return int86(0x10, &r, &r) & 255;
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}


