/*--------------------------------------------------------------------*/
/*  File: SAVE.CPP                                                    */
/*                                                                    */
/*  Purpose: class for saving/retrieving values                       */
/*                                                                    */
/*  Usage: in main(char **argv , int argc) define instance of         */
/*                                                                    */
/*  class Parameters: Parameters parameters(argv);                    */
/*  This initializes the filename                                     */
/*  Use then parameters.LoadFixed() to retrieve stored parameters     */
/*  and parameters.SaveFixed() to store parameters to disk            */
/*                                                                    */
/*  Parameters are stored in a file.                                  */
/*                                                                    */
/*  It is assumed that the EXE-file of the program is called          */
/*  "NNNNvvv.EXE" where NNNNN = 5 characters name, vvv = 3 digits     */
/*  version number (e.g. PROG100.EXE).                                */
/*                                                                    */
/*  The parameter file name will then be "NNNNvvv.PAR"                */
/*                                                                    */
/*  To make updating program versions easier,Parameters::LoadFixed()  */
/*  first looks for a file with the calling EXE-file's version number */
/*  If it doesn't succeed (e.g. an update with a higher version       */
/*  number is called the first time), it will search for the para-    */
/*  meter file with the highest available version number.             */
/*                                                                    */
/*  Parameter files are stored in the same directory as the           */
/*  calling EXE-file.                                                 */
/*                                                                    */
/*  Parameters are all variables (independent of type) declared       */
/*  AND INITIALIZED between char STARTSAVE[] and char ENDSAVE[]       */
/*  (see file SAVE.VAR).                                              */
/*                                                                    */
/*  To make a variable/array/struct permanent, just insert it         */
/*  between those two variables. Note that it has to be initialized!  */
/*  When adding a new permanent variable, make sure it is inserted    */
/*  at the end of the list (just before ENDSAVE[]) otherwise          */
/*  old parameter files will corrupt the data because there is no     */
/*  check on the sequence of variables.                               */
/*                                                                    */
/*  There is a check for file length and a checksum, but the checksum */
/*  can only be calculated on the default values, and if all variables*/
/*  are initialized with 0, the checksum test will not find out       */
/*  a mix-up.                                                         */
/*                                                                    */
/*                                                                    */
/*  This code is public domain and can be used freely.                */
/*  There is no guarantee that this code is bug-free!                 */
/*  You use it at your own risk.                                      */
/*                                                                    */
/*  (c)Sagner Software & Systeme                                      */
/*  PO Box 19 05 46                                                   */
/*  D 42705 Solingen , Germany                                        */
/*                                                                    */
/*  CIS 100022,2357                                                   */
/*                                                                    */
/*                                                                    */
/*--------------------------------------------------------------------*/

#include <ttypes.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <io.h>
#include <stdio.h>
#include <mem.h>
#include <string.h>
#include <stdlib.h>
#include <dir.h>
#include <dos.h>

/*--------------------------------------------------------------------*/

//#define SAVEMAIN

//#include "save.h"
/*--------------------------------------------------------------------*/

#define SAVEMAIN

#include "save.var"

#include "\bcutil\save.h"

/*--------------------------------------------------------------------*/

Parameters::Parameters(void)
{
	sprintf(SaveFileName , "SAVABLE.PAR");

	startptr = &STARTSAVE[0];
	endptr = &ENDSAVE[strlen(ENDSAVE)];
}

/*--------------------------------------------------------------------*/

Parameters::Parameters(char** argv)
{
	char drive[3] , dir[80] , name[10] , ext[5];

	fnsplit(argv[0] , drive , dir , name , ext);

	fnmerge(SaveFileName , drive , dir , name , "par");

	startptr = &STARTSAVE[0];
	endptr = &ENDSAVE[strlen(ENDSAVE)];



}

/*--------------------------------------------------------------------*/

Parameters::~Parameters()
{

}

/*--------------------------------------------------------------------*/

unsigned int Parameters::calcCkSum(void *BufStart , unsigned int BufLength)
{
	unsigned int CkSum = 0;
	unsigned int i;
	char *Ptr;
	char c;

	Ptr = (char*)BufStart;

	for(i = 0 ; i < BufLength ; i++)
	{
		memcpy(&c , Ptr+i , 1);
		CkSum = CkSum ^ c;

	}

	return(CkSum);
}

/*--------------------------------------------------------------------*/

void Parameters::SaveFixed(void)
{
	int handle;
	char TempBuf[100];

	_dos_setfileattr(SaveFileName , _A_NORMAL);

	handle = open(SaveFileName , O_RDWR | O_BINARY , S_IREAD | S_IWRITE);

	if (handle == -1)
	{
		handle = open(SaveFileName , O_CREAT | O_BINARY| O_RDWR , S_IREAD | S_IWRITE) ;

		close(handle);

		handle = open(SaveFileName , O_RDWR | O_BINARY , S_IREAD | S_IWRITE);

		if(handle == -1)
		{
			perror("cannot open parameter file");
		}
	}

	if(handle != -1)
	{
		memcpy(TempBuf , &ParamCkSum , sizeof(ParamCkSum));

		write (handle , TempBuf , sizeof(ParamCkSum));

		memcpy(TempBuf , &ParamLength , sizeof(ParamLength));

		write (handle , TempBuf , sizeof(ParamLength));

		write (handle , startptr ,  endptr - startptr);
	}
	else
	{
		perror("problem opening/creating parameter file");
	}

	close(handle);

	_dos_setfileattr(SaveFileName , _A_RDONLY);
}


/*--------------------------------------------------------------------*/


void Parameters::LoadFixed(void)
{
	int handle;
	unsigned int CkBuf1 , CkBuf2;
	unsigned long int i;
	char TempBuf[100];
	char ConvBuf[10];
	char SavePath[80];

	int VerNbr;

	char drive[3] , dir[80] , name[10] , ext[5];

	ParamLength = endptr - startptr;

	ParamCkSum = calcCkSum(startptr , ParamLength);


	strcpy(SavePath , SaveFileName);

	_dos_setfileattr(SavePath , _A_NORMAL);

	handle = open(SavePath , O_RDWR | O_BINARY , S_IREAD);




	if (handle == -1)
	{
		struct ffblk fblock;

		memcpy(ConvBuf , &name[strlen(name) - 3]  , 3);
		ConvBuf[3] = '\0';


		VerNbr = 0;

		memcpy(&name[strlen(name) - 3] , "???" , 3);

		fnmerge(SavePath , drive , dir , name , "par");


		if(findfirst ( SavePath , &fblock, FA_ARCH)==0)
		{
			do
			{
				fnsplit(fblock.ff_name , drive , dir , name , ext);

				memcpy(ConvBuf , &name[strlen(name) - 3]  , 3);
				ConvBuf[3] = '\0';

				if(atoi(ConvBuf) > VerNbr)
				{
					VerNbr = atoi(ConvBuf);
				}
			}

			while(findnext ( &fblock) == 0);

			fnsplit(SavePath , drive , dir , name , ext);

			sprintf(ConvBuf , "%3.3i" , VerNbr);

			memcpy(&name[strlen(name) - 3] , ConvBuf , 3);

			fnmerge(SavePath , drive , dir , name , "par");

			handle = open(SavePath , O_RDWR | O_BINARY , S_IREAD);


		}

		if(handle == -1)
		{
			perror("cannot open parameter file");
			fixedLoaded = 1;
		}
	}

	if(handle != -1)
	{

		read (handle , TempBuf , sizeof(CkBuf1));
		memcpy(&CkBuf1 , TempBuf , sizeof(CkBuf1));

		read (handle , TempBuf , sizeof(CkBuf2));
		memcpy(&CkBuf2 , TempBuf , sizeof(CkBuf2));


		if((CkBuf1 == ParamCkSum) && (CkBuf2 == ParamLength))
		{
			read (handle , &STARTSAVE[0] , ParamLength);
			fixedLoaded = 1;
		}
		else
		{
			fixedLoaded = 0;
			printf("\nParameter-Datei inkonsistent:");
			printf("\nCRC-Soll: %i  Ist: %i" , ParamCkSum , CkBuf1);
			printf("\nDateilnge Soll: %i  Ist: %i" , ParamLength , CkBuf2);

			for(i=0 ; i < 3000000 ; i++); /* wait here */

			read (handle , &STARTSAVE[0] , ParamLength);
			fixedLoaded = 1;

		}

	}
	close(handle);

	_dos_setfileattr(SavePath , _A_RDONLY);
}

/*--------------------------------------------------------------------*/
