
/* A refined print-screen untility which can be called from
   your application programs.  (Shown here with
   a main() function for demonstration purposes.)

   This version allows you to interactively specify 
   a region of the screen to be printed by using
   the cursor keys.  

   It also allows you to specify the graphics density
   you desire.
*/

#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.  If you use a different C compiler, refer
    to the text.
*/

#include "stdio.h"
#include "stdlib.h"
#include "bios.h"
#include "dos.h"


/* These values work for video graphics mode 16;
   you may need to change them for other modes. 
*/
#define RED 12
#define GREEN 10
#define WHITE 15

/* these values are from mode 16 graphics only */
#define XDIM 639
#define YDIM 349

#define SINGLE_WIDTH 480
#define DOUBLE_WIDTH 960
#define QUAD_WIDTH   1920

#define SINGLE 0
#define DOUBLE 1
#define QUAD   2

void print(char ch);
void set_graphics(int cols, int density);
void reset(void);

void print_scr(int startx, int starty, int endx, int endy,
               int pagex, int pagey, int density);

void mode(int mode_code);
void putpoint(int x, int y, int color, int how);
char getpoint(int x, int y);
char getkey(void);

void box(int startx, int starty,
         int endx, int endy, int color, int how);

void def_region(int *startx, int *starty, 
                int *endx, int *endy);
void locator(int x, int y, int color); 

char far *egabase;

main(void)
{
  int startx, starty, endx, endy;

#ifdef TURBOC
  egabase = (char far *) MK_FP(0xA000, 0000);
#endif
#ifdef MICROSOFTC
  egabase = (char far *) 0xA0000000;
#endif

  mode(16);

  printf("this is a test");
  box(0, 20, 20, 40, WHITE, 0);
  box(20, 50, 50, 100, WHITE, 0);
  box(10, 12, 100, 30, WHITE, 0);
  box(40, 100, 200, 300, WHITE, 0);
  box(100, 40, 200, 90, WHITE, 0);
  box(150, 75, 500, 250, WHITE, 0);

  def_region(&startx, &starty, &endx, &endy);
  print_scr(startx, starty, endx, endy, 0, 0, SINGLE);
  print_scr(startx, starty, endx, endy, 0, 0, DOUBLE);
  print_scr(startx, starty, endx, endy, 0, 0, QUAD);
  reset();  
  mode(3);
  return 0;
}


/* This function prints the specified screen region at the
   specified page location.  

   Startx,starty are the coordinates of the upper left
   corner of the region of the screen to print; endx,endy
   specify the lower right corner.

   The starting location of the page to print the image
   is specified by pagex,pagey. The horizontal page
   location is in characters, the vertical is in lines 
   (1/6").  Upper left corner of page is 0, 0.

   The graphics density is specified by the density
   parameter.
*/
void print_scr(int startx, int starty, int endx, int endy,
               int pagex, int pagey, int density)
{
  register int i, x, y, px;
  int cols, color, sum;

  endx++;  endy++;
  cols = endx - startx;

  for(; pagey>=0; pagey--) print('\n');

  for(y = starty; y<endy; y+=8) {
    for(px=0; px<pagex; px++) print(' ');
    set_graphics(cols, density);
    for(x = startx; x<endx; x++) {
      sum = 0;
      for(i=0; i<8; i++) {
        if(y+i < endy) {
          color = getpoint(x, y+i);
          if(color) sum += 1<<(7-i);
        }
      }
      print(sum);
    }
    print('\n');
  }
}

/* Interactively define a region using the arrow keys.
   When the locator is at the upper left corner of the 
   region you want printed, press F1.  Next, move
   the locator to the lower right corner of the region
   and press F2.  By default, each time you press an arrow
   key, the locator moves 1 pixel.  However, by pressing
   F3, you can cause the locator to move in increments of
   5 pixels.  F3 acts as a toggle, switching between 
   increments of 1 and 5 each time it is pressed.

   To print the region, press F4.
*/
void def_region(int *startx, int *starty, 
                int *endx, int *endy)
{
  register int x, y;
  char ch;
  int inc;

  inc = 1;

  *startx = *starty = *endx = *endy = 0;
  x = 0; y = 0;
  do {
    /* show a box around the region selected for
       printing
    */
    box(*startx, *starty, *endx, *endy, RED, 0x18);
    locator(x, y, GREEN); /* display locator */
    ch = getkey();
    locator(x, y, GREEN); /* erase locator */
    /* erase box */
    box(*startx, *starty, *endx, *endy, RED, 0x18);

    switch(ch) {
      case 75: /* left */
        x -= inc;
	break;
      case 77: /* right */
        x += inc;
	break;
      case 72: /* up */
        y -= inc;
	break;
      case 80: /* down */
        y += inc;
	break;
      case 71: /* up left */
	x -= inc; y -= inc;
	break;
      case 73: /* up right */
	x += inc; y -= inc;
	break;
      case 79: /* down left*/
	x -= inc; y += inc;
	break;
      case 81: /* down right */
	x += inc; y += inc;
	break;
      case 59: /* F1 define upper right corner */
        *startx = x;
        *starty = y;
	break;
      case 60: /* F2 - define lower left corner */
        *endx = x;
        *endy = y;
	break;
      case 61: /* F3 - fast/slow */
        if(inc==1) inc = 5;
        else inc = 1;
        break;
    }
    if(x < 0) x++;  if(x > XDIM-1) x--;
    if(y < 0) y++;  if(y > YDIM-1) y--;

  } while(ch != 62); /* F4 - quit */
}

/* Display a small crosshairs style locator
   using XOR mode. */
void locator(int x, int y, int color)
{
  putpoint(x, y, color, 0x18); 
  putpoint(x+1, y, color, 0x18); 
  putpoint(x-1, y, color, 0x18); 
  putpoint(x, y+1, color, 0x18); 
  putpoint(x, y-1, color, 0x18); 
}

/* Set the graphics density as specified. */
void set_graphics(int cols, int density)
{
  union {
    unsigned char c[2];
    unsigned int i;
  } u;
  char den_code;

  u.i = cols;
  
  switch(density) {
    case SINGLE: den_code = 75; 
      break;
    case DOUBLE: den_code = 76;
      break;
    case QUAD: den_code = 90;
      break;
  }   

  print(27); print(65); print(8); /* 8/72 line spacing */
  print(27); print(den_code); print(u.c[0]); print(u.c[1]);
}

/* Reset the printer to defaults. */
void reset(void)
{
  print(27); print(64);
}

/* Send a character to the printer. */
void print(char ch)
{
#ifdef TURBOC
  biosprint(0, ch, 0);
#endif
#ifdef MICROSOFTC
  _bios_printer(0, 0, ch);
#endif
}

/**************************************************
 Video functions
**************************************************/

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Returns the value at the specified pixel. */
char getpoint(int x, int y)
{
  union REGS r;

  r.h.ah = 13;
  r.x.dx = y;
  r.x.cx = x;
  r.h.bh = 0;
  int86(16, &r, &r);
  return r.h.al;
}

/* This version of putpoint will work for all video
   adapters but is incredibly slow!
*/
/*
void putpoint(int x, int y, int color, int how)
{
  union REGS r;

  if(how==0x18) color = color | 128;
  r.h.bh = 0;
  r.h.ah = 12;
  r.h.al = color;
  r.x.dx = y;
  r.x.cx = x;
  int86(16, &r, &r);
}
*/

/* This function sets the specified pixel to the 
   specified color using an EGA/VGA video adapter.
   The value of how may be one of these:

   action        value

   overwrite       0
   XOR             0x18
   AND             8
   OR              0x10
   
*/
#define ENABLE 0x0F
#define INDEXREG 0x3CE
#define VALREG 0x3CF

#define OUTINDEX(index, val) outp((INDEXREG), (index));\
                             outp((VALREG), (val));

#define WIDTH 80L

/* These range values assume EGA mode 16 */
#define XMAX 639
#define YMAX 349
#define XMIN 0
#define YMIN 0

void putpoint(int x, int y, int color, int how)
{
  register unsigned char mask = 0x80;
  register char far *base;
  unsigned dummy;

  /* if you want range checking at this point, activate
     this line of code.  It will slow down the function
     however.
  */
  /*if(x<XMIN || x>XMAX || y<YMIN || y>YMAX) return;*/

  base = (char far *) (egabase 
         + ( (long) y * WIDTH + (long) x/8L ));

  mask >>= x % 8;

  /* This causes the memory READ necessary to
     load the EGA/VGA's internal registers.
  */
  dummy = *base;

  OUTINDEX(0, color);
  OUTINDEX(1, ENABLE);
  OUTINDEX(3, how);
  OUTINDEX(8, mask);

  *base = 1;

  OUTINDEX(0, 0);
  OUTINDEX(1, 0);
  OUTINDEX(3, 0); 
  OUTINDEX(8, 0xff);
}


/* Draw a box in the foreground color. The
   how parameter determines how the box will
   be displayed on the screen.  See putpoint()
   for details. 
*/
void box(int startx, int starty,
         int endx, int endy, int color, int how)
{
  register int x, y;

  for(x=startx; x<endx; x++)
    putpoint(x, endy, color, how);
  for(y=starty; y<endy; y++)
    putpoint(endx, y, color, how);
  for(x=startx; x<endx; x++)
    putpoint(x, starty, color, how);
  for(y=starty; y<endy; y++)
    putpoint(startx, y, color, how);
}
  

/* Return the scan code of the key pressed. 
   This function is compatible with Turbo/Borland C/C++ and
   Microsoft C/C++ (be sure to #define the appropriate
   symbol).  If you use a different compiler, consult
   the text.
*/
char getkey(void)
{
  union key {
    int i;
    char ch[2];
  } k;

#ifdef TURBOC
  k.i = bioskey(0);
#endif
#ifdef MICROSOFTC
  k.i = _bios_keybrd(0);
#endif
  return k.ch[1];
}
