

/* Demonstration program for printing text in color, 
   sizing the cursor, and scrolling a window.
*/
#include "stdio.h"
#include "dos.h"

#define BLUE 	   1
#define GREEN      2
#define RED 	   4
#define INTENSE    8
#define BLUE_BACK  16
#define GREEN_BACK 32
#define RED_BACK   64
#define BLINK 	   128

#define UP   0
#define DOWN 1

void color_puts(char *s, char color);
void read_cursor_xy(char *x, char *y);
void mode(int mode_code), goto_xy(int x, int y);
void size_cursor(char start, char end);
void scroll_window(char startx, char starty,
                   char endx, char endy, char lines, 
                   char direct);

main(void)
{
  int i, j;

  /* set video mode to 80 column color */
  mode(3); /* color text */

  /* make the cursor a small block */
  size_cursor(4, 7);

  /* write in color */
  goto_xy(0, 0);
  color_puts("this is a test\n", BLUE | RED | INTENSE);
  
  for(i=0; i<22; i++ ) {
    for(j=0; j<79; j++) 
      printf("%c", i+'a');
    printf("\n");
  }
  getchar();
  /* scroll part of the screen down 3 lines */
  scroll_window(10, 10, 50, 15, 3, DOWN);
  getchar();

  /* reset the cursor */
  size_cursor(6, 7);

  return 0;
}

/* Print a string in color. */
void color_puts(char *s, char color)
{
  union REGS r;
  char x, y;

  read_cursor_xy(&x, &y); /* get current cursor position */  
  while(*s) {
    if(*s=='\n') { /* process a newline */
      printf("\n");
      s++;
      x = 0; y++; /* advance to next line */
      continue;
    }

    r.h.ah = 9; /* write character and attribute */
    r.h.al = *s++; /* character to write */
    r.h.bl = color; /* color attribute */
    r.h.bh = 0; /* video page 0 */
    r.x.cx = 1; /* write it one time */
    int86(0x10, &r, &r);
    x++;
    goto_xy(x, y); /* advance the cursor */
  }
}

/* Read the current cursor position. */
void read_cursor_xy(char *x, char *y)
{
  union REGS r;
  
  r.h.ah = 3; /* read cursor postion */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
  *x = r.h.dl;
  *y = r.h.dh;
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Send the cursor to the specified X,Y position. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Set the size of the cursor. */
void size_cursor(char start, char end)
{
  union REGS r;

  r.h.ah = 1; /* cursor sizing function */
  r.h.ch = start;
  r.h.cl = end;
  int86(0x10, &r, &r);
}

/* Scroll a window up or down. */
void scroll_window(
  char startx, char starty, /* upper left corner */
  char endx, char endy, /* lower right corner */
  char lines, /* number of lines to scroll */
  char direct /* up or down */
)
{
  union REGS r;

  if(direct==UP) r.h.ah = 6; /* scroll up */
  else r.h.ah = 7; /* scroll down */

  r.h.al = lines; 
  r.h.ch = starty;
  r.h.cl = startx;
  r.h.dh = endy;
  r.h.dl = endx;
  r.h.bh = 0; /* display attribute */
  int86(0x10, &r, &r);
}


