
/* This program demonstrates how a graphics image
   can be built in RAM and then printed to the printer.
*/
   
#define TURBOC
/*  If you use Microsoft C/C++, define MICROSOFTC instead 
    of TURBOC.  If you use a different C compiler, refer
    to the text.
*/

#include "stdio.h"
#include "stdlib.h"
#include "bios.h"
#include "dos.h"


/* These values work for video graphics mode 16; 
   you may need to change them for other modes.
*/
#define RED 12
#define GREEN 10
#define WHITE 15

#define XDIM 639
#define YDIM 349

#define SINGLE 0
#define DOUBLE 1
#define QUAD   2

void print(char ch);
void set_graphics(int cols, int density);
void reset(void);

void print_buf(char *buf,
               int startx, int starty, int endx, int endy,
               int pagex, int pagey, int density);

void rambox(char *buf, int startx, int starty,
            int endx, int endy, int color);

void putbuf(char *buf, int x, int y, int val);
unsigned char getbuf(char *buf, int x, int y);

char far *egabase;

char *buf;

main(void)
{
  unsigned int i;

#ifdef TURBOC
  egabase = (char far *) MK_FP(0xA000, 0000);
#endif
#ifdef MICROSOFTC
  egabase = (char far *) 0xA0000000;
#endif

  buf = malloc((unsigned)47520);
  if(!buf) exit(1);
  for(i=0; i<(unsigned)47520; i++) buf[i] = 0;

  rambox(buf, 0, 20, 20, 40, 1);
  rambox(buf, 20, 50, 50, 100, 1);
  rambox(buf, 10, 12, 100, 30, 1);
  rambox(buf, 40, 100, 200, 300, 1);
  rambox(buf, 400, 150, 450, 175, 1);
  rambox(buf, 100, 40, 200, 90, 1);
  rambox(buf, 150, 75, 200, 750, 1);

  printf("printing...");

  print_buf(buf, 0, 0, 479, 791, 0, 0, SINGLE);
  reset();  
  return 0;
}


/* This function prints the specified region of the 
   specified buffer at the specified page location.

   Startx,starty are the coordinates of the upper left
   corner of the region of the buffer to print; endx,endy
   specify the lower right corner.

   The starting location of the page to print the image
   is specified by pagex,pagey. The horizontal page
   location is in characters, the vertical is in
   lines (1/6").  Upper left corner of the page is 0, 0.

   The graphics density is specified by the density
   parameter.

*/
void print_buf(char *buf,
               int startx, int starty, int endx, int endy,
               int pagex, int pagey, int density)
{
  register int i, x, y, px;
  int cols, color, sum;

  endx++;  endy++;
  cols = endx - startx;

  for(; pagey>=0; pagey--) print('\n');

  for(y = starty; y<endy; y+=8) {
    for(px=0; px<pagex; px++) print(' ');
    set_graphics(cols, density);
    for(x = startx; x<endx; x++) {
      sum = 0;
      for(i=0; i<8; i++) {
        if(y+i < endy) {
          color = getbuf(buf, x, y+i);
          if(color) sum += 1<<(7-i);
        }
      }
      print(sum);
    }
    print('\n');
  }
}

/* Write a value to the specified buffer
   at the specified location.
*/
void putbuf(char *buf, int x, int y, int val)
{ 
  /* compute correct byte */
  buf += y * 60;
  buf += x / 8;

  val &= 1;

  /* write correct bit within byte */
  if(val) *buf |= val << (x % 8); /* write a 1 */
  else *buf &= val << (x%8); /* write a 0 */
}

/* Read value from the specified buffer at the
   specified location. 
*/
unsigned char getbuf(char *buf, int x, int y)
{
  unsigned char mask;

  mask = 1;

  /* computer correct byte */
  buf += y * 60;
  buf += x / 8;
  
  mask <<= x % 8;  /* set mask for proper bit */
  return (*buf & mask); /* return value */
}

/* Set the graphics density as specified. */
void set_graphics(int cols, int density)
{
  union {
    unsigned char c[2];
    unsigned int i;
  } u;
  char den_code;

  u.i = cols;
  
  switch(density) {
    case SINGLE: den_code = 75; 
      break;
    case DOUBLE: den_code = 76;
      break;
    case QUAD: den_code = 90;
      break;
  }   

  print(27); print(65); print(8); /* 8/72 line spacing */
  print(27); print(den_code); print(u.c[0]); print(u.c[1]);
}

/* Reset the printer to defaults. */
void reset(void)
{
  print(27); print(64);
}

/* Send a character to the printer. */
void print(char ch)
{
#ifdef TURBOC
  biosprint(0, ch, 0);
#endif
#ifdef MICROSOFTC
  _bios_printer(0, 0, ch);
#endif
}

/* Construct a box in RAM. 
*/
void rambox(char *buf, int startx, int starty,
            int endx, int endy, int color)
{
  register int x, y;

  for(x=startx; x<endx; x++)
    putbuf(buf, x, endy, color);
  for(y=starty; y<endy; y++)
    putbuf(buf, endx, y, color);
  for(x=startx; x<endx; x++)
    putbuf(buf, x, starty, color);
  for(y=starty; y<endy; y++)
    putbuf(buf, startx, y, color);
}
  


