

/* Animated sprite example. */

#include "dos.h"
#include "stdio.h"
#include "stdlib.h"

#define XMAX 319
#define YMAX 199

void mempoint(int x, int y, int color);
void palette(int pnum);
void mode(int mode_code);
void display_object(int ob[][5], int lines);
void line(int startx, int starty, int endx, int endy, 
         int color);
void update_object(int ob[][5], int x, int y,
                   register int sides);
int is_legal(int ob[][5], int x, int y, int sides);
int readkey(void);

int sprite[2][5] = {
  3, 0, 3, 5, 1,
  0, 3, 5, 3, 1
};

main(void)
{
  union k{
    char c[2];
    int i;
  } key;

  int deltax = 0, deltay = 0; /* direction of movement */
 
  mode(4);  /* switch to mode 4 graphics */
  palette(0); /* palette 0 */
  
  display_object(sprite, 2);
  do {
    deltay = deltax = 0;
    key.i = readkey();
    if(!key.c[0]) switch(key.c[1]) {
      case 75: /* left */
        deltax = -1;
        break;
      case 77: /* right */
        deltax = 1;
        break;
      case 72: /* up */
        deltay = -1;
        break;
      case 80: /* down */
        deltay = 1;
        break;
      case 71: /* up left */
        deltax = -1; deltay = -1;
        break;
      case 73: /* up right */
        deltay = -1; deltax = 1;
        break;
      case 79: /* down left*/
        deltay = 1; deltax = -1;
        break;
      case 81: /* down right */
        deltax = 1; deltay = 1;
        break;
    }
    /* erase sprite's current position */
    display_object(sprite, 2);

    /* if move is legal, update object definition */
    if(is_legal(sprite, deltax, deltay, 2))
      update_object(sprite, deltax, deltay, 2);

    /* redisplay sprite in new position */
    display_object(sprite, 2);
  } while (key.c[0] != 'q');

  mode(3);
  return 0;
}

/* Set the palette. */
void palette(int pnum)
{
  union REGS r;

  r.h.bh = 1;   /* code for palette select */
  r.h.bl = pnum;
  r.h.ah = 11;  /* set palette function */
  int86(0x10, &r, &r);
}

/* Set the video mode. */
void mode(int mode_code)
{
  union REGS r;

  r.h.al = mode_code;
  r.h.ah = 0;
  int86(0x10, &r, &r);
}

/* Draw a line in specified color
   using Bresenham's integer based algorithm.
*/
void line(int startx, int starty, /* upper left corner */
          int endx, int endy, /* lower right corner */
          int color) /* color */
{
  register int t, distance;
  int x=0, y=0, delta_x, delta_y;
  int incx, incy;

  /* Compute the distances in both directions. */
  delta_x = endx-startx;
  delta_y = endy-starty;

  /* Compute the direction of the increment.
     An increment of 0 means either a vertical
     or horizontal line.
  */
  if(delta_x>0) incx = 1;
  else if(delta_x==0) incx = 0;
  else incx = -1;

  if(delta_y>0) incy = 1;
  else if(delta_y==0) incy = 0;
  else incy = -1;

  /* determine which distance is greater */
  delta_x = abs(delta_x);
  delta_y = abs(delta_y);
  if(delta_x>delta_y) distance = delta_x;
  else distance = delta_y;

  /* draw the line */
  for(t=0; t<=distance+1; t++) {
    mempoint(startx, starty, color);
    x += delta_x;
    y += delta_y;  
    if(x>distance) {
      x -= distance;
      startx += incx;
    }
    if(y>distance) {
      y -= distance;
      starty += incy;
    }
  }
}

/* Write a point directly to mode 4 graphics RAM. */
void mempoint(int x, int y, /* coordinate of point */
              int color) /* color */
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;
  char xor; /* xor color in or overwrite */

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 0xFF3F;  /* 11111111 00111111 in binary */

  /* check range */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return;

  xor = color & 128; /* see if xor mode is set */
  color = color & 127; /* mask off high bit */
 
  /* set bit_mask and color bits to the right location */
  bit_position = x%4;
  color <<= 2*(3-bit_position);
  bit_mask.i >>= 2*bit_position;


  /* find the correct byte in screen memory */
  index = y*40 + (x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* write the color */
  if(!xor) { /* overwrite mode */
    t = *(ptr+index) & bit_mask.c[0];
    *(ptr+index) = t | color;
  }
  else { /* xor mode */
    t = *(ptr+index) | (char) 0;
    *(ptr+index) = t ^ color;
  }
}

/* Read byte directly from the video RAM in mode 4. */
unsigned char read_point(int x, int y)
{
  union mask {
    char c[2];
    int i;
  } bit_mask;
  int index, bit_position;
  unsigned char t;

  /* pointer to mode 4 graphics RAM */
  char far *ptr = (char far *) 0xB8000000; 

  bit_mask.i = 3;       /* 11111111 00111111 in binary */

  /* check range for mode 4 */
  if(x<0 || x>XMAX || y<0 || y>YMAX) return 0;

  /* set bit_mask and color_code bits to the right location */
  bit_position = x%4;
  bit_mask.i <<= 2*(3-bit_position);
  /* find the correct byte in screen memory */
  index = y*40 +(x >> 2);
  if(y % 2) index += 8152; /* if odd use 2nd bank */

  /* read the color */
  t = *(ptr+index) & bit_mask.c[0];
  t >>= 2*(3-bit_position);
  return t;
}

/* Display an object. */
void display_object(int ob[][5], int lines)
{
  register int i;

  for(i=0; i<lines; i++)
    line(ob[i][0], ob[i][1],
         ob[i][2], ob[i][3], ob[i][4] | 128);
}

/* Update an object's position as specified in x, y. */
void update_object(int ob[][5], /* object */
  int x, int y, /* amount to update */
  register int sides) /* number of sides */
{
  sides--;
  for( ; sides>=0; sides--) {
    ob[sides][0] += x;
    ob[sides][1] += y;
    ob[sides][2] += x;
    ob[sides][3] += y;
  }
}

/* See if a prospective move is legal. 
   Returns 1 if legal; 0 otherwise. 
*/
is_legal(int ob[][5], /* object */
  int x, int y, /* increment to move */
  int sides) /* number of sides the object has */
{
  if(x==0 && y==0) return 1;
  sides--;
  for( ; sides>=0; sides--) {
    /* check for out of range */
    if(ob[sides][0]+x>XMAX || ob[sides][1]+y >YMAX) return 0;
    if(ob[sides][2]+x<0 || ob[sides][3]+y<0) return 0;
    /* check for obstacle */
    if(read_point(ob[sides][0]+x, ob[sides][1] + y)==2) return 0;
    if(read_point(ob[sides][2]+x, ob[sides][3] + y)==2) return 0;
  }
  return 1;
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}

