
/* A minimal two-task kernel for DOS. */
#include "stdio.h"
#include "dos.h"
#include "alloc.h"

typedef struct int_regs {
  unsigned bp;
  unsigned di;
  unsigned si;
  unsigned ds;
  unsigned es;
  unsigned dx;
  unsigned cx;
  unsigned bx;
  unsigned ax;
  unsigned ip;
  unsigned cs;
  unsigned flags;
} ;


struct task_struct {
  unsigned sp;
  unsigned ss;
  unsigned char *stck;
} tasks[2];

typedef int (far *task_type) (void);

void interrupt multitask(void);
void interrupt int8_task_switch(void);
void interrupt (*old_int8)(void);
void make_task(task_type task, unsigned stck, int id);
void f2(void), f1(void);

unsigned oldss, oldsp;
int tswitch = 0;
char tasking = 1;

int i=0, j=0;

main(void)
{
  /* Initialize the tasks */
  make_task((task_type) f1, 1024, 0);
  make_task((task_type) f2, 1024, 1);
  
  /* start multitasking */
  multitask(); 
  printf("\ndone\n");
  printf("%d %d", i, j);

  return 0;
}
  
/* This intercepts timer interrupt 8 and performs
   a task switch.
*/
void interrupt int8_task_switch(void)
{

  (*old_int8)(); /* call old int8 function */

  tasks[tswitch].ss = _SS;  /* save old task stack */
  tasks[tswitch].sp = _SP;

  tswitch = !tswitch;  /* switch tasks */
  
  _SS = tasks[tswitch].ss;  /* set up stack of new task */
  _SP = tasks[tswitch].sp;

  if(!tasking) { /* stop tasking */
    disable();
      _SS = oldss;
      _SP = oldsp;
      setvect(8, old_int8);
      free(tasks[0].stck);
      free(tasks[1].stck);
    enable();
  }
}

/* Define a new task. */
void make_task(task_type task, unsigned stck_size, 
               int id)
{
  struct int_regs *r;

  disable();
    tasks[id].stck = malloc(stck_size +
                     sizeof(struct int_regs));

    r = (struct int_regs *) tasks[id].stck + 
         stck_size - sizeof(struct int_regs);

    /* initialize task stack */
    tasks[id].sp = FP_OFF((struct int_regs far *) r);
    tasks[id].ss = FP_SEG((struct int_regs far *) r);

    /* set up task code segment and IP */
    r->cs = FP_SEG(task);
    r->ip = FP_OFF(task);

    /* set up DS and ES segments */
    r->ds = _DS;  
    r->es = _ES;  

    /* enable interrupts - see text */
    r->flags = r->flags | 0x200;
  enable();
}

/* Start up the multitasking kernel. */
void interrupt multitask(void)
{
  disable();
    /* reset interrupt 8 */
    old_int8 = getvect(8);
    setvect(8, int8_task_switch);
    
    /* save original stack segment and pointer */
    oldss = _SS;
    oldsp = _SP;
  
    /* reroute stack to first task */
    _SS = tasks[tswitch].ss;
    _SP = tasks[tswitch].sp;
  enable();
  /* task will begin running upon return */
}

/* Task 1 */
void f1(void)
{
  for(;;i++) {
    if(!(i%10)) printf("%d %d\n", i, j);
    if(i==1000) tasking = 0;
  }
}

/* Task 2 */
void f2(void)
{
  for(;;j++) ;
}
