
/* Terminate and stay resident skeleton and sample
   applications. 
*/

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "dos.h"
#include "ctype.h"
#include "fcntl.h"
#include "conio.h"
#include "bios.h"
#include "io.h"

#define BORDER 1
#define MAX_FRAME 3
#define REV_VID 0x70
#define NORM_VID 7
#define VID_SIZE 4000
#define STK_SIZE 0x2000

/* window function prototypes */
void save_video(int num), restore_video(int num);
void write_string(int x, int y, char *p, int attrib);
void write_char(int x, int y, char ch, int attrib);
void tsrdisplay_header(int num), tsrdraw_border(int num);
void tsrwindow_gets(int num , char *s);
void tsrwindow_cleol(int num), tsrwindow(int num);
void tsrwindow_cls(int num),  set_vid_mem(void);
void tsrdeactive(int num);
void tsrwindow_bksp(int num);
void goto_xy(int x, int y);
int make_tsrwindow(int num, char *header, int startx,
                int starty, int endx, int endy, int border);
int tsrwindow_xy(int num, int x, int y);
int tsrwindow_puts(int num, char *str);
int video_mode(void), tsrwindow_putchar(int num, char ch);
int tsrwindow_getche(int num);
int tsrwindow_upline(int num), tsrwindow_downline(int num);
int readkey(void);

/* TSR function prototypes */
void cursor_pos(void);
void activate_tsr(void);
void interrupt tsr_keystroke(void);
void interrupt dos_idle(void);
void interrupt new_int8(void);
void interrupt (*old_int28)();
void interrupt (*old_int8)();
void interrupt (*old_key)();
void interrupt (*old_int64)();
void notepad(void), calc(void);
void code(void);
int push(int i), pop(void);

char far *vid_mem;
char video;

struct window_frame {
  int startx, endx, starty, endy; /* corners of window */
  int curx, cury; /* current cursor position in window */
  unsigned char *p; /* pointer to buffer */
  char *header; /* header message */
  int border; /* border on/off */
  int active; /* 1 if active, 0 if not */
} frame[MAX_FRAME];

char wp[VID_SIZE]; /* buffer to hold contents of the screen 
                      when saved by save_video() */

/* busy is set to 1 when the program is active, 0 otherwise */ 
char busy = 0;

unsigned char stck[STK_SIZE];
unsigned int sp, ss;

int old_row, old_col;
int key;
char far *dos_active;

main(void)
{
  union REGS r;
  struct SREGS s;

  /* see if already loaded */
  old_int64 = getvect(0x64); /* unused interrupt */
  if(!old_int64) setvect(0x64, new_int8); /* set a flag */
  else {
    printf("TSR already loaded.");
    return 1;
  }

  /* obtain the dos_active flag address */
  r.h.ah = 0x34;
  int86x(0x21, &r, &r, &s);
  dos_active = MK_FP(s.es, r.x.bx);

  /* get old ISR addresses */
  old_key = getvect(9); /* keystroke int */
  old_int28 = getvect(0x28);  /* int 28 */
  old_int8 = getvect(8);  /* timer interrupt */

  /* re-route interrupt vectors to new handlers */
  setvect(9, tsr_keystroke);
  setvect(0x28, dos_idle);
  setvect(8, new_int8);

  printf("F1: Notepad    F2: Calculator    F3: File Coder");
  printf("    F4: Deactivate\n");

  key = 0; 
  set_vid_mem();  /* get address of video memory */

  /* initialize all windows */
  make_tsrwindow(0, " Notepad [Esc to exit] ",
                 20, 5, 60, 16, BORDER); 
  make_tsrwindow(1, " Calculator ", 20, 8, 60, 12, BORDER); 
  make_tsrwindow(2, " File Coder ", 20, 8, 60, 10, BORDER); 

  keep(0, 2000); /* terminate and stay resident */

  return 0;
}

/* This is the function that intercepts the keystroke
   interrupt (int 9). */
void interrupt tsr_keystroke(void)
{
  int far *t2 = (int far *) 1050;  /* address of head pointer */
  char far *t = (char far *) 1050; /* address of head pointer */

  (*old_key)();  /* first, call old keystroke ISR */

  if(*t != *(t+2)) {/* if not empty */
    t += *t-30+5; /* advance to the character position */
    if(*t>=59 && *t<=62) {
      switch(*t) {  /* see what is in scan code */
        case 59: /* F1 - notepad */
          key = 1;
          break;
        case 60: /* F2 - calculator */
          key = 2;
          break;
        case 61: /* F3: file encrypter */
          key = 3;
          break;
        case 62: /* F4: deactivate TSR */
          key = 4;
          break;
      }
      *(t2+1) = *t2;  /* zero buffer */
    }
  }
}

/* New interrupt 8. */
void interrupt new_int8(void)
{
  (*old_int8)(); /* call original int 8 */

  /* If DOS is idle, TSR is not already active,
     and if a hot key has been hit, activate the TSR. */
  if(!*dos_active && !busy && key) 
        activate_tsr(); 
}

/* This function intercepts int 0x28  (the DOS-idle
   interrupt).
*/
void interrupt dos_idle(void)
{
  (*old_int28)();  /* call old int 0x28 ISR */
  if(!busy && key) activate_tsr();
}

/* Pop up the TSR. */
void activate_tsr(void)
{
  /* set up TSR stack */
  disable();  /* disable interrupts */
    ss = _SS;  
    sp = _SP;
    _SS = _DS;
    _SP = (unsigned) &stck[STK_SIZE-2];
  enable();   /* enable interrupts */

  cursor_pos();  /* get current cursor position */

  /* check video mode - don't pop up if not in
     an 80-column text mode
  */
  video = video_mode();
  if(!busy && (video==7 || video==3 || video==2)) {
    busy = !busy;  /* don't allow a second activation */
    switch(key) {
      case 1: notepad();
        break;
      case 2: calc();
        break;
      case 3: code();
        break;
    }
    busy = !busy;
  }
  if(key==4) /* deactivating TSR */
    write_string(old_col, old_row, "TSR deactivated", NORM_VID);

  goto_xy(old_col, old_row); /* reset cursor */

  /* restore old stack */
  disable();
    _SP = sp;
    _SS = ss; 
  enable();

  if(key==4) {  /* reset old ISR routines and free memory */
    setvect(8, old_int8);
    setvect(9, old_key);
    setvect(0x28, old_int28);
    setvect(0x64, old_int64);
    freemem(_psp);  /* free program block */
  } else  key = 0; 
}

/**********************************************************/
/* Pop-up window application functions.                   */
/**********************************************************/

/**********************************************************/
/* Stack based, postfix notation four-function calculator */
/**********************************************************/
#define MAX 100

int *p;   /* pointer into the stack */
int *tos; /* points to top of stack */
int *bos; /* points to bottom of stack */
int stack[MAX];

void calc(void)
{
  int answer;
  int a, b;
  char in[80], out[80];

  p = stack;
  tos = p; 
  bos = p+MAX-1;

  tsrwindow(1);
  do {
    tsrwindow_xy(1, 0, 0); 
    tsrwindow_cleol(1);
    tsrwindow_puts(1, ": "); /* calc prompt */
    tsrwindow_gets(1, in);
    tsrwindow_puts(1, "\n ");
    tsrwindow_cleol(1);
    switch(*in) {
      case '+':
        a = pop();
        b = pop();
        answer = a+b;
        push(a+b);
        break;
      case '-':
        a = pop();
        b = pop();
        answer = b-a;
        push(b-a);
        break;
      case '*':
        a = pop();
        b = pop();
        answer = b*a;
        push(b*a);
        break;
      case '/':
        a = pop();
        b=pop();
        if(a==0) {
          tsrwindow_puts(0, "divide by 0\n");
          break;
        }
        answer = b/a;
        push(b/a);
        break;
      default:
        push(atoi(in));
 	continue;
    }
    itoa(answer, out, 10);
    tsrwindow_puts(1, out);
  } while(*in);  
  tsrdeactive(1);      
}

/* Push a number on the stack.
   Returns 1 if successful, 0 if stack is full.
*/
push(int i)
{
  if(p>bos) return 0;
 
  *p=i;
  p++;
  return 1;
}

/* Retrieve top element from the stack.
   Returns 0 on stack underflows.
*/ 
pop(void)   
{
  p--;
  if(p<tos) {
    p++;
    return 0;
  }
  return *p;
}

/*********************************************************/
/* Pop-up note pad.                                      */
/*********************************************************/
#define MAX_NOTE 10 
char notes[MAX_NOTE][80];

void notepad(void)
{
  static first=1;
  register int i, j, k;
  union inkey {
    char ch[2];
    int i;
  } c;
  char ch;

  /* initialize notes array if necessary */
  if(first) {
    for(i=0; i<MAX_NOTE; i++)
      *notes[i] = '\0';
    first = !first;
  }

  tsrwindow(0);
  /* display the existing notes */
  for(i=0; i<MAX_NOTE; i++) {
    if(*notes[i]) tsrwindow_puts(0, notes[i]);
    tsrwindow_putchar(0, '\n');
  }

  tsrwindow_xy(0, 0, 0);

  for(i=0;;) {
    c.i = readkey(); 	 /* read the key */
    if(tolower(c.ch[0])==27) { /* Esc to quit */
      tsrdeactive(0);
      break;
    }

    /* if normal key */
    if(isprint(c.ch[0]) || c.ch[0]=='\b') { 
      tsrwindow_cleol(0);
      notes[i][0] = c.ch[0];
      j = 1; 
      tsrwindow_putchar(0, notes[i][0]);
      do {
	ch = tsrwindow_getche(0);
	if(ch=='\b') {
          if(j>0) {
            j--;
	    tsrwindow_bksp(0);
	  }
	} 
	else {
	  notes[i][j] = ch;
          j++;
        }
      } while(notes[i][j-1]!='\r');
      notes[i][j-1] = '\0';
      if(i<MAX_NOTE-1) i++;
      tsrwindow_putchar(0, '\n');
    }
    else {  /* is special key */
      switch(c.ch[1]) {
        case 72:  /* up arrow */
	  if(i>0) {
	    i--;
	    tsrwindow_upline(0);
	  }
	  break;
        case 80:  /* down arrow */
	  if(i<MAX_NOTE-1) {
            i++;
 	    tsrwindow_downline(0);
	  }
	  break;
        case 63: /* F5: clear the notepad */
          tsrwindow_cls(0);
          for(k=0; k<MAX_NOTE; k++) *notes[k] = '\0';
          tsrwindow_xy(0, 0, 0);
      }
    }
  }
}

/*********************************************************/
/* A simple file encryption function.                    */
/*********************************************************/
void code(void)
{
  int fd, num;
  char ch, fname[80];
  char far *old_dta;
  long sk;

  old_dta = getdta();  /* save old dta */
  setdta(MK_FP(_psp, 0x80)); /* construct new dta */

  tsrwindow(2);
  tsrwindow_xy(2, 0, 0);
  tsrwindow_puts(2, "Enter name for file: ");
  tsrwindow_gets(2, fname);

  if((fd = _open(fname, O_RDWR)) < 0) {
    tsrwindow_xy(2, 0, 0);
    tsrwindow_cleol(2);
    tsrwindow_puts(2, "Cannot open file.  Press a key.\n");
    tsrwindow_getche(2);
    tsrdeactive(2);
    setdta(old_dta); /* restore old dta */
    return;
  }

  num = 1;
  for(sk = 0; num==1 ; sk++) {
    lseek(fd, sk, SEEK_SET);
    num = _read(fd, &ch, 1);
    ch = ~ch; /* complement each byte */
    lseek(fd, sk, SEEK_SET);
    if(num==1) _write(fd, &ch , 1);
  }

  _close(fd);
  tsrdeactive(2);
  setdta(old_dta); /* restore old dta */
}

/*********************************************************/
/* Window functions.  These functions are adapted from   */
/* from those developed in Chapter 2.                    */
/*********************************************************/

/* Display a pull-down window. */
void tsrwindow(int num) /* window number */
{
  /* get active window */
  if(!frame[num].active) { /* not currently in use */ 
    save_video(num);       /* save the current screen */
    frame[num].active = 1; /* set active flag */
  }

  if(frame[num].border) tsrdraw_border(num);
  tsrdisplay_header(num); /* display the window */
}

/* Construct a pull down window frame.
   1 is returned if window frame can be constructed;
   otherwise 0 is returned.
*/
make_tsrwindow(
  int num, /* window number */
  char *header, /* header text */
  int startx, int starty, /* coordinates of upper left corner */
  int endx, int endy, /* coordinates of lower right corner */
  int border /* no border if 0 */
)
{
  if(num>MAX_FRAME) {
    tsrwindow_puts(0, "Too many windows\n");
    return 0;
  }

  if((startx>78) || (startx<0) || (starty>24) || (starty<0)) {
    tsrwindow_puts(0, "range error");
    return 0;
  } 

  if((endx>79) || (endy>25)) {
    tsrwindow_puts(0, "window won't fit");
    return 0;
  } 

  /* construct the frame */
  frame[num].startx = startx; frame[num].endx = endx;
  frame[num].starty = starty; frame[num].endy = endy;
  frame[num].p = wp; 
  frame[num].header = header;
  frame[num].border = border;
  frame[num].active = 0;
  frame[num].curx = 0; frame[num].cury = 0;
  return 1;  
}

/* Deactivate a window and remove it from the screen. */
void tsrdeactive(int num)
{
  /* reset the cursor postion to upper left corner */
  frame[num].curx = 0;
  frame[num].cury = 0;
  frame[num].active = 0;
  restore_video(num);
}

/* Display the header message in its proper location. */
void tsrdisplay_header(int num)
{
  register int x, len;

  x = frame[num].startx;

  /* Calculate the correct starting position to center
     the header message - if negative, message won't
     fit.
  */
  len = strlen(frame[num].header);
  len = (frame[num].endx - x - len) / 2;
  if(len<0) return; /* don't display it */
  x = x +len;

  write_string(x, frame[num].starty, 
               frame[num].header, NORM_VID);
}

/* Draw a window's border. */
void tsrdraw_border(int num)
{
  register int i;
  char far *v, far *t;

  v = vid_mem;
  t = v;
  for(i=frame[num].starty+1; i<frame[num].endy; i++) {
     v += (i*160) + frame[num].startx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
     v += (i*160) + frame[num].endx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
  }
  for(i=frame[num].startx+1; i<frame[num].endx; i++) {
     v += (frame[num].starty*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
     v += (frame[num].endy*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
  }

  /* draw the corners of the border */
  write_char(frame[num].startx, frame[num].starty, 218, NORM_VID);
  write_char(frame[num].startx, frame[num].endy, 192, NORM_VID);
  write_char(frame[num].endx, frame[num].starty, 191, NORM_VID);
  write_char(frame[num].endx, frame[num].endy, 217, NORM_VID);
}

/* Write a string at the current cursor position
   in the specified window.
   Returns 0 if window not active, 1 otherwise.
*/
tsrwindow_puts(int num, char *str)
{
   /* make sure window is active */
  if(!frame[num].active) return 0;

  for( ; *str;  str++) 
    tsrwindow_putchar(num, *str);
  return 1;
}

/* Write a character at the current cursor position
   in the specified window.
   Returns 0 if window not active, 1 otherwise.
*/
tsrwindow_putchar(int num, char ch)
{
   int x, y;
  char far *v;

  /* make sure window is active */
  if(!frame[num].active) return 0;

  x = frame[num].curx + frame[num].startx + 1;
  y = frame[num].cury + frame[num].starty + 1;

  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  if(y>=frame[num].endy) return 1;
  if(x>=frame[num].endx) return 1;

  if(ch=='\n') { /* newline char */
    y++;
    x = frame[num].startx+1;
    v = vid_mem;
    v += (y*160) + x*2; /* compute the address */
    frame[num].cury++;  /* increment Y */
    frame[num].curx = 0; /* reset X */ 
  }
  else {
    frame[num].curx++;
    *v++ = ch;  /* write the character */
    *v++ = NORM_VID; /* normal video attribute */
  }
  tsrwindow_xy(num, frame[num].curx, frame[num].cury);
  return 1;
}

/* Position cursor in a window at specified location.
   Returns 0 if out of range; non-zero otherwise.
*/
tsrwindow_xy(int num, int x, int y)
{
  if(x<0 || x+frame[num].startx>=frame[num].endx-1)
    return 0;
  if(y<0 || y+frame[num].starty>=frame[num].endy-1)
    return 0;
  frame[num].curx = x;
  frame[num].cury = y;
  goto_xy(frame[num].startx+x+1, frame[num].starty+y+1);
  return 1;
}

/* Read a string from a window. */
void tsrwindow_gets(int num, char *s)
{
  char ch, *temp;

  temp = s;
  for(;;) {
    ch = tsrwindow_getche(num);
    switch(ch) {
      case '\r':  /* the ENTER key is pressed */  
        *s = '\0';
        return;
      case '\b': /* backspace */
        if(s>temp) {
          s--;
          frame[num].curx--;
          if(frame[num].curx<0) frame[num].curx = 0;
          tsrwindow_xy(num, frame[num].curx, frame[num].cury);
          write_char(frame[num].startx+ frame[num].curx+1, 
            frame[num].starty+frame[num].cury+1, ' ', NORM_VID);
        }
        break;
      default: *s = ch;
	s++;
    }
  }
}


/* Input keystrokes inside a window.
   Returns full 16 bit scan code.
*/
tsrwindow_getche(int num)
{
  union inkey {
    char ch[2];
    int i;
  } c;
  
  if(!frame[num].active) return 0; /* window not active */

  tsrwindow_xy(num, frame[num].curx, frame[num].cury);

  c.i = readkey(); 	 /* read the key */

  if(c.ch[0]) {
    switch(c.ch[0]) {
      case '\r': /* the ENTER key is pressed */
        break;
      case '\b': /* backspace */
        break;
      default: 
        if(frame[num].cury+frame[num].starty < frame[num].endy-1) {
          write_char(frame[num].startx+frame[num].curx+1, 
            frame[num].starty+frame[num].cury+1, c.ch[0], NORM_VID);
          frame[num].curx++;
        }
    }
    if(frame[num].curx < 0) frame[num].curx = 0;
    if(frame[num].curx+frame[num].startx > frame[num].endx-2) 
      frame[num].curx--;
    tsrwindow_xy(num, frame[num].curx, frame[num].cury);
  }
  return c.i;
}

/* Clear to end of line. */
void tsrwindow_cleol(int num)
{
  register int i, x, y;

  x = frame[num].curx;
  y = frame[num].cury;
  tsrwindow_xy(num, frame[num].curx, frame[num].cury);

  for(i=frame[num].curx; i<frame[num].endx-1; i++)
    tsrwindow_putchar(num,' ');
  tsrwindow_xy(num, x, y);
}

/* Clear a window. */
void tsrwindow_cls(int num)
{
  register int i, j;
  char far *v;

  for(i=frame[num].starty+1; i<frame[num].endy; i++) 
    for(j=frame[num].startx+1; j<frame[num].endx; j++) {
      v = vid_mem;
      v += (i*160) + j*2;
      *v++ = ' ';  /* write a space */
      *v++ = NORM_VID;
    }
  frame[num].curx = 0;
  frame[num].cury = 0;
}

/* Move cursor up one line.
   Returns non-zero if successful, 0 otherwise.
*/
tsrwindow_upline(int num)
{
  if(frame[num].cury > 0) {
    frame[num].cury--;
    tsrwindow_xy(num, frame[num].curx, frame[num].cury);
    return 1;
  }
  return 0;
}

/* Move cursor down one line.
   Returns non-zero if successful, 0 otherwise.
*/
tsrwindow_downline(int num)
{
  if(frame[num].cury < frame[num].endy-frame[num].starty-1) {
    frame[num].cury++;
    tsrwindow_xy(num, frame[num].curx, frame[num].cury);
    return 1;
  }
  return 1;
}

/* Back up one character. */
void tsrwindow_bksp(int num)
{
  if(frame[num].curx>0) {
    frame[num].curx--;
    tsrwindow_xy(num, frame[num].curx, frame[num].cury);
    tsrwindow_putchar(num, ' ');
    frame[num].curx--;
    tsrwindow_xy(num, frame[num].curx, frame[num].cury); 
 }
}

/*********************************************************/
/* Low-level video functions.                            */
/*********************************************************/

/* Display a string with specifed attribute. */
void write_string(int x, int y, char *p, int attrib)
{
  char far *v;

  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  for(; *p; ) {
    *v++ = *p++;  /* write the character */
    *v++ = attrib;  /* write the attribute */
   }
}

/* Write character with specified attribute. */
void write_char(int x, int y, char ch, int attrib)
{
  char far *v;
  
  v = vid_mem;
  v += (y*160) + x*2;
  *v++ = ch;  /* write the character */
  *v = attrib;    /* write the attribute */
}

/* Save a portion of the screen. */
void save_video(int num)
{
  register int i,j;
  char *buf_ptr;
  char far *v, far *t;

  buf_ptr = frame[num].p;
  v = vid_mem;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      t = (v + (j*160) + i*2);
      *buf_ptr++ = *t++;
      *buf_ptr++ = *t;
      *(t-1) = ' ';  /* clear the window */
    }
}

/* Restore a portion of the screen. */
void restore_video(int num)
{
  register int i,j;
  char far *v, far *t;
  char *buf_ptr;

  buf_ptr = frame[num].p;
  v = vid_mem;
  t = v;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      v = t;
      v += (j*160) + i*2;
      *v++ = *buf_ptr++;  /* write the character */
      *v = *buf_ptr++;    /* write the attribute */
    }
  frame[num].active = 0; /* restore_video */
}

/* Returns the current video mode. */
video_mode(void)
{
  union REGS r;

  r.h.ah = 15;  /* get video mode */
  return int86(0x10, &r, &r) & 255;
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}

/* Send the cursor to the specified X,Y position. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Set the vid_mem pointer to the start of video
   memory.
*/
void set_vid_mem(void)
{
  int vmode;

  vmode = video_mode();
  if((vmode!=2) && (vmode!=3) && (vmode!=7)) {
    printf("video must be in 80 column text mode");
    exit(1);
  }
  /* set proper address of video RAM */
  if(vmode==7) vid_mem = (char far *) MK_FP(0xB000, 0000);
  else vid_mem = (char far *) MK_FP(0xB800, 0000);
}

/* Read and save cursor coordinates. */
void cursor_pos(void)
{
  union REGS i, o;

  i.h.bh = 0;
  i.h.ah = 3;
  int86(16, &i, &o);
  
  old_row = o.h.dh;
  old_col = o.h.dl;
}

