
/* Terminate and stay resident utilities.
*/
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "dos.h"
#include "ctype.h"
#include "fcntl.h"
#include "dir.h"
#include "conio.h"
#include "bios.h"
#include "io.h"

#define BORDER 1
#define MAX_FRAME 4
#define REV_VID 0x70
#define NORM_VID 7
#define BKSP 8
#define VID_SIZE 4000
#define STK_SIZE 0x2000

#define HELP_SIZE 75
#define KEY	   0
#define MSG	   1
#define HELP_BUF_SIZE 32000

/* window function prototypes */
void save_video(int num), restore_video(int num);
void write_string(int x, int y, char *p, int attrib);
void write_char(int x, int y, char ch, int attrib);
void display_header(int num), draw_border(int num);
void tsrwindow_gets(int num , char *s);
void tsrwindow_cleol(int num), tsrwindow(int num);
void tsrwindow_cls(int num),  set_vid_mem(void);
int make_tsrwindow(int num, char *header, int startx,
                int starty, int endx, int endy, 
                int border);
int tsrwindow_xy(int num, int x, int y);
int tsrwindow_puts(int num, char *str);
void deactivate(int num);
int video_mode(void), tsrwindow_putchar(int num, char ch);
int tsrwindow_getche(int num);

/* TSR function prototypes */
int load_help(void);
void read_at_cursor(char *str);
void do_help(void), cursor_pos(void);
void activate_sctsr(void);
void interrupt tsr_keystroke(void);
void interrupt dos_idle(void);
void interrupt new_int8(void);
void interrupt (*old_int28)(void);
void interrupt (*old_int8)(void);
void interrupt (*old_key)(void);
void interrupt (*old_int64)(void);
void do_filelist(void), dirlist(void), dectohex(void);

char far *vid_mem;
char video;

struct window_frame {
  int startx, endx, starty, endy; /* corners of window */
  int curx, cury; /* current cursor position in window */
  unsigned char *p; /* pointer to buffer */
  char *header; /* header message */
  int border; /* border on/off */
  int active; /* on screen yes/no */
} frame[MAX_FRAME];

char wp[VID_SIZE]; /* buffer to hold contents of the screen 
                      when saved by save_video() */

/* busy is set to 1 when the program is active, 0 otherwise */ 
char busy = 0;

char *help_index[HELP_SIZE][2];

char help_buf[HELP_BUF_SIZE];  /* holds the help file */

unsigned char stack[STK_SIZE];
unsigned int sp, ss;

int old_row, old_col;
int key;
char far *dos_active;

void main()
{
  union REGS r;
  struct SREGS s;

  if(!load_help()) {  /* load the help file */
    /* You may want to take other action here. */
    exit(1);
  }

  /* see if already loaded */
  old_int64 = getvect(0x64); /* unused interrupt */
  if(!old_int64) setvect(0x64, new_int8); /* set a flag */
  else {
    printf("TSR already loaded");
    exit(0);
  }

  /* obtain the dos_active flag address */
  r.h.ah = 0x34;
  int86x(0x21, &r, &r, &s);
  dos_active = MK_FP(s.es, r.x.bx);

  /* get old ISR addresses */
  old_key = getvect(9); /* keystroke int */
  old_int28 = getvect(0x28);  /* int 28 */
  old_int8 = getvect(8);  /* timer interrupt */

  /* re-route interrupt vectors to our handlers */
  setvect(9, tsr_keystroke);
  setvect(0x28, dos_idle);
  setvect(8, new_int8);

  printf("SC-TSR installed\n");
  printf("F1: help    F2: file list    F3: directory");
  printf("    F4: base convertor\n");

  key = 0; 
  set_vid_mem();  /* get address of video memory */

  /* initialize all windows */
  make_tsrwindow(0, " Help ('quit' to exit, 'topics' for topics) ",
              0, 0, 79, 24, BORDER); 
  make_tsrwindow(1, " File Lister ", 0, 0, 79, 16, BORDER); 
  make_tsrwindow(2, " Directory ", 0, 0, 79, 12, BORDER);
  make_tsrwindow(3, " Base Converter ", 0, 0, 50, 3, BORDER);

  keep(0, 4000); /* terminate and stay resident */
}

/* This is the function that intercepts the keystroke
   interrupt (int 9). */
void interrupt tsr_keystroke(void)
{
  int far *t2 = (int far *) 1050;  /* address of head pointer */
  char far *t = (char far *) 1050; /* address of head pointer */

  (*old_key)();  /* first, call old keystroke ISR */

  if(*t != *(t+2)) {/* if not empty */
    t += *t-30+5; /* advance to the character position */
    if(*t>=59 && *t <=63) {
      switch(*t) {  /* see what is in scan code */
        case 59:  /* F1 - help */
          key = 1;
          break;
        case 60:  /* F2 - file list */
          key = 2;
          break;
        case 61:  /* F3 - directory */
          key = 3;
          break;
        case 62:  /* F4 - dec to hex */
          key = 4;
          break;
        case 63: /* F5 - detache tsr */
          key = 5;
          break;
      }
      *(t2+1) = *t2;  /* zero buffer */
    }
  }
}

/* New interrupt 8. */
void interrupt new_int8(void)
{
  (*old_int8)();
  if(!*dos_active && !busy && key) 
        activate_sctsr(); 
}

/* This function intercepts int 0x28  (the dos-idle
   interrupt).
*/
void interrupt dos_idle(void)
{
  (*old_int28)();  /* call old int 0x28 ISR */
  if(!busy && key) activate_sctsr();
}

/* Pop up the SCTSR. */
void activate_sctsr(void)
{
  /* setup TSR stack */
  disable();  /* disable interrupts */
    ss = _SS;  
    sp = _SP;
    _SS = _DS;
    _SP = (unsigned) &stack[STK_SIZE-2];
  enable();   /* enable interrupts */

  /* check video mode - don't popup if not in
     an 80-column text mode
  */
  video = video_mode();
  if(!busy && (video==7 || video==3 || video==2)) {
    busy = !busy;  /* don't allow a second activation */
    switch(key) {
      case 1: do_help();
        break;
      case 2: do_filelist();
        break;
      case 3: dirlist();
        break;
      case 4: dectohex();
        break;
    }
    busy = !busy;
  }
  if(key==5){ /* deactivating TSR */
    cursor_pos();  /* get current cursor position */
    write_string(old_col, old_row, "SC-TSR deactivated", NORM_VID);
  }

  /* restore old stack */
  disable();
    _SP = sp;
    _SS = ss; 
  enable();

  if(key==5) {  /* reset old ISR routines and free memory */
    setvect(8, old_int8);
    setvect(9, old_key);
    setvect(0x28, old_int28);
    setvect(0x64, old_int64);
    freemem(_psp);  /* free program block */
  } else  key = 0; 
}

/* This is the help function.  (See text for
   details.)
*/
void do_help(void)
{
  char str[80];
  int i;

  /* See if a topic is located at the current cursor 
     position. */
  read_at_cursor(str); 
  cursor_pos();  /* save old cursor coordinates */

  tsrwindow(0);  /* activate window */
  tsrwindow_xy(0, 1, 1);
  tsrwindow_puts(0, "Topic: ");
  if(*str) tsrwindow_puts(0, str);
  for(;;) {
    if(!strcmp("quit", str)) break;
    tsrwindow_xy(0, 1, 3);

    if(!strcmp("topics", str)) 
      /* show help topics */
      for(i=0; help_index[i][KEY]; i++) {
        if(!(i%6)) tsrwindow_puts(0, "\n");
        tsrwindow_puts(0, help_index[i][KEY]);
        tsrwindow_puts(0, "  ");
      }
    else {
      for(i=0; help_index[i][KEY]; i++)
        /* lookup topic and display message if found */
        if(!strcmp(help_index[i][KEY], str)) {
          tsrwindow_puts(0, help_index[i][MSG]);
          break;
        }

      if(!help_index[i][KEY]) tsrwindow_puts(0, "Not found.");
    }
    tsrwindow_xy(0, 8, 1);
    while(!bioskey(1)) ;  /* wait for keystroke */
    tsrwindow_cls(0);
    tsrwindow_xy(0, 1, 1);
    tsrwindow_puts(0, "Topic: ");
    tsrwindow_gets(0, str);
  }
  deactivate(0);  /* close window */
  gotoxy(old_col, old_row); /* restore old cursor position */
}

/* List a file. */
void do_filelist(void)
{
  int fd;  /* file descriptor */
  int numbytes, cr;
  char fname[80];
  char ch;
  union inkey {
    char ch[2];
    int i;
  } c;

  char far *old_dta;
  long fpos, oldpos;

  cursor_pos();  /* save old cursor coordinates */
  old_dta = getdta();  /* get the dta */
  setdta(MK_FP(_psp, 0x80)); /* make TSR's dta current */

  tsrwindow(1);  /* activate window */

  tsrwindow_xy(1, 0, 0);
  tsrwindow_puts(1, "enter name: ");
  tsrwindow_gets(1, fname);
  
  if((fd = _open(fname, O_RDONLY)) < 0) {
    tsrwindow_puts(1, "cannot open file");
    deactivate(1);
    setdta(old_dta);  /* restore old dta */
    gotoxy(old_col, old_row);
    return;
  }

  do {
    cr = 0;
    tsrwindow_xy(1, 0, 0);
    tsrwindow_cleol(1);
    tsrwindow_puts(1, fname);
    tsrwindow_puts(1, ":");
    tsrwindow_xy(1, 0, 1);
    oldpos = tell(fd);
    do {
      numbytes = _read(fd, &ch, 1);
      tsrwindow_putchar(1, ch);
      if(ch=='\n') cr++;
    }while(numbytes && cr<frame[1].endy-2);
    c.i = tsrwindow_getche(1);
    tsrwindow_cls(1);
    switch(c.ch[1]) {
      case 71:  /* home: top of file */
        lseek(fd, 0L, SEEK_SET); 
        break;
      case 73: /* pgup: previous page */
        fpos = oldpos - 512L;
        if(fpos<0L) fpos = 0L;       
        lseek(fd, fpos, SEEK_SET);
        break;
      case 81: /* pgdn: next page */
        break;
      case 79: /* end */
        lseek(fd, -512L, SEEK_END);
        break;
    }
  } while(tolower(c.ch[0]!='q'));
    
  _close(fd); /* close file */
  deactivate(1); /* close window */
  setdta(old_dta); /* reset old dta */
  gotoxy(old_col, old_row); /* restore old cursor position */
}

/* List the current directory. */
void dirlist(void)
{
  struct ffblk f;
  register int done, i;
  int cr, len;
  char ch;
  char far *old_dta;

  cursor_pos();  /* save old cursor coordinates */

  old_dta = getdta();  /* get the dta */
  setdta(MK_FP(_psp, 0x80)); /* make TSR's dta current */

  tsrwindow(2);  /* activate window */

  /* find first match */
  done = findfirst("*.*", &f, 0);
  i = 1;
  do {
    cr = 0;
    tsrwindow_cls(2);
    while(!done) {
      tsrwindow_puts(2, f.ff_name);
      for(len=strlen(f.ff_name); len<14; len++) 
        tsrwindow_puts(2, " ");
      i++;
      done = findnext(&f); /* find next match */
      if(!(i%6)) { /* print six across */
        tsrwindow_puts(2, "\n");
        i = 1;
        cr++;
        if(cr == frame[2].endy-1) break;
      }
    }
    ch = tsrwindow_getche(2);
  } while(!done && tolower(ch) != 'q');
  deactivate(2);  /* close window */
  setdta(old_dta); /* reset old dta */
  gotoxy(old_col, old_row); /* restore old cursor position */
}

/* Base convertor: hex to dec, dec to hex. */
void dectohex(void)
{
  char str[80], out[80], ch;
  int num;

  cursor_pos();  /* save old cursor coordinates */
  tsrwindow(3);  /* activate window */
  tsrwindow_puts(3, "(1) hex to dec (2) dec to hex");
  ch = tsrwindow_getche(3);
  tsrwindow_puts(3, "\n");
  if(ch == '1') {
    tsrwindow_puts(3, "enter hex: ");
    tsrwindow_gets(3, str);
    sscanf(str, "%x", &num);
    tsrwindow_puts(3, "  dec: ");
    sprintf(out, "%d", num);
  }
  else {
    tsrwindow_puts(3, "enter dec: ");
    tsrwindow_gets(3, str);
    sscanf(str, "%d", &num);
    tsrwindow_puts(3, "  hex: ");
    sprintf(out, "%X", num);
  }
  tsrwindow_puts(3, out);
  tsrwindow_getche(3);
  deactivate(3);  /* close window */
  gotoxy(old_col, old_row); /* restore old cursor position */
}

/* Load the help file. This file must be called
   TSRHELP.DAT.   See text for details. 
*/
load_help(void)
{
  FILE *fp;
  char *p, ch;
  int i;

  p = help_buf;

  if((fp = fopen("TSRHELP.DAT", "r"))==NULL) {
    printf("cannot open TSRHELP.DAT");
    return 0;
  }

  ch = getc(fp);
  for(i=0; i<HELP_SIZE && !feof(fp); i++) {
    if(ch != '^') {
      printf("TSRHELP.DAT corrupted - use backup");
      fclose(fp);
      return 0;
    }
    if(p > help_buf+HELP_BUF_SIZE) {
      printf("Help file too big");
      fclose(fp);
      return 0;
    }

    help_index[i][KEY] = p;  /* pointer to keywords */

    while((*p=tolower(getc(fp)))!='\n') p++;  /* read keyword */
    *p = '\0'; /* null terminate the keyword */

    p++;

    help_index[i][MSG] = p;  /* pointer to help msg about keyword */

    while((*p=getc(fp))!='^' && !feof(fp)) p++; /* read message */
    ch = *p;
    *p = '\0'; /* null terminate the message */
    p++;
  }
  fclose(fp);
  return 1;
}

/* Set the vid_mem pointer to the start of video
   memory.
*/
void set_vid_mem(void)
{
  int vmode;

  vmode = video_mode();
  if((vmode!=2) && (vmode!=3) && (vmode!=7)) {
    printf("video must be in 80 column text mode");
    exit(1);
  }
  /* set proper address of video RAM */
  if(vmode==7) vid_mem = (char far *) MK_FP(0xB000, 0000);
  else vid_mem = (char far *) MK_FP(0xB800, 0000);
}

/* Read the word that is at the current cursor location. */
void read_at_cursor(char *str)
{
  int x;
  char far *p;
  union REGS i, o;

  i.h.bh = 0;
  i.h.ah = 3;
  int86(16, &i, &o);
  
  p = vid_mem + (o.h.dh * 160) + (o.h.dl * 2 );

  /* look for a word to the right of cursor */
  for(x=0; *p && p<vid_mem+VID_SIZE && x<78; x++) {
    if(isspace(*p) || strchr("(!,;{}<>", *p)) break;
    *str = *p;
    p += 2; str++;
  } 
  *str = NULL;  
  if(!x) { /* no word to the right, check left */
    p -= 2;
    while(!isspace(*p) && !strchr("(,;{}[]<>", *p))
          p -= 2;
    p += 2;
    for(x=0; *p && p>=vid_mem && x<78; x++) {
      if(isspace(*p) || strchr("(!,;{}[]<>", *p)) break;
      *str = *p;
      p += 2; str++;
    } 
    *str = NULL;       
  }
}

/* Read and save cursor coordinates. */
void cursor_pos(void)
{
  union REGS i, o;

  i.h.bh = 0;
  i.h.ah = 3;
  int86(16, &i, &o);
  
  old_row = o.h.dh;
  old_col = o.h.dl;
}

/* Send cursor to specified X,Y (0,0 is upper
   left corner). */
void gotoxy(int x, int y)
{
  union REGS i;

  i.h.dh = y;
  i.h.dl = x;
  i.h.ah = 2;
  i.h.bh = 0;
  int86(16, &i, &i);
}

/*********************************************************/
/* Window functions          
 				 
   These functions are adapted from those developed in
   "The Art of C", Herbert Schildt, 
   Osborne/McGraw-Hill, 1991.  Details of their operation
   are found there.
*/
/*********************************************************/

/* Display a pull-down window. */
void tsrwindow(int num) /* window number */
{
  /* get active window */
  if(!frame[num].active) { /* not currently in use */ 
    save_video(num);       /* save the current screen */
    frame[num].active = 1; /* set active flag */
  }

  if(frame[num].border) draw_border(num);
  display_header(num); /* display the window */
}

/* Construct a pull down window frame 
   1 is returned if window frame can be constructed;
   otherwise 0 is returned.
*/
make_tsrwindow(
            int num, /* window number */
            char *header, /* header text */
            int startx, int starty, /* upper left corner */
            int endx, int endy, /* lower right corner */
            int border /* no border if 0 */
)
{
  if(num>MAX_FRAME) {
    tsrwindow_puts(0, "Too many windows\n");
    return 0;
  }

  if((startx>78) || (startx<0) || (starty>24) || (starty<0)) {
    tsrwindow_puts(0, "range error");
    return 0;
  } 

  if((endx>79) || (endy>25)) {
    tsrwindow_puts(0, "window won't fit");
    return 0;
  } 

  /* construct the frame */
  frame[num].startx = startx; frame[num].endx = endx;
  frame[num].starty = starty; frame[num].endy = endy;
  frame[num].p = wp; 
  frame[num].header = header;
  frame[num].border = border;
  frame[num].active = 0;
  frame[num].curx = 0; frame[num].cury = 0;
  return 1;  
}

/* Deactivate a window and remove it from the screen. */
void deactivate(int num)
{
  /* reset the cursor postion to upper left corner */
  frame[num].curx = 0;
  frame[num].cury = 0;
  restore_video(num);
}

/* Display the header message in its proper location. */
void display_header(int num)
{
  register int x, len;

  x = frame[num].startx;

  /* Calculate the correct starting position to center
     the header message - if negative, message won't
     fit.
  */
  len = strlen(frame[num].header);
  len = (frame[num].endx - x - len) / 2;
  if(len<0) return; /* don't display it */
  x = x +len;

  write_string(x, frame[num].starty, 
               frame[num].header, NORM_VID);
}

/* Draw a border around the window. */
void draw_border(int num)
{
  register int i;
  char far *v, far *t;

  v = vid_mem;
  t = v;
  for(i=frame[num].starty+1; i<frame[num].endy; i++) {
     v += (i*160) + frame[num].startx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
     v += (i*160) + frame[num].endx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
  }
  for(i=frame[num].startx+1; i<frame[num].endx; i++) {
     v += (frame[num].starty*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
     v += (frame[num].endy*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
  }
  write_char(frame[num].startx, frame[num].starty, 218, NORM_VID);
  write_char(frame[num].startx, frame[num].endy, 192, NORM_VID);
  write_char(frame[num].endx, frame[num].starty, 191, NORM_VID);
  write_char(frame[num].endx, frame[num].endy, 217, NORM_VID);
}

/* Write a string at the current cursor position
   in the specified window.
   Returns 0 if window not active;
   1 otherwise.
*/
tsrwindow_puts(int num, char *str)
{
   /* make sure window is active */
  if(!frame[num].active) return 0;

  for( ; *str;  str++) 
    tsrwindow_putchar(num, *str);
  return 1;
}

/* Write a character at the current cursor position
   in the specified window.
   Returns 0 if window not active;
   1 otherwise.
*/
tsrwindow_putchar(int num, char ch)
{
   int x, y;
  char far *v;

  /* make sure window is active */
  if(!frame[num].active) return 0;

  x = frame[num].curx + frame[num].startx + 1;
  y = frame[num].cury + frame[num].starty + 1;

  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  if(y>=frame[num].endy) {
    return 1;
  }
  if(x>=frame[num].endx) {
    return 1;
  }

  if(ch=='\n') { /* newline char */
    y++;
    x = frame[num].startx+1;
    v = vid_mem;
    v += (y*160) + x*2; /* compute the address */
    frame[num].cury++;  /* increment Y */
    frame[num].curx = 0; /* reset X */ 
  }
  else if(ch=='\r') return 1;
  else {
    frame[num].curx++;
    *v++ = ch;  /* write the character */
    *v++ = NORM_VID;    /* normal video attribute */
  }
  tsrwindow_xy(num, frame[num].curx, frame[num].cury);
  return 1;
}

/* Position cursor in a window at specified location.
   Returns 0 if out of range;
   non-zero otherwise.
*/
tsrwindow_xy(int num, int x, int y)
{
  if(x<0 || x+frame[num].startx>=frame[num].endx-1)
    return 0;
  if(y<0 || y+frame[num].starty>=frame[num].endy-1)
    return 0;
  frame[num].curx = x;
  frame[num].cury = y;
  gotoxy(frame[num].startx+x+1, frame[num].starty+y+1);
  return 1;
}

/* Read a string from a window. */
void tsrwindow_gets(int num, char *s)
{
  char ch, *temp;

  temp = s;
  for(;;) {
    ch = tsrwindow_getche(num);
    switch(ch) {
      case '\r':  /* the ENTER key is pressed */  
        *s='\0';
        return;
      case BKSP: /* backspace */
        if(s>temp) {
          s--;
          frame[num].curx--;
          if(frame[num].curx<0) frame[num].curx = 0;
            tsrwindow_xy(num, frame[num].curx, frame[num].cury);
              write_char(frame[num].startx+ frame[num].curx+1, 
            frame[num].starty+frame[num].cury+1, ' ', NORM_VID);
        }
        break;
      default: *s = ch;
	s++;
    }
  }
}


/* Input keystrokes inside a window.
   Returns full 16 bit scan code.
*/
tsrwindow_getche(int num)
{
  union inkey {
    char ch[2];
    int i;
  } c;
  
  if(!frame[num].active) return 0; /* window not active */

  tsrwindow_xy(num, frame[num].curx, frame[num].cury);

  c.i = bioskey(0); 	 /* read the key */

  if(c.ch[0]) {
    switch(c.ch[0]) {
      case '\r': /* the ENTER key is pressed */
        break;
      case BKSP: /* back space */
        break;
      default: 
        if(frame[num].curx+frame[num].startx < frame[num].endx-1) {
        write_char(frame[num].startx+ frame[num].curx+1, 
          frame[num].starty+frame[num].cury+1, c.ch[0], NORM_VID);
          frame[num].curx++;
        }
    }
    if(frame[num].curx < 0) frame[num].curx = 0;
    if(frame[num].curx+frame[num].startx > frame[num].endx-2) 
      frame[num].curx--;
    tsrwindow_xy(num, frame[num].curx, frame[num].cury);
  }
  return c.i;
}

/* Clear to end of line. */
void tsrwindow_cleol(int num)
{
  register int i, x, y;

  x = frame[num].curx;
  y = frame[num].cury;
  tsrwindow_xy(num, frame[num].curx, frame[num].cury);

  for(i=frame[num].curx; i<frame[num].endx-1; i++)
    tsrwindow_putchar(num,' ');
  tsrwindow_xy(num, x, y);
}

void tsrwindow_cls(int num)
{
  register int i, j;
  char far *v;

  for(i=frame[num].starty+1; i<frame[num].endy; i++) 
    for(j=frame[num].startx+1; j<frame[num].endx; j++) {
      v = vid_mem;
      v += (i*160) + j*2;
      *v++ = ' ';  /* write a space */
      *v++ = NORM_VID;
    }
  frame[num].curx = 0;
  frame[num].cury = 0;
}

/****************************************************
 Low-level video functions.
****************************************************/

/* Display a string with specifed attribute. */
void write_string(int x, int y, char *p, int attrib)
{
  char far *v;

  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  for(; *p; ) {
    *v++ = *p++;  /* write the character */
    *v++ = attrib;    /* write the attribute */
   }
}

/* Write character with specified attribute. */
void write_char(int x, int y, char ch, int attrib)
{
  char far *v;
  
  v = vid_mem;
  v += (y*160) + x*2;
  *v++ = ch;  /* write the character */
  *v = attrib;    /* write the attribute */
}

/* Save a portion of the screen. */
void save_video(int num)
{
  register int i,j;
  char *buf_ptr;
  char far *v, far *t;

  buf_ptr = frame[num].p;
  v = vid_mem;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      t = (v + (j*160) + i*2);
      *buf_ptr++ = *t++;
      *buf_ptr++ = *t;
      *(t-1) = ' ';  /* clear the window */
    }
}

/* Restore a portion of the screen. */
void restore_video(int num)
{
  register int i,j;
  char far *v, far *t;
  char *buf_ptr;

  buf_ptr = frame[num].p;
  v = vid_mem;
  t = v;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      v = t;
      v += (j*160) + i*2;
      *v++ = *buf_ptr++;  /* write the character */
      *v = *buf_ptr++;    /* write the attribute */
  }
  frame[num].active = 0; /* restore_video */
}

/* Returns the current video mode. */
video_mode(void)
{
  union REGS r;

  r.h.ah = 15;  /* get video mode */
  return int86(0x10, &r, &r) & 255;
}
