

/* Window routines with simple demostration program.
   An editor is simulated.  Three special pop-up window
   utilities illustrate both the power and magic of
   windowing software and also serve as examples of
   window programming.  The pop-up window routines
   are a four-function calculator, a decimal to hex
   converter, and a pop-up note pad.
*/

#include "stdio.h"
#include "dos.h"
#include "bios.h"
#include "stdlib.h"
#include "string.h"

#define BORDER 1
#define ESC 27
#define MAX_FRAME 10
#define REV_VID 0x70
#define NORM_VID 7

void save_video(int num), restore_video(int num);
void goto_xy(int x, int y), cls(void);
void write_string(int x, int y, char *p, int attrib);
void write_char(int x, int y, char ch, int attrib);
void draw_border(int num);
void display_header(int num);
void window_gets(int num, char *s), size(int num);
void move(int num), window_cls(int num);
void window_cleol(int num), window(int num);
void dectohex(void), notepad(void), calc(void);
void deactivate(int num);
void window_bksp(int num);
int window_upline(int num), window_downline(int num);
int make_window(int num, char *header, int startx, int starty,
                int endx, int endy, int border);
int window_getche(int num);
int window_putchar(int num, char ch);
int window_xy(int num, int x, int y);
int video_mode(void), get_special(void);
int push(int i), pop(void);
int readkey(void);

char far *vid_mem;

struct window_frame {
  int startx, endx, starty, endy; /* window position */
  int curx, cury; /* current cursor position in window */
  unsigned char *p; /* pointer to buffer */
  char *header; /* header message */
  int border; /* border on/off */
  int active; /* on screen yes/no */
} frame[MAX_FRAME];

main(void)
{
  union inkey {
    char ch[2];
    int i;
  } c;
  int i;
  char ch;

  cls();
  goto_xy(0,0);

  /* first, create the window frames */
  make_window(0, " Editor [Esc to exit] ", 0, 0, 78, 24, BORDER);
  make_window(1, " Decimal to Hex ", 40, 7, 70, 10, BORDER);
  make_window(2, " Calculator ", 20, 8, 60, 12, BORDER); 
  make_window(3, " Notepad [F1 to exit] ", 20, 5, 60, 16, BORDER); 

  /* use window() to activate the specified window */
  window(0);
  do {
    c.i = window_getche(0);
    ch = c.i; /* use only low-order byte */
    if(ch=='\r') /* must do explicit crlf */
        window_putchar(0, '\n');

    switch(c.ch[1]) { /* see if arrow or function key */
      case 59: /* F1 demostrate the window_xy() function */
        window(1);
        for(i=0; i<10; i++) 
          if(window_xy(1, i, i)) window_putchar(1,'X');
        window_getche(1);
        deactivate(1);
        break;
      case 60: /* F2 demonstrate sizing and moving a window */
        size(1);
        move(1);
        break;
      case 61: /* F3 invoke the calculator */
        calc();
        break;
      case 62: /* F4 invoke the dec to hex converter */
        dectohex();
        break;
      case 63: /* F5 invoke the note pad */
        notepad();
        break;
      case 72: /* up */
        window_upline(0);
        break;
      case 80: /* down */
        window_downline(0);
        break;
    }
  } while (ch!=ESC);
  deactivate(0); /* remove window */
  return 0;
}

/*********************************************************/
/* Window functions                                      */
/*********************************************************/

/* Display a pull-down window. */
void window(int num) /* window number */
{
  int  vmode;
  int x, y;

  vmode = video_mode();
  if((vmode!=2) && (vmode!=3) && (vmode!=7)) {
    printf("video must be in 80 column text mode");
    exit(1);
  }
  /* set proper address of video RAM */
  if(vmode==7) vid_mem = (char far *) 0xB0000000;
  else vid_mem = (char far *) 0xB8000000;

  /* get active window */
  if(!frame[num].active) { /* not currently in use */ 
    save_video(num);       /* save the current screen */
    frame[num].active = 1; /* set active flag */
  }

  if(frame[num].border) draw_border(num);
  display_header(num); /* display the window */

  x = frame[num].startx + frame[num].curx + 1;
  y = frame[num].starty + frame[num].cury + 1;
  goto_xy(x, y);
}

/* Construct a pull-down window frame.
   1 is returned if window frame can be constructed;
   otherwise 0 is returned.
*/
make_window(
  int num, /* window number */
  char *header, /* header text */  
  int startx, int starty, /* X,Y coordinates of upper left corner */
  int endx, int endy, /* X,Y coordinates of lower right corner */
  int border) /* no border if 0 */
{
  unsigned char *p;

  if(num>MAX_FRAME) {
    printf("Too many windows\n");
    return 0;
  }

  if((starty>24) || (starty<0) || (startx>78) || (startx<0)) {
    printf("range error");
    return 0;
  } 

  if((endy>24) || (endx>79)) {
    printf("window won't fit");
    return 0;
  } 

  /* allocate enough memory to hold it */
  p = (unsigned char *) malloc(2*(endy-starty+1)*(endx-startx+1));
  if(!p) exit(1); /* put your own error handler here */

  /* construct the frame */
  frame[num].startx = startx; frame[num].endx = endx;
  frame[num].starty = starty; frame[num].endy = endy;
  frame[num].p = p; 
  frame[num].header = header;
  frame[num].border = border;
  frame[num].active = 0;
  frame[num].curx = 0; frame[num].cury = 0;
  return 1;  
}

/* Deactivate a window and remove it from the screen. */
void deactivate(int num)
{
  /* reset the cursor postion to upper left corner */
  frame[num].curx = 0;
  frame[num].cury = 0;
  frame[num].active = 0;
  restore_video(num);
}

/* Interactively change the size of a window.
*/
void size(int num)
{
  char ch;
  int x, y, startx, starty;

  /* activate if necessary */
  if(!frame[num].active) window(num);

  startx = x = frame[num].startx;
  starty = y = frame[num].starty;
  window_xy(num, 0, 0);
  
  do {
    ch = get_special();
    switch(ch) {
      case 75: /* left */
        startx--;
        break;
      case 77: /* right */
        startx++;
        break;
      case 72: /* up */
        starty--;
        break;
      case 80: /* down */
        starty++;
        break;
      case 71: /* up left */
        starty--; startx--;
        break;
      case 73: /* up right */
        starty--; startx++;
        break;
      case 79: /* down left*/
        starty++; startx--;
        break;
      case 81: /* down right */
        starty++; startx++;
        break;
      case 60: /* F2: cancel and use original size */
        startx = x;
        starty = y;
        ch = 59;
    }
    /* see if out-of-range */
    if(startx<0) startx++;
    if(startx>=frame[num].endx) startx--;
    if(starty<0) starty++;
    if(starty>=frame[num].endy) starty--;
    deactivate(num);
    frame[num].startx = startx;
    frame[num].starty = starty;
    window(num);
  } while(ch!=59);  /* F1 to stop */
  deactivate(num);
}

/* Interactively move a window. */
void move(int num)
{
  char ch;
  int x, y, ex, ey, startx, starty, endx, endy;

  /* activate if necessary */
  if(!frame[num].active) window(num);

  startx = x = frame[num].startx;
  starty = y = frame[num].starty;
  endx = ex = frame[num].endx;
  endy = ey = frame[num].endy;
  window_xy(num, 0, 0);

  do {
    ch = get_special();
    switch(ch) {
      case 75: /* left */
        startx--;
        endx--;
        break;
      case 77: /* right */
        startx++;
        endx++;
        break;
      case 72: /* up */
        starty--;
        endy--;
        break;
      case 80: /* down */
        starty++;
        endy++;
        break;
      case 71: /* up left */
        starty--; startx--;
        endy--; endx--;
        break;
      case 73: /* up right */
        starty--; startx++;
        endy--; endx++;
        break;
      case 79: /* down left*/
        starty++; startx--;
        endy++; endx--;
        break;
      case 81: /* down right */
        starty++; startx++;
        endy++; endx++;
        break;
      case 60: /* F2: cancel and use original size */
        startx = x;
        starty = y;
        endx = ex;
        endy = ey;
        ch = 59;
    }

    /* see if out-of-range */
    if(startx<0) {
      startx++;
      endx++;
    }
    if(endx>=79) {
      startx--;
      endx--;
    }
    if(starty<0) {
      starty++;
      endy++;
    }
    if(endy>=25) {
      starty--;
      endy--;
    }
    deactivate(num);
    frame[num].startx = startx;
    frame[num].starty = starty;
    frame[num].endx = endx;
    frame[num].endy = endy;
    window(num);
  } while(ch!=59); /* F1 to stop */
  deactivate(num);
}

/* Display the header message in its proper location. */
void display_header(int num)
{
  register int  x, len;

  x = frame[num].startx;

  /* Calculate the correct starting position to center
     the header message - if negative, message won't
     fit.
  */
  len = strlen(frame[num].header);
  len = (frame[num].endx - x - len) / 2;
  if(len<0) return; /* don't display it */
  x = x + len;

  write_string(x, frame[num].starty, 
               frame[num].header, NORM_VID);
}

/* Draw a window's border. */
void draw_border(int num)
{
  register int i;
  char far *v, far *t;

  v = vid_mem;
  t = v;
  for(i=frame[num].starty+1; i<frame[num].endy; i++) {
     v += (i*160) + frame[num].startx*2;
     *v++ = 179; /* vertical bar */
     *v = NORM_VID;
     v = t;
     v += (i*160) + frame[num].endx*2;
     *v++ = 179;
     *v = NORM_VID;
     v = t;
  }
  for(i=frame[num].startx+1; i<frame[num].endx; i++) {
     v += (frame[num].starty*160) + i*2;
     *v++ = 196; /* horizontal bar */
     *v = NORM_VID;
     v = t;
     v += (frame[num].endy*160) + i*2;
     *v++ = 196;
     *v = NORM_VID;
     v = t;
  }

  /* draw the corners of the border */
  write_char(frame[num].startx, frame[num].starty,
             (char) 218, NORM_VID);
  write_char(frame[num].startx, frame[num].endy,
             (char) 192, NORM_VID);
  write_char(frame[num].endx, frame[num].starty,
             (char) 191, NORM_VID);
  write_char(frame[num].endx, frame[num].endy,
             (char) 217, NORM_VID);
}

/*************************************************************/
/* Window I/O functions                                      */
/*************************************************************/

/* Write a string at the current cursor position
   in the specified window.
   Returns 0 if window not active;
   1 otherwise.
*/
window_puts(int num, char *str)
{
   /* make sure window is active */
  if(!frame[num].active) return 0;

  for( ; *str;  str++) 
    window_putchar(num, *str);
  return 1;
}

/* Write a character at the current cursor position
   in the specified window.
   Returns 0 if window not active;
   1 otherwise.
*/
window_putchar(int num, char ch)
{
  register int x, y;
  char far *v;

  /* make sure window is active */
  if(!frame[num].active) return 0;

  x = frame[num].curx + frame[num].startx + 1;
  y = frame[num].cury + frame[num].starty + 1;

  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  if(y>=frame[num].endy) {
    return 1;
  }
  if(x>=frame[num].endx) {
    return 1;
  }

  if(ch=='\n') { /* newline char */
    y++; 
    x = frame[num].startx+1;
    v = vid_mem;
    v += (y*160) + x*2; /* compute the address */
    frame[num].curx = 0;  /* reset X */
    frame[num].cury++; /* increment Y */ 
  }
  else {
    frame[num].curx++;
    *v++ = ch;  /* write the character */
    *v++ = NORM_VID;    /* normal video attribute */
  }
  window_xy(num, frame[num].curx, frame[num].cury);
  return 1;
}

/* Position cursor in a window at specified location.
   Returns 0 if out of range;
   non-zero otherwise.
*/
window_xy(int num, int x, int y)
{
  if(x<0 || x+frame[num].startx>=frame[num].endx-1)
    return 0;
  if(y<0 || y+frame[num].starty>=frame[num].endy-1)
    return 0;
  frame[num].curx = x;
  frame[num].cury = y;
  goto_xy(frame[num].startx+x+1, frame[num].starty+y+1);
  return 1;
}

/* Read a string from a window. */
void window_gets(int num, char *s)
{
  char ch, *temp;

  temp = s;
  for(;;) {
    ch = window_getche(num);
    switch(ch) {
      case '\r':  /* the ENTER key is pressed */  
        *s='\0';
        return;
      case '\b': /* backspace */
        if(s>temp) {
          s--;
          frame[num].curx--;
          if(frame[num].curx<0) frame[num].curx = 0;
            window_xy(num, frame[num].curx, frame[num].cury);
              write_char(frame[num].startx+frame[num].curx+1, 
            frame[num].starty+frame[num].cury+1, ' ', NORM_VID);
        }
        break;
      default: *s = ch;
        s++;
    }
  }
}


/* Input keystrokes inside a window.
   Returns full 16 bit scan code.
*/
window_getche(int num)
{
  union inkey {
    char ch[2];
    int i;
  } c;
  
  if(!frame[num].active) return 0; /* window not active */

  window_xy(num, frame[num].curx, frame[num].cury);

  c.i = readkey();      /* read the key */

  if(c.ch[0]) {
    switch(c.ch[0]) {
      case '\r': /* the ENTER key is pressed */
        break;
      case '\b': /* backspace */
        break;
      default: 
        if(frame[num].cury+frame[num].starty < frame[num].endy-1) {
          write_char(frame[num].startx+ frame[num].curx+1, 
            frame[num].starty+frame[num].cury+1, c.ch[0], NORM_VID);
          frame[num].curx++;
        }
    }
    if(frame[num].curx < 0) frame[num].curx = 0;
    if(frame[num].curx+frame[num].startx > frame[num].endx-2) 
      frame[num].curx--;
    window_xy(num, frame[num].curx, frame[num].cury);
  }
  return c.i;
}

/* Clear a window. */
void window_cls(int num)
{
  register int i,j;
  char far *v, far *t;

  v = vid_mem;
  t = v;
  for(i=frame[num].starty+1; i<frame[num].endy; i++)
    for(j=frame[num].startx+1; j<frame[num].endx; j++) {
      v = t;
      v += (i*160) + j*2;
      *v++ = ' ';  /* write a space */
      *v = NORM_VID;    /* normal */
  }
  frame[num].curx = 0;
  frame[num].cury = 0;
}

/* Clear to end of line. */
void window_cleol(int num)
{
  register int i, x, y;

  x = frame[num].curx;
  y = frame[num].cury;
  window_xy(num, frame[num].curx, frame[num].cury);

  for(i=frame[num].curx; i<frame[num].endx-1; i++)
    window_putchar(num,' ');
  window_xy(num, x, y);
}

/* Move cursor up one line.
   Returns non-zero if successful;
   0 otherwise.
*/
window_upline(int num)
{
  if(frame[num].cury > 0) {
    frame[num].cury--;
    window_xy(num, frame[num].curx, frame[num].cury);
    return 1;
  }
  return 0;
}

/* Move cursor down one line.
   Returns non-zero if successful;
   0 otherwise.
*/
window_downline(int num)
{
  if(frame[num].cury < frame[num].endy-frame[num].starty-1) {
    frame[num].cury++;
    window_xy(num, frame[num].curx, frame[num].cury);
    return 1;
  }
  return 1;
}

/* Back up one character. */
void window_bksp(int num)
{
  if(frame[num].curx>0) {
    frame[num].curx--;
    window_xy(num, frame[num].curx, frame[num].cury);
    window_putchar(num, ' ');
    frame[num].curx--;
    window_xy(num, frame[num].curx, frame[num].cury); 
 }
}

/*****************************************************/
/* Misc. functions                                   */
/*****************************************************/

/* Display a string with specifed attribute. */
void write_string(int x, int y, char *p, int attrib)
{
  char far *v;

  v = vid_mem;
  v += (y*160) + x*2; /* compute the address */
  while(*p) { 
    *v++ = *p++;  /* write the character */
    *v++ = attrib;    /* write the attribute */
   }
}

/* Write character with specified attribute. */
void write_char(int x, int y, char ch, int attrib)
{
  char far *v;
  
  v = vid_mem;
  v += (y*160) + x*2;
  *v++ = ch;  /* write the character */
  *v = attrib;    /* write the attribute */
}

/* Save a portion of the screen. */
void save_video(int num)
{
  register int i,j;
  char *buf_ptr;
  char far *v, far *t;

  buf_ptr = frame[num].p;
  v = vid_mem;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      t = (v + (j*160) + i*2);
      *buf_ptr++ = *t++;
      *buf_ptr++ = *t;
      *(t-1) = ' ';  /* clear the window */
    }
}

/* Restore a portion of the screen. */
void restore_video(int num)
{
  register int i,j;
  char far *v, far *t;
  char *buf_ptr;

  buf_ptr = frame[num].p;
  v = vid_mem;
  t = v;
  for(i=frame[num].startx; i<frame[num].endx+1; i++)
    for(j=frame[num].starty; j<frame[num].endy+1; j++) {
      v = t;
      v += (j*160) + i*2;
      *v++ = *buf_ptr++;  /* write the character */
      *v = *buf_ptr++;    /* write the attribute */
  }
  frame[num].active = 0; /* restore_video */
}

/* Clear the screen. */
void cls(void)
{
  union REGS r;

  r.h.ah = 6; /* screen scroll code */
  r.h.al = 0; /* clear screen code */
  r.h.ch = 0; /* start row */
  r.h.cl = 0; /* start column */
  r.h.dh = 24; /* end row */
  r.h.dl = 79; /* end column */
  r.h.bh = 7;  /* blank line is black */
  int86(0x10, &r, &r);
}

/* Send the cursor to the specified X,Y position. */
void goto_xy(int x, int y)
{
  union REGS r;

  r.h.ah = 2; /* cursor addressing function */
  r.h.dl = x; /* column coordinate */
  r.h.dh = y; /* row coordinate */
  r.h.bh = 0; /* video page */
  int86(0x10, &r, &r);
}

/* Return the position code of arrow and function keys. */
get_special(void)
{
  union inkey {
    char ch[2];
    int i;
  } c;

  c.i = readkey(); /* read the key */

  return c.ch[1];
}

/* Returns the current video mode. */
video_mode(void)
{
  union REGS r;

  r.h.ah = 15;  /* get video mode */
  return int86(0x10, &r, &r) & 255;
}

/* Return the 16-bit scan code from the keyboard. */
readkey(void)
{
  union REGS r; 

  r.h.ah = 0;
  return int86(0x16, &r, &r);
}

#include "ctype.h"
/***************************************************/
/* pop-up window functions                         */
/***************************************************/

#define MAX 100

int *p;   /* pointer into the stack */
int *tos; /* points to top of stack */
int *bos; /* points to bottom of stack */

/* Stack based, postfix notation four-function calculator. */
void calc(void)
{
  char in[80], out[80];
  int answer, stack[MAX];
  int a,b;

  p = stack;
  tos = p; 
  bos = p+MAX-1;

  window(2);
  do {
    window_xy(2, 0, 0); 
    window_cleol(2);
    window_puts(2, ": "); /* calc prompt */
    window_gets(2, in);
    window_puts(2, "\n ");
    window_cleol(2);
    switch(*in) {
      case '+':
        a = pop();
        b = pop();
        answer = a+b;
        push(a+b);
        break;
      case '-':
        a = pop();
        b = pop();
        answer = b-a;
        push(b-a);
        break;
      case '*':
        a = pop();
        b = pop();
        answer = b*a;
        push(b*a);
        break;
      case '/':
        a = pop();
        b=pop();
        if(a==0) {
          window_puts(2, "divide by 0\n");
          break;
        }
        answer = b/a;
        push(b/a);
        break;
      default:
        push(atoi(in));
        continue;
    }
    sprintf(out, "%d", answer);
    window_puts(2, out);
  } while(*in);  
  deactivate(2);      
}

/* Place a number on the stack.
   Returns 1 if successful;
   0 if stack is full.
*/
push(int i)
{
  if(p>bos) return 0;
 
  *p=i;
  p++;
  return 1;
}

/* Retrieve top element from the stack.
   Returns 0 on stack underflows.
*/ 
pop(void)   
{
  p--;
  if(p<tos) {
    p++;
    return 0;
  }
  return *p;
}

/*****************************************************/

/* Decimal to hexadecimal converter. */
void dectohex(void)
{
  char in[80], out[80];
  int n;

  window(1);
  do {
    window_xy(1, 0, 0);  /* go to first line */
    window_cleol(1); /* clear the line */
    window_puts(1, "dec: "); /* prompt */
    window_gets(1, in); /* read the number */
    window_putchar(1, '\n'); /* go to next line */ 
    window_cleol(1); /* clear it */
    sscanf(in,"%d", &n); /* convert to internal format */
    sprintf(out, "%s%X", "hex: ",n); /* convert to hex */
    window_puts(1, out); /* output hex */
  } while(*in);
  deactivate(1);
}

/****************************************************/

/* Pop-up note pad. */
#define MAX_NOTE 10 
char notes[MAX_NOTE][80];

void notepad(void)
{
  static first=1;
  register int i, j, k;
  union inkey {
    char ch[2];
    int i;
  } c;
  char ch;

  /* initialize notes array if necessary */
  if(first) {
    for(i=0; i<MAX_NOTE; i++)
      *notes[i] = '\0';
    first = !first;
  }

  window(3);
  /* display the existing notes */
  for(i=0; i<MAX_NOTE; i++) {
    if(*notes[i]) window_puts(3, notes[i]);
    window_putchar(3, '\n');
  }

  i=0;  
  window_xy(3, 0, 0);

  for(;;) {
    c.i = readkey();    /* read the key */
    if(tolower(c.ch[1])==59) { /* F1 to quit */
      deactivate(3);
      break;
    }

    /* if normal key */
    if(isprint(c.ch[0]) || c.ch[0]=='\b') { 
      window_cleol(3);
      notes[i][0] = c.ch[0];
      j = 1; 
      window_putchar(3, notes[i][0]);
      do {
        ch = window_getche(3);
        if(ch=='\b') {
          if(j>0) {
            j--;
            window_bksp(3);
          }
        } 
        else {
          notes[i][j] = ch;
          j++;
        }
      } while(notes[i][j-1]!='\r');
      notes[i][j-1] = '\0';
      if(i<MAX_NOTE-1) i++;
      window_putchar(3, '\n');
    }
    else {  /* is special key */
      switch(c.ch[1]) {
        case 72:  /* up arrow */
          if(i>0) {
            i--;
            window_upline(3);
          }
          break;
        case 80:  /* down arrow */
          if(i<MAX_NOTE-1) {
            i++;
            window_downline(3);
          }
          break;
        case 60: /* clear the notepad */
          window_cls(3);
          for(k=0; k<MAX_NOTE; k++) *notes[k] = '\0';
          window_xy(3, 0, 0);
      }
    }
  }
}

