/*
ͻ
 Source File    X10_EXMP.C                                                
                                                                          
 System         X-10 Professional Developers Library for C                
                                                                          
 Purpose        Example X-10 program for:  Morrison, Dale & Jeanie        
                                           Kent, WA                       
                                                                          
 Author         Dale Morrison                                             
 Copyright      Smart Home Technology, 1993                               
ͼ


Ŀ
 Load the file containing declarations and constants for the X10_PDLC.LIB  
 */
#include "X10_PDLC.H";

/*
Ŀ
 Define your custom variables that are initialized in the Startup procedure
 */
int JrIsWorking, CancelBedtime, ResetMotion;

/*
Ŀ
 Start of your program                                                     
 */
main()
{
	char KeyHit, NewTime[6];
	int	 Xpos, Ypos;

    /*
    Ŀ
     Your basic clear the screen command                                   
     */
    clrscr();

    /*
    Ŀ
     Define the interrupt address for the x10 driver                       
     */
    Addr523 = 0x60

    /*
    Ŀ
     Open communications to the resident TW523 driver                      
     */
    open_tw523(SinglePhase);

    /*
    Ŀ
     Do your Startup procedure to initialize your modules and variables    
     */
    startup();

    /*
    Ŀ
     Start a timer to be used for saving the status of the modules         
     */
	start_timer(0, 1);

    /*
    Ŀ
     Now check to see if there was a power outage since the last boot up   
     */
    if (PowerOutage) {
        load_status();
        play_catchup(); }

    /*
    Ŀ
     Now start the loop to monitor your schedule until a keypress          
     */
    do {

        /*
        Ŀ
         Get the current system time based on the 24 hour clock, ie: 23:00 
         */
        get_x10time();

        /*
        Ŀ
         Get the next received X-10 command from the driver, Null if none  
         */
        get_x10cmd();

        /*
        Ŀ
         If you want to see the time and date when the system catches up   
         */
		if (PowerOutage) {
            gotoxy(1, wherey() );
			printf("    %-8s  %s  %s", X10Time, X10Dow, X10Date); }

        /*
        Ŀ
         The part of your schedule that deals with time of day events      
         */

        if (time_is("05:00")) {                     /* Dales wakeup time                */
            turn_module("F01", xOn);                /*      Coffee pot on               */
            if (time_is_before(Dawn)) {
                turn_all_modules("O", xOff);        /*      Outside lights off          */
                dim_module("E05", 8); } }           /*      Stove light to 8            */

        if (time_is(Dawn)) {                        /* First morning light              */
            turn_all_modules("E", xOff);            /*      Inside lights off           */
            turn_all_modules("O", xOff); }          /*      Outside lights off          */

        if (time_is("20:00"))                       /* Boys strip to underware time     */
            flash_module("E02", 15);                /*      Flash table lamp            */

        if (time_is("20:15"))                       /* Boys pajamas time                */
            flash_module("E03", 15);                /*      Flash living ovhd           */

        if (time_is("20:30"))                       /* Boys get toys for bed            */
            turn_module("E06", xOn);                /*      Playroom light on           */
        /**/
        if (time_is("21:00")) {                     /* Boys already in bed              */
            turn_module("E06", xOff);               /*      Playroom light off          */
            turn_module("F01", xOff); }             /*      Coffee pot off              */

        if (time_is("21:30") && !CancelBedtime) {   /* Dales Bedtime                    */
            select_module("E01");                   /*      Living room flourescents    */
            select_module("E03");                   /*      Living room ovhd lights     */
            select_module("E04");                   /*      Kitchen stove light         */
            select_module("E05");                   /*      Kitchen ovhd lights         */
            select_module("E06");                   /*      Playroom lights             */
            turn_sel_modules(xOff);                 /*      Turn them all off           */
            dim_module("E02", 10);                  /*      Living table lamp to 10     */
            turn_module("E05", xOn);                /*      Kitchen ovhd lights on      */
            dim_module("G03", 5); }                 /*      Master bed lights to 10     */
        if (time_is(Dusk))                          /* Its now dark outside             */
            turn_all_modules("O", xOn);             /*      Outside lights on           */

        if (time_is("23:50") && JrIsWorking) {      /* 10 minutes before jr comes home  */
            dim_module("E02", 8);                   /*      Living table lamp to 8      */
            dim_module("E05", 9);                   /*      Kitchen ovhd lights to 9    */
            pause(10);                              /*      Let house adjust            */
            turn_module("O01", xOff);               /*      Front porch light off       */
            pause(3);                               /*      Force motion light on       */
            turn_module("O01", xOn);                /*      Front porch light on        */
            ResetMotion = Yes; }

        if (time_is("00:01")) {                     /* Its a new day so set variables   */
            load_sun_times();                       /*      Load the sun time variables */
            get_x10dow();                           /*      Get the day of the week     */
            get_x10date();                          /*      Get todays date             */
            if (dow_is("Sun") || dow_is("Mon")) {   /*      If Jr does not work today   */
                JrIsWorking = No;                   /*         set flag saying such     */
                CancelBedtime = Yes; }              /*         and cancel dales bedtime */
            else {                                  /*      If Jr does work today       */
                JrIsWorking = Yes;                  /*         set flag saying such     */
                CancelBedtime = No; }               /*         dont cancel dales bedtime*/
            display_x10_globals();
			printf("    Jr is working: %s", (JrIsWorking) ? "Yes\n\n" : "No\n\n"); }


        if (time_is("00:30") && ResetMotion) {      /* Jr is home by now                */
            turn_module("O01", xOff);               /*      Front porch light off       */
            pause(10);                              /*      Force back to sense motion  */
            turn_module("O01", xOn);                /*      Front porch light on        */
            ResetMotion = No; }

        if (time_is("02:00")) {                     /* Safety reasons                   */
            turn_all_modules("E", xOff);            /*      All inside lights off       */
            turn_all_modules("F", xOff);            /*      All appliance modules off   */
            turn_all_modules("G", xOff); }          /*      All bedroom lights off      */

        /*
        Ŀ
         The part of your schedule that deals with the received X-10 cmds  
         */

        if (x10cmd_is("G01 Off")) {                 /* Master bedroom ctrler            */
            turn_all_modules("E", xOff);            /*      All modules off             */
            turn_all_modules("G", xOff); }          /*      Master bed lights off       */
        /**/
        if (x10cmd_is("G01 On")) {                  /* Master bedroom ctrler            */
            turn_all_modules("E", xOn);             /*      All lights on               */
            turn_all_modules("G", xOn);             /*      All lights on               */
            turn_all_modules("O", xOn); }           /*      All lights on               */

        if (x10cmd_is("G04 Off"))                   /* Master bedroom ctrler            */
            JrIsWorking = No;                       /*      Jr is not working today     */

        if (x10cmd_is("G04 On"))                    /* Master bedroom ctrler            */
            CancelBedtime = Yes;                    /*      Cancel dales bedtime        */

        if (x10cmd_is("H04 On")) {                  /* Living room sundowner ctrler     */
            dim_module("E02", 8);                   /*      Living table lamp to 8      */
            start_timer(1, 10); }                   /*      Start a 10 minute timer     */
        if (x10cmd_is("H03 On"))                    /* Living room sundowner ctrler     */
            turn_all_modules("O", xOn);             /*      Outside lights on           */

        if (x10cmd_is("H03 Off"))                   /* Living room sundowner ctrler     */
            turn_all_modules("O", xOff);            /*      Outside lights off          */

        /*
        Ŀ
         The part of your schedule that would deal with any expired timers 
         */

        if (timer_expired(0)) {                     /* Time to save status of modules   */
            save_status();                          /*      save status to disk         */
            start_timer(0, 60); }                   /*      set timer for next save     */

        if (timer_expired(1))                       /* Our 10 minute timer expired      */
            dim_module("E02", 16);                  /*      Living table lamp to 16     */

        /*
        Ŀ
         Put any other conditions in this area.  Its wide open             
         */

        if (date_is("04-04") && time_is("02:00"))
            set_daylight_time();

        if (date_is("10-25") && time_is("02:00"))
            set_standard_time();

        if (kbhit()) KeyHit = toupper(getch());

        if (KeyHit == 'T') {
            display_x10_globals();
			printf("    Jr is working: %s\n\n", (JrIsWorking) ? "Yes" : "No");
			KeyHit = '\0';
			printf("    Enter new time (00:00): ");
			gets(NewTime);
			if (strlen(NewTime) == 5) {
				set_time(NewTime);
				printf("    Time has been reset to %s\n\n", NewTime); } }

    /*
    Ŀ
     End of the schedule loop.  If the user pressed the x key then exit    
     */

    } while (KeyHit != 'X');

    /*
    Ŀ
     Make sure you close the communications to the TW523 driver before quit
     */
    close_tw523();
}

/*
ͻ
 Function       startup                                                   
                                                                          
 Purpose        To initialize all of your X-10 modules to a known state.  
                Usually this would be to turn all modules to Off.         
                This is where you would also do any first time            
                initialization of any variables you will be using.        
                                                                          
 Syntax         startup()                                                 
                                                                          
 Parameters     None                                                      
                                                                          
 Example        startup();                                                
                                                                          
 Assumes        Nothing                                                   
                                                                          
 Side Effects   Depends on what you do inside the procedure               
                                                                          
 Returns        Nothing                                                   
                                                                          
͹
 Notes:                                                                    
                                                                           
ͼ */

startup()
{
    /*
    Ŀ
     Define the addresses of your plug-in lamp modules                     
     */
    LampModules = "E02E05";

    /*
    Ŀ
     Initialize the variables for sunrise/sunset calcs.    ie: for Seattle 
     */
    set_map_coord(122.2, 47.37, 8, "04-04", "10-25");

    /*
    Ŀ
     Load the times for the variables Dawn, Sunrise, Sunset and Dusk       
     */
    load_sun_times();

    /*
    Ŀ
     Initialize whether we are currently in Daylight or Standard time      
     */
    set_daylight_flag();

    /*
    Ŀ
     Do you want to display what is being transmitted and received?        
     */
    ShowTransmit = Yes;

    /*
    Ŀ
     Turn all modules off, including appliance modules                     
     */
    turn_all_modules("E", xOff);
    turn_all_modules("F", xOff);
    turn_all_modules("G", xOff);
    turn_all_modules("O", xOff);

    /*
    Ŀ
     Do your custom variable initializations, ie: your days off            
     */
    if (dow_is("Sun") || dow_is("Mon")) {
        JrIsWorking = No;
        CancelBedtime = Yes; }
    else {
        JrIsWorking = Yes;
        CancelBedtime = No; }

    display_x10_globals();
	printf("    Jr is working: %s", (JrIsWorking) ? "Yes\n\n" : "No\n\n");
}

/*
Ŀ
 End of file: X10_EXMP.C                                                   
 */
