/*
ͻ
 Source File    X10_SHEL.C                                                
                                                                          
 System         X-10 Professional Developers Library for C                
                                                                          
 Purpose        Example program shell for Home Automation using X-10      
                modules and a computer serial port interface.             
                                                                          
 Author         Dale Morrison                                             
 Copyright      Smart Home Technology, 1993                               
ͼ


Ŀ
 Load the file containing declarations and constants for the X10_PDLC.LIB  
 */
#include "X10_PDLC.H";

/*
Ŀ
 Define your custom variables that are initialized in the Startup procedure
 */
int DayOff;

/*
Ŀ
 Start of your program                                                     
 */
main()
{
    char KeyHit;

    /*
    Ŀ
     Your basic clear the screen command                                   
     */
    clrscr();

    /*
    Ŀ
     Open communications to the resident TW523 driver                      
     */
    open_tw523(SinglePhase);

    /*
    Ŀ
     Do your Startup procedure to initialize your modules and variables    
     */
    startup();

    /*
    Ŀ
     Start a timer to be used for saving the status of the modules         
     */
    start_timer(0, 60);

    /*
    Ŀ
     Now check to see if there was a power outage since the last boot up   
     */
    if (PowerOutage && load_status())
        play_catchup();
    
    /*
    Ŀ
     Now start the loop to monitor your schedule until a keypress          
     */
    do {

        /*
        Ŀ
         Get the current system time based on the 24 hour clock, ie: 23:00 
         */
        get_time(X10Time);

        /*
        Ŀ
         Get the next received X-10 command from the driver, Null if none  
         */
        get_x10cmd(X10Cmd);

        /*
        Ŀ
         The part of your schedule that deals with time of day events      
         */

        if (time_is("00:01")) {
            load_sun_times();
            get_dow(DayOfWeek);
            get_date(X10Date);
            display_x10_globals(); }

        /*
        Ŀ
         The part of your schedule that deals with the received X-10 cmds  
         */

        if (x10cmd_is("A01 On")) {

           /* ...do something here... */ }

        /*
        Ŀ
         The part of your schedule that would deal with any expired timers 
         */

        if (timer_expired(0)) {                         /* Save module status */
            save_status();
            start_timer(0, 60); }

        /*
        Ŀ
         Put any other conditions in this area.  Its wide open             
         */

        if (date_is("04-04") && time_is("02:00"))
            set_daylight_time();

        if (date_is("10-25") && time_is("02:00"))
            set_standard_time();

    
    /*
    Ŀ
     End of the schedule loop.  If the user pressed the x key then exit    
     */

        if (kbhit()) KeyHit = toupper(getch());

    } while (KeyHit != 'X');
    
    /*
    Ŀ
     Make sure you close the communications to the TW523 driver before quit
     */
    close_tw523();
}

/*
ͻ
 Function       startup                                                   
                                                                          
 Purpose        To initialize all of your X-10 modules to a known state.  
                Usually this would be to turn all modules to Off.         
                This is where you would also do any first time            
                initialization of any variables you will be using.        
                                                                          
 Syntax         startup()                                                 
                                                                          
 Parameters     None                                                      
                                                                          
 Example        startup();                                                
                                                                          
 Assumes        Nothing                                                   
                                                                          
 Side Effects   Depends on what you do inside the procedure               
                                                                          
 Returns        Nothing                                                   
                                                                          
͹
 Notes:                                                                    
                                                                           
ͼ */

startup()
{
    /*
    Ŀ
     Define the addresses of your plug-in lamp modules                     
     */
    LampModules = "A01A02";

    /*
    Ŀ
     Initialize the variables for sunrise/sunset calcs.    ie: for Seattle 
     */
    set_map_coord(122.2, 47.37, 8, "04-04", "10-25");

    /*
    Ŀ
     Load the times for the variables Dawn, Sunrise, Sunset and Dusk       
     */
    load_sun_times();

    /*
    Ŀ
     Initialize the current day of the week and date variables             
     */
    get_dow(DayOfWeek);
    get_date(X10Date);

    /*
    Ŀ
     Initialize whether we are currently in Daylight or Standard time      
     */
    set_daylight_flag();

    /*
    Ŀ
     Do you want to display what is being transmitted and received?        
     */
    ShowTransmit = Yes;

    /*
    Ŀ
     Turn all modules off, including appliance modules                     
     */
    turn_all_modules("D", xOff);
    turn_all_modules("E", xOff);
    turn_all_modules("F", xOff);
    turn_all_modules("G", xOff);
    turn_all_modules("O", xOff);

    /*
    Ŀ
     Do your custom variable initializations, ie: your days off            
     */
    if (dow_is("Mon") || dow_is("Tue"))
        DayOff = True;

}

/*
Ŀ
 End of file: X10_SHEL.C                                                   
 */
